@@ -1,12 +1,13 @@
+{-# OPTIONS_GHC -Wno-orphans #-}
 module Darcs.Test.HashedStorage ( tests, unsafeMakeName ) where
 
 import Prelude hiding ( filter, readFile, writeFile, lookup, (<$>) )
 import qualified Prelude
 import qualified Data.ByteString.Lazy.Char8 as BLC
 import qualified Data.ByteString.Char8 as BC
-import System.Directory( doesFileExist, removeFile, doesDirectoryExist )
-import System.FilePath( (</>) )
-import Control.Monad.Identity
+import System.Directory( doesFileExist, removeFile )
+import Control.Monad ( when, forM_ )
+import Control.Monad.Identity ( Identity, runIdentity )
 import Control.Monad.Trans( lift )
 import Control.Applicative( (<$>) )
 import Codec.Archive.Zip( extractFilesFromArchive, toArchive )
@@ -14,8 +15,12 @@ import Codec.Archive.Zip( extractFilesFromArchive, toArchive )
 import Data.Maybe
 import Data.List( sort, intercalate, intersperse )
 
+import Darcs.Repository.Inventory.Format ( peekPristineHash )
+import Darcs.Repository.Paths ( hashedInventoryPath )
+
+import Darcs.Util.Cache ( mkRepoCache )
 import Darcs.Util.Path hiding ( setCurrentDirectory )
-import Darcs.Util.Lock ( withTempDir )
+import Darcs.Util.Lock ( withPermDir )
 import Darcs.Util.Tree hiding ( lookup )
 import Darcs.Util.Index
 import Darcs.Util.Tree.Hashed
@@ -31,6 +36,7 @@ import Test.HUnit hiding ( path )
 import Test.Framework( testGroup )
 import qualified Test.Framework as TF ( Test )
 import Test.QuickCheck
+import Test.QuickCheck.Instances.ByteString ()
 
 import Test.Framework.Providers.HUnit
 import Test.Framework.Providers.QuickCheck2
@@ -40,24 +46,24 @@ import Test.Framework.Providers.QuickCheck2
 --
 
 blobs :: [(AnchoredPath, BLC.ByteString)]
-blobs = [ (floatPath "foo_a", BLC.pack "a\n")
-        , (floatPath "foo_dir/foo_a", BLC.pack "a\n")
-        , (floatPath "foo_dir/foo_b", BLC.pack "b\n")
-        , (floatPath "foo_dir/foo_subdir/foo_a", BLC.pack "a\n")
-        , (floatPath "foo space/foo\nnewline", BLC.pack "newline\n")
-        , (floatPath "foo space/foo\\backslash", BLC.pack "backslash\n")
-        , (floatPath "foo space/foo_a", BLC.pack "a\n") ]
+blobs = [ (unsafeFloatPath "foo_a", BLC.pack "a\n")
+        , (unsafeFloatPath "foo_dir/foo_a", BLC.pack "a\n")
+        , (unsafeFloatPath "foo_dir/foo_b", BLC.pack "b\n")
+        , (unsafeFloatPath "foo_dir/foo_subdir/foo_a", BLC.pack "a\n")
+        , (unsafeFloatPath "foo space/foo\nnewline", BLC.pack "newline\n")
+        , (unsafeFloatPath "foo space/foo\\backslash", BLC.pack "backslash\n")
+        , (unsafeFloatPath "foo space/foo_a", BLC.pack "a\n") ]
 
 files :: [AnchoredPath]
 files = map fst blobs
 
 dirs :: [AnchoredPath]
-dirs = [ floatPath "foo_dir"
-       , floatPath "foo_dir/foo_subdir"
-       , floatPath "foo space" ]
+dirs = [ unsafeFloatPath "foo_dir"
+       , unsafeFloatPath "foo_dir/foo_subdir"
+       , unsafeFloatPath "foo space" ]
 
 emptyStub :: TreeItem IO
-emptyStub = Stub (return emptyTree) NoHash
+emptyStub = Stub (return emptyTree) Nothing
 
 unsafeMakeName :: String -> Name
 unsafeMakeName = either error id . makeName
@@ -66,14 +72,14 @@ testTree :: Tree IO
 testTree =
     makeTree [ (unsafeMakeName "foo", emptyStub)
              , (unsafeMakeName "subtree", SubTree sub)
-             , (unsafeMakeName "substub", Stub getsub NoHash) ]
+             , (unsafeMakeName "substub", Stub getsub Nothing) ]
     where sub = makeTree [ (unsafeMakeName "stub", emptyStub)
-                         , (unsafeMakeName "substub", Stub getsub2 NoHash)
+                         , (unsafeMakeName "substub", Stub getsub2 Nothing)
                          , (unsafeMakeName "x", SubTree emptyTree) ]
           getsub = return sub
           getsub2 = return $ makeTree [ (unsafeMakeName "file", File emptyBlob)
                                       , (unsafeMakeName "file2",
-                                         File $ Blob (return $ BLC.pack "foo") NoHash) ]
+                                         File $ Blob (return $ BLC.pack "foo") Nothing) ]
 
 equals_testdata :: Tree IO -> IO ()
 equals_testdata t = sequence_ [
@@ -149,7 +155,7 @@ index = [ testCase "index versioning" check_index_versions
                exist <- doesFileExist "_darcs/index"
                performGC -- required in win32 to trigger file close
                when exist $ removeFile "_darcs/index"
-               idx <- treeFromIndex =<< updateIndexFrom "_darcs/index" darcsTreeHash x
+               idx <- treeFromIndex =<< updateIndexFrom "_darcs/index" x
                return (x, idx)
           check_index = extractRepoAndRun $
             do (pris, idx) <- build_index
@@ -196,52 +202,52 @@ tree = [ testCase "modifyTree" check_modify
        , testProperty "restrict is a subtree of both" prop_restrict_subtree
        , testProperty "overlay keeps shape" prop_overlay_shape
        , testProperty "overlay is superset of over" prop_overlay_super ]
-    where blob x = File $ Blob (return (BLC.pack x)) (sha256 $ BLC.pack x)
+    where blob x = File $ Blob (return (BLC.pack x)) (Just $ sha256 $ BLC.pack x)
           name = unsafeMakeName
           check_modify =
               let t = makeTree [(name "foo", blob "bar")]
-                  modify = modifyTree t (floatPath "foo") (Just $ blob "bla")
-               in do x <- readBlob $ fromJust $ findFile t (floatPath "foo")
-                     y <- readBlob $ fromJust $ findFile modify (floatPath "foo")
+                  modify = modifyTree t (unsafeFloatPath "foo") (Just $ blob "bla")
+               in do x <- readBlob $ fromJust $ findFile t (unsafeFloatPath "foo")
+                     y <- readBlob $ fromJust $ findFile modify (unsafeFloatPath "foo")
                      assertEqual "old version" x (BLC.pack "bar")
                      assertEqual "new version" y (BLC.pack "bla")
                      assertBool "list has foo" $
-                                isJust (Prelude.lookup (floatPath "foo") $ list modify)
+                                isJust (Prelude.lookup (unsafeFloatPath "foo") $ list modify)
                      length (list modify) @?= 1
           check_modify_complex =
               let t = makeTree [ (name "foo", blob "bar")
                                , (name "bar", SubTree t1) ]
                   t1 = makeTree [ (name "foo", blob "bar") ]
-                  modify = modifyTree t (floatPath "bar/foo") (Just $ blob "bla")
-               in do foo <- readBlob $ fromJust $ findFile t (floatPath "foo")
-                     foo' <- readBlob $ fromJust $ findFile modify (floatPath "foo")
+                  modify = modifyTree t (unsafeFloatPath "bar/foo") (Just $ blob "bla")
+               in do foo <- readBlob $ fromJust $ findFile t (unsafeFloatPath "foo")
+                     foo' <- readBlob $ fromJust $ findFile modify (unsafeFloatPath "foo")
                      bar_foo <- readBlob $ fromJust $
-                                findFile t (floatPath "bar/foo")
+                                findFile t (unsafeFloatPath "bar/foo")
                      bar_foo' <- readBlob $ fromJust $
-                                 findFile modify (floatPath "bar/foo")
+                                 findFile modify (unsafeFloatPath "bar/foo")
                      assertEqual "old foo" foo (BLC.pack "bar")
                      assertEqual "old bar/foo" bar_foo (BLC.pack "bar")
                      assertEqual "new foo" foo' (BLC.pack "bar")
                      assertEqual "new bar/foo" bar_foo' (BLC.pack "bla")
                      assertBool "list has bar/foo" $
-                                isJust (Prelude.lookup (floatPath "bar/foo") $ list modify)
+                                isJust (Prelude.lookup (unsafeFloatPath "bar/foo") $ list modify)
                      assertBool "list has foo" $
-                                isJust (Prelude.lookup (floatPath "foo") $ list modify)
+                                isJust (Prelude.lookup (unsafeFloatPath "foo") $ list modify)
                      length (list modify) @?= length (list t)
           check_modify_remove =
               let t1 = makeTree [(name "foo", blob "bar")]
                   t2 :: Tree Identity = makeTree [ (name "foo", blob "bar")
                                                  , (name "bar", SubTree t1) ]
-                  modify1 = modifyTree t1 (floatPath "foo") Nothing
-                  modify2 = modifyTree t2 (floatPath "bar") Nothing
-                  file = findFile modify1 (floatPath "foo")
-                  subtree = findTree modify2 (floatPath "bar")
+                  modify1 = modifyTree t1 (unsafeFloatPath "foo") Nothing
+                  modify2 = modifyTree t2 (unsafeFloatPath "bar") Nothing
+                  file = findFile modify1 (unsafeFloatPath "foo")
+                  subtree = findTree modify2 (unsafeFloatPath "bar")
                in do assertBool "file is gone" (isNothing file)
                      assertBool "subtree is gone" (isNothing subtree)
 
           no_stubs t = null [ () | (_, Stub _ _) <- list t ]
-          path = floatPath "substub/substub/file"
-          badpath = floatPath "substub/substub/foo"
+          path = unsafeFloatPath "substub/substub/file"
+          badpath = unsafeFloatPath "substub/substub/foo"
           check_expand = do
             x <- expand testTree
             assertBool "no stubs in testTree" $ not (no_stubs testTree)
@@ -253,7 +259,7 @@ tree = [ testCase "modifyTree" check_modify
             test_exp <- expand testTree
             t <- expandPath testTree path
             t' <- expandPath test_exp path
-            t'' <- expandPath testTree $ floatPath "substub/x"
+            t'' <- expandPath testTree $ unsafeFloatPath "substub/x"
             assertBool "path not reachable in testTree" $ path `notElem` (map fst $ list testTree)
             assertBool "path reachable in t" $ path `elem` (map fst $ list t)
             assertBool "path reachable in t'" $ path `elem` (map fst $ list t')
@@ -268,24 +274,24 @@ tree = [ testCase "modifyTree" check_modify
                        badpath `notElem` (map fst $ list t')
 
           check_expand_path_sub = do
-            t <- expandPath testTree $ floatPath "substub"
-            t' <- expandPath testTree $ floatPath "substub/stub"
-            t'' <- expandPath testTree $ floatPath "subtree/stub"
+            t <- expandPath testTree $ unsafeFloatPath "substub"
+            t' <- expandPath testTree $ unsafeFloatPath "substub/stub"
+            t'' <- expandPath testTree $ unsafeFloatPath "subtree/stub"
             assertBool "leaf is not a Stub" $
-                isNothing (findTree testTree $ floatPath "substub")
-            assertBool "leaf is not a Stub" $ isJust (findTree t $ floatPath "substub")
-            assertBool "leaf is not a Stub (2)" $ isJust (findTree t' $ floatPath "substub/stub")
-            assertBool "leaf is not a Stub (3)" $ isJust (findTree t'' $ floatPath "subtree/stub")
+                isNothing (findTree testTree $ unsafeFloatPath "substub")
+            assertBool "leaf is not a Stub" $ isJust (findTree t $ unsafeFloatPath "substub")
+            assertBool "leaf is not a Stub (2)" $ isJust (findTree t' $ unsafeFloatPath "substub/stub")
+            assertBool "leaf is not a Stub (3)" $ isJust (findTree t'' $ unsafeFloatPath "subtree/stub")
 
           check_diffTrees = extractRepoAndRun $
                  do Prelude.writeFile "foo_dir/foo_a" "b\n"
                     working_plain <- filter nondarcs `fmap` readPlainTree "."
                     working <- treeFromIndex =<<
-                                 updateIndexFrom "_darcs/index" darcsTreeHash working_plain
+                                 updateIndexFrom "_darcs/index" working_plain
                     pristine <- readDarcsPristine "."
                     (working', pristine') <- diffTrees working pristine
-                    let foo_work = findFile working' (floatPath "foo_dir/foo_a")
-                        foo_pris = findFile pristine' (floatPath "foo_dir/foo_a")
+                    let foo_work = findFile working' (unsafeFloatPath "foo_dir/foo_a")
+                        foo_pris = findFile pristine' (unsafeFloatPath "foo_dir/foo_a")
                     working' `shapeEq` pristine'
                              @? show working' ++ " `shapeEq` " ++ show pristine'
                     assertBool "foo_dir/foo_a is in working'" $ isJust foo_work
@@ -337,34 +343,29 @@ tree = [ testCase "modifyTree" check_modify
                   notStub _ = True
 
 hash :: [TF.Test]
-hash = [ testProperty "decodeBase16 . encodeBase16 == id" prop_base16 ]
-    where prop_base16 x = (decodeBase16 . encodeBase16) x == x
+hash = [ testProperty "decodeBase16 . encodeBase16 == Just" prop_base16 ]
+    where prop_base16 x = (decodeBase16 . encodeBase16) x == Just x
 
 monad :: [TF.Test]
 monad = [ testCase "path expansion" check_virtual
         , testCase "rename" check_rename ]
     where check_virtual = virtualTreeMonad run testTree >> return ()
-              where run = do file <- readFile (floatPath "substub/substub/file")
-                             file2 <- readFile (floatPath "substub/substub/file2")
+              where run = do file <- readFile (unsafeFloatPath "substub/substub/file")
+                             file2 <- readFile (unsafeFloatPath "substub/substub/file2")
                              lift $ BLC.unpack file @?= ""
                              lift $ BLC.unpack file2 @?= "foo"
           check_rename = do (_, t) <- virtualTreeMonad run testTree
                             t' <- darcsAddMissingHashes =<< expand t
                             forM_ [ (p, i) | (p, i) <- list t' ] $ \(p,i) ->
-                               assertBool ("have hash: " ++ show p) $ itemHash i /= NoHash
-              where run = do rename (floatPath "substub/substub/file") (floatPath "substub/file2")
+                               assertBool ("have hash: " ++ show p) $ itemHash i /= Nothing
+              where run = do rename (unsafeFloatPath "substub/substub/file") (unsafeFloatPath "substub/file2")
 
 ----------------------------------
 -- Arbitrary instances
 --
 
-instance Arbitrary BLC.ByteString where
-    arbitrary = BLC.pack `fmap` arbitrary
-
 instance Arbitrary Hash where
-    arbitrary = sized hash'
-        where hash' 0 = return NoHash
-              hash' _ = SHA256 . BC.pack <$> sequence [ arbitrary | _ <- [1..32] :: [Int] ]
+    arbitrary = mkHash . BC.pack <$> sequence (replicate 32 arbitrary)
 
 instance (Monad m) => Arbitrary (TreeItem m) where
   arbitrary = sized tree'
@@ -372,14 +373,14 @@ instance (Monad m) => Arbitrary (TreeItem m) where
           tree' n = oneof [ file n, subtree n ]
           file 0 = return (File emptyBlob)
           file _ = do content <- arbitrary
-                      return (File $ Blob (return content) NoHash)
+                      return (File $ Blob (return content) Nothing)
           subtree n = do branches <- choose (1, n)
                          let sub name = do t <- tree' ((n - 1) `div` branches)
                                            return (unsafeMakeName $ show name, t)
                          sublist <- mapM sub [0..branches]
                          oneof [ tree' 0
                                , return (SubTree $ makeTree sublist)
-                               , return $ (Stub $ return (makeTree sublist)) NoHash ]
+                               , return $ (Stub $ return (makeTree sublist)) Nothing ]
 
 instance (Monad m) => Arbitrary (Tree m) where
   arbitrary = do item <- arbitrary
@@ -468,31 +469,12 @@ nondarcs (AnchoredPath []) _ = True
 
 readDarcsPristine :: FilePath -> IO (Tree IO)
 readDarcsPristine dir = do
-  let darcs = dir </> "_darcs"
-      h_inventory = darcs </> "hashed_inventory"
-  repo <- doesDirectoryExist darcs
-  unless repo $ fail $ "Not a darcs repository: " ++ dir
-  isHashed <- doesFileExist h_inventory
-  if isHashed
-     then do inv <- BC.readFile h_inventory
-             let thelines = BC.split '\n' inv
-             case thelines of
-               [] -> return emptyTree
-               (pris_line:_) -> do
-                          let thehash = decodeDarcsHash $ BC.drop 9 pris_line
-                              thesize = decodeDarcsSize $ BC.drop 9 pris_line
-                          when (thehash == NoHash) $ fail $ "Bad pristine root: " ++ show pris_line
-                          readDarcsHashed (darcs </> "pristine.hashed") (thesize, thehash)
-     else do have_pristine <- doesDirectoryExist $ darcs </> "pristine"
-             have_current <- doesDirectoryExist $ darcs </> "current"
-             case (have_pristine, have_current) of
-               (True, _) -> readPlainTree $ darcs </> "pristine"
-               (False, True) -> readPlainTree $ darcs </> "current"
-               (_, _) -> fail "No pristine tree is available!"
+  ph <- peekPristineHash <$> BC.readFile hashedInventoryPath
+  readDarcsHashed (mkRepoCache dir) ph
 
 extractRepoAndRun :: IO a -> IO a
 extractRepoAndRun action = do
   zipFile <- toArchive . BLC.fromStrict <$> BC.readFile "harness/hstestdata.zip"
-  withTempDir "_test_playground" $ \_ -> do
+  withPermDir "_test_playground" $ \_ -> do
     extractFilesFromArchive [] zipFile
     action
