@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -Wno-orphans #-}
 {-# LANGUAGE OverloadedStrings, UndecidableInstances #-}
 module Darcs.Test.Patch.Arbitrary.NamedPrim ( aPatchId ) where
 
@@ -15,6 +16,7 @@ import Darcs.Test.Patch.Arbitrary.Shrink
 
 import Darcs.Test.Patch.WithState
 import Darcs.Test.Patch.RepoModel
+import Darcs.Test.Patch.Types.Pair ( Pair(..) )
 import Darcs.Test.TestOnly.Instance ()
 
 import Darcs.Patch.Witnesses.Maybe
@@ -51,6 +53,14 @@ instance ArbitraryState prim => ArbitraryState (NamedPrim prim) where
     pid <- aPatchId
     return $ Sealed $ WithEndState (namedPrim pid p) repo'
 
+  arbitraryStatePair repo = do
+    Sealed (WithEndState (Pair (p1:>p2)) repo') <- arbitraryStatePair repo
+    pid1 <- aPatchId
+    pid2 <- aPatchId
+    return $ Sealed $ WithEndState (Pair (namedPrim pid1 p1 :> namedPrim pid2 p2)) repo'
+
+instance (ArbitraryState prim, RepoModel (ModelOf prim)) => ArbitraryWS (NamedPrim prim) where
+  arbitraryWS = makeWS2Gen aSmallRepo
 
 instance PropagateShrink prim1 prim2 => PropagateShrink prim1 (PrimWithName n2 prim2) where
   propagateShrink (p1 :> PrimWithName n2 p2) = do
