@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -Wno-orphans #-}
 {-# LANGUAGE OverloadedStrings #-}
 module Darcs.Test.Patch.Arbitrary.PrimFileUUID where
 
@@ -10,6 +11,7 @@ import Darcs.Test.Patch.RepoModel
 
 import Test.QuickCheck
 import Darcs.Test.Patch.WithState
+import Darcs.Test.Patch.Types.Pair ( Pair(..) )
 import Darcs.Patch.Witnesses.Sealed
 import Darcs.Patch.Witnesses.Eq
 import Darcs.Patch.Witnesses.Unsafe
@@ -23,7 +25,6 @@ import Darcs.Test.Util.QuickCheck ( notIn, maybeOf )
 import qualified Data.ByteString as B
 import Data.Maybe ( fromJust, isJust )
 import qualified Data.Map as M
-import Darcs.Util.Hash( Hash(..) )
 
 type instance ModelOf Prim = FileUUIDModel
 instance ArbitraryPrim Prim where
@@ -81,7 +82,7 @@ aDemanifest :: UUID -> Location -> Gen (Prim wX wY)
 aDemanifest uuid loc = return $ Demanifest uuid loc
 
 -- | Generates any type of 'Prim' patch, except binary and setpref patches.
-aPrim :: FileUUIDModel wX -> Gen (WithEndState FileUUIDModel (Prim wX) wY)
+aPrim :: FileUUIDModel wX -> Gen (Sealed (WithEndState FileUUIDModel (Prim wX)))
 aPrim repo
   = do mbFile <- maybeOf repoFiles -- some file, not necessarily manifested
        dir <- elements repoDirs -- some directory, not necessarily manifested
@@ -95,7 +96,7 @@ aPrim repo
            whenmanifested = whenjust mbManifested
        patch <- frequency
                   [ ( whenfile 12, aTextHunk $ fromJust mbFile ) -- edit an existing file
-                  , ( 2, aTextHunk (fresh, Blob (return "") NoHash) ) -- edit a new file
+                  , ( 2, aTextHunk (fresh, Blob (return "") Nothing) ) -- edit a new file
                   , ( whendemanifested 2 -- manifest an existing object
                     , aManifest (fromJust mbDemanifested) dir
                     )
@@ -104,7 +105,7 @@ aPrim repo
                     )
                   ]
        let repo' = unFail $ repoApply repo patch
-       return $ WithEndState patch repo'
+       return $ seal $ WithEndState patch repo'
   where
       manifested = [ (uuid, (L dirid name)) | (dirid, Directory dir) <- repoDirs
                                           , (name, uuid) <- M.toList dir ]
@@ -133,7 +134,7 @@ hunkPair (uuid, (Blob file _)) =
 hunkPair _ = error "impossible case"
 
 aPrimPair :: FileUUIDModel wX
-          -> Gen (WithEndState FileUUIDModel ((Prim :> Prim) wX) wY)
+          -> Gen (Sealed (WithEndState FileUUIDModel (Pair Prim wX)))
 aPrimPair repo
   = do mbFile <- maybeOf repoFiles
        frequency
@@ -141,11 +142,15 @@ aPrimPair repo
             , do p1 :> p2 <- hunkPair $ fromJust mbFile
                  let repo'  = unFail $ repoApply repo  p1
                      repo'' = unFail $ repoApply repo' p2
-                 return $ WithEndState (p1 :> p2) repo''
+                 return $ seal $ WithEndState (Pair (p1 :> p2)) repo''
             )
           , ( 1
-            , do Sealed wesP <- arbitraryState repo
-                 return $ unsafeCoerceP1 wesP
+            , do
+                -- construct the underlying pair directly to avoid any
+                -- risk of indirectly calling arbitraryStatePair (which
+                -- would cause a loop).
+                Sealed (WithEndState pair repo') <- arbitraryState repo
+                return $ seal $ WithEndState (Pair pair) repo'
             )
           ]
   where
@@ -155,24 +160,8 @@ aPrimPair repo
 -- Arbitrary instances
 
 instance ArbitraryState Prim where
-  arbitraryState s = seal <$> aPrim s
+  arbitraryState = aPrim
+  arbitraryStatePair = aPrimPair
 
-
--- use the special generator for pairs
-arbitraryPair :: Gen (Sealed2 (WithState (Prim :> Prim)))
-arbitraryPair = do
-  repo <- aSmallRepo
-  WithEndState pp repo' <- aPrimPair repo
-  return $ seal2 $ WithState repo pp repo'
-
-instance Arbitrary (Sealed2 Prim) where
-  arbitrary = makeS2Gen aSmallRepo
-
-instance Arbitrary (Sealed2 (Prim :> Prim)) where
-  arbitrary = mapSeal2 wsPatch <$> arbitraryPair
-
-instance Arbitrary (Sealed2 (WithState Prim)) where
-  arbitrary = makeWS2Gen aSmallRepo
-
-instance Arbitrary (Sealed2 (WithState (Prim :> Prim))) where
-  arbitrary = arbitraryPair
+instance ArbitraryWS Prim where
+  arbitraryWS = makeWS2Gen aSmallRepo
