@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -Wno-orphans #-}
 module Darcs.Test.Patch.Arbitrary.PrimV1
     ( aPrim
     , aPrimPair
@@ -19,18 +20,19 @@ import Darcs.Test.Patch.Arbitrary.Shrink
 import Control.Applicative ( (<|>) )
 import Test.QuickCheck
 import Darcs.Test.Patch.WithState
+import Darcs.Test.Patch.Types.Pair ( Pair(..) )
 import Darcs.Patch.Witnesses.Sealed
 import Darcs.Patch.Witnesses.Eq
 import Darcs.Patch.Witnesses.Unsafe
 import Darcs.Patch.Witnesses.Ordered
-import Darcs.Patch.Prim.V1.Core ( FilePatchType( Hunk ), isIdentity )
+import Darcs.Patch.Prim ( isIdentity )
+import Darcs.Patch.Prim.V1.Core ( FilePatchType( Hunk ) )
 import qualified Darcs.Patch.Prim.V1.Core as Prim ( Prim( FP ) )
 import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
 import qualified Darcs.Patch.V2.Prim as V2 ( Prim(..) )
 
 import Darcs.Test.Patch.V1Model
 import Darcs.Util.Path
-import Darcs.Util.Tree ( Tree )
 import Darcs.Test.Util.QuickCheck ( alpha, notIn, maybeOf )
 
 import Darcs.UI.Commands.Replace ( defaultToks )
@@ -203,7 +205,7 @@ shrinkName n = do
   n' <- shrink (BC.unpack . encodeWhiteName $ n)
   guard (n' /= ".")
   guard (not $ null n')
-  return $ decodeWhiteName $ BC.pack n'
+  either (const []) (:[]) $ decodeWhiteName $ BC.pack n'
 
 aModelShrinkName :: V1Model wX -> [Sealed (Prim.Prim wX)]
 aModelShrinkName repo = do
@@ -234,8 +236,8 @@ aModelShrinkFileContent repo = do
 
 
 -- | Generates any type of 'prim' patch, except binary and setpref patches.
-aPrim :: forall prim wX wY . (PrimPatch prim, ApplyState prim ~ RepoState V1Model)
-      => V1Model wX -> Gen (WithEndState V1Model (prim wX) wY)
+aPrim :: forall prim wX . (PrimPatch prim, ApplyState prim ~ RepoState V1Model)
+      => V1Model wX -> Gen (Sealed (WithEndState V1Model (prim wX)))
 aPrim repo
   = do mbFile <- maybeOf repoFiles
        mbEmptyFile <- maybeOf $ filter (isEmpty . snd) repoFiles
@@ -272,31 +274,13 @@ aPrim repo
                     )
                   ]
        let repo' = unFail $ repoApply repo patch
-       return $ WithEndState patch repo'
+       return $ seal $ WithEndState patch repo'
   where
       repoItems = list repo
       repoFiles = filterFiles repoItems
       repoDirs  = filterDirs repoItems
       rootDir   = (anchoredRoot,root repo)
 
-{- [COVERAGE OF aPrim]
-
-  PLEASE,
-  if you change something that may affect the coverage of aPrim then
-      a) recalculate it, or if that is not possible;
-      b) indicate the need to do it.
-
-  Patch type
-  ----------
-  42% hunk
-  22% tokreplace
-  14% move
-   6% rmdir
-   6% addfile
-   6% adddir
-   4% rmfile
--}
-
 ----------------------------------------------------------------------
 -- *** Pairs of primitive patches
 
@@ -324,7 +308,7 @@ aPrimPair :: ( PrimPatch prim
              , ModelOf prim ~ V1Model
              )
           => V1Model wX
-          -> Gen (WithEndState V1Model ((prim :> prim) wX) wY)
+          -> Gen (Sealed (WithEndState V1Model (Pair prim wX)))
 aPrimPair repo
   = do mbFile <- maybeOf repoFiles
        frequency
@@ -332,45 +316,21 @@ aPrimPair repo
             , do p1 :> p2 <- hunkPairP $ fromJust mbFile
                  let repo'  = unFail $ repoApply repo p1
                      repo'' = unFail $ repoApply repo' p2
-                 return $ WithEndState (p1 :> p2) repo''
+                 return $ seal $ WithEndState (Pair (p1 :> p2)) repo''
             )
           , ( 1
-            , do Sealed wesP <- arbitraryState repo
-                 return $ unsafeCoerceP1 wesP
+            , do
+                -- construct the underlying pair directly to avoid any
+                -- risk of indirectly calling arbitraryStatePair (which
+                -- would cause a loop).
+                Sealed (WithEndState pair repo') <- arbitraryState repo
+                return $ seal $ WithEndState (Pair pair) repo'
             )
           ]
   where
       repoItems = list repo
       repoFiles = filterFiles repoItems
 
-{- [COVERAGE OF aPrimPair]
-
-  PLEASE,
-  if you change something that may affect the coverage of aPrimPair then
-      a) recalculate it, or if that is not possible;
-      b) indicate the need to do it.
-
-  Rate of ommutable pairs
-  -----------------------
-  67% commutable
-
-  Commutable coverage (for 1000 tests)
-  -------------------
-  21% hunks-B
-  20% hunks-A
-  14% file:>dir
-  12% file:>move
-   8% trivial-FP
-   8% hunk:>tok
-   4% hunks-D
-   3% tok:>tok
-   2% hunks-C
-   1% move:>move
-   1% dir:>move
-   1% dir:>dir
-   0% emptyhunk:>file
--}
-
 ----------------------------------------------------------------------
 -- Arbitrary instances
 
@@ -379,22 +339,11 @@ type instance ModelOf Prim.Prim = V1Model
 instance ShrinkModel Prim.Prim where
   shrinkModelPatch s = aModelShrink s
 
--- use the special generator for pairs
-arbitraryPair :: ( PrimPatch prim
-                 , ApplyState prim ~ Tree
-                 , ArbitraryState prim
-                 , ModelOf prim ~ V1Model
-                 )
-              => Gen (Sealed2 (WithState (prim :> prim)))
-arbitraryPair = do
-  repo <- aSmallRepo
-  WithEndState pp repo' <- aPrimPair repo
-  return $ seal2 $ WithState repo pp repo'
-
 -- Prim1
 
 instance ArbitraryState Prim1 where
-  arbitraryState s = seal <$> aPrim s
+  arbitraryState = aPrim
+  arbitraryStatePair = aPrimPair
 
 instance ShrinkModel Prim1 where
   shrinkModelPatch s = map (mapSeal V1.Prim) $ shrinkModelPatch s
@@ -402,23 +351,14 @@ instance ShrinkModel Prim1 where
 instance PropagateShrink Prim1 Prim1 where
   propagateShrink = propagatePrim
 
-
-instance Arbitrary (Sealed2 Prim1) where
-  arbitrary = makeS2Gen aSmallRepo
-
-instance Arbitrary (Sealed2 (Prim1 :> Prim1)) where
-  arbitrary = mapSeal2 wsPatch <$> arbitraryPair
-
-instance Arbitrary (Sealed2 (WithState Prim1)) where
-  arbitrary = makeWS2Gen aSmallRepo
-
-instance Arbitrary (Sealed2 (WithState (Prim1 :> Prim1))) where
-  arbitrary = arbitraryPair
+instance ArbitraryWS Prim1 where
+  arbitraryWS = makeWS2Gen aSmallRepo
 
 -- Prim2
 
 instance ArbitraryState Prim2 where
-  arbitraryState s = seal <$> aPrim s
+  arbitraryState = aPrim
+  arbitraryStatePair = aPrimPair
 
 instance ShrinkModel Prim2 where
   shrinkModelPatch s = map (mapSeal V2.Prim) $ shrinkModelPatch s
@@ -426,15 +366,5 @@ instance ShrinkModel Prim2 where
 instance PropagateShrink Prim2 Prim2 where
   propagateShrink = propagatePrim
 
-
-instance Arbitrary (Sealed2 Prim2) where
-  arbitrary = makeS2Gen aSmallRepo
-
-instance Arbitrary (Sealed2 (Prim2 :> Prim2)) where
-  arbitrary = mapSeal2 wsPatch <$> arbitraryPair
-
-instance Arbitrary (Sealed2 (WithState Prim2)) where
-  arbitrary = makeWS2Gen aSmallRepo
-
-instance Arbitrary (Sealed2 (WithState (Prim2 :> Prim2))) where
-  arbitrary = arbitraryPair
+instance ArbitraryWS Prim2 where
+  arbitraryWS = makeWS2Gen aSmallRepo
