@@ -15,6 +15,7 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
+{-# OPTIONS_GHC -Wno-orphans #-}
 {-# LANGUAGE OverloadedStrings #-}
 module Darcs.Test.Patch.Examples.Set1
        ( knownCommutes, knownCantCommutes, knownMerges
@@ -40,13 +41,14 @@ import Darcs.Patch.FromPrim ( PrimOf, FromPrim(..) )
 import qualified Darcs.Patch.V1 as V1 ( RepoPatchV1 )
 import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
 import Darcs.Test.Patch.Properties.Check( checkAPatch )
+import Darcs.Test.Patch.Types.Pair ( Pair(..) )
 import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Sealed ( unseal )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePEnd )
-import Darcs.Util.Path ( AnchoredPath, floatPath )
+import Darcs.Util.Path ( AnchoredPath, unsafeFloatPath )
 
 instance IsString AnchoredPath where
-  fromString = floatPath
+  fromString = unsafeFloatPath
 
 type Patch = V1.RepoPatchV1 V1.Prim
 
@@ -123,20 +125,20 @@ mergePairs =
 -- the result gives us patch A and patch B again.  The set of patches (A,B)
 -- is chosen from the set of all pairs of test patches by selecting those which
 -- commute with one another.
-commutePairs :: [(FL Patch :> FL Patch) wX wY]
+commutePairs :: [Pair (FL Patch) wX wY]
 commutePairs =
-  take 200 [(p1:>p2)|
+  take 200 [Pair (p1:>p2)|
             p1<-testPatches,
             p2<-filter (\p->checkAPatch (p1:>:p:>:NilFL)) testPatches,
             commute (p1:>p2) /= Nothing]
 
-primitiveCommutePairs :: [(FL Patch :> FL Patch) wX wY]
+primitiveCommutePairs :: [Pair (FL Patch) wX wY]
 primitiveCommutePairs =
-  [(p2:>p1)|
+  [Pair (p2:>p1)|
    p1<-primitiveTestPatches,
    p2<-primitiveTestPatches,
-   commute (p2:>p1) /= Nothing,
-   checkAPatch (p2:>:p1:>:NilFL)]
+   checkAPatch (p2:>:p1:>:NilFL),
+   commute (p2:>p1) /= Nothing]
 
 -- ----------------------------------------------------------------------
 -- * Commute tests
