@@ -16,6 +16,9 @@
 -- Boston, MA 02110-1301, USA.
 
 {-# LANGUAGE OverloadedStrings #-}
+{-# OPTIONS_GHC -Wno-orphans #-}
+{-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
+
 module Darcs.Test.Patch.Examples.Set2Unwitnessed
        ( primPermutables, primPatches
        , commutables, commutablesFL
@@ -54,7 +57,7 @@ import qualified Data.ByteString as B ( ByteString )
 import Darcs.Test.Patch.V1Model ( V1Model, Content
                                 , makeRepo, makeFile)
 import Darcs.Test.Patch.WithState ( WithStartState(..) )
-import Darcs.Util.Path ( AnchoredPath, floatPath, makeName )
+import Darcs.Util.Path ( AnchoredPath, unsafeFloatPath, makeName )
 import Darcs.Patch.FromPrim ( PrimPatchBase(..), FromPrim )
 import Darcs.Patch.Merge ( Merge )
 import Darcs.Test.Patch.Arbitrary.PatchTree
@@ -66,7 +69,7 @@ import Darcs.Test.Patch.Arbitrary.PatchTree
     )
 
 instance IsString AnchoredPath where
-  fromString = floatPath
+  fromString = unsafeFloatPath
 
 
 -- import Debug.Trace
