@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -Wno-orphans #-}
 {-# LANGUAGE OverloadedStrings #-}
 
 -- | Repository model
@@ -7,7 +8,6 @@ module Darcs.Test.Patch.FileUUIDModel
   , repoApply
   , emptyFile
   , emptyDir
-  , nullRepo
   , root, rootId
   , repoObjects, repoIds
   , aFilename, aDirname
@@ -31,7 +31,6 @@ import Darcs.Patch.Witnesses.Sealed ( Sealed, seal )
 import Darcs.Patch.Witnesses.Show
 
 import Darcs.Util.Path ( Name, makeName )
-import Darcs.Util.Hash( Hash(..) )
 
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
@@ -40,7 +39,6 @@ import Data.Maybe ( fromJust )
 import Test.QuickCheck
   ( Arbitrary(..)
   , Gen, choose, vectorOf, frequency, oneof )
--- import Text.Show.Pretty ( ppShow )
 
 ----------------------------------------------------------------------
 -- * Model definition
@@ -54,7 +52,11 @@ instance Show (Object Fail) where
   show (Directory l) = show l
   show (Blob c _) = show c
 
-deriving instance Eq (Object Fail)
+instance Eq (Object Fail) where
+  Blob _ (Just h1) == Blob _ (Just h2) = h1 == h2
+  Blob (Right c1) _ == Blob (Right c2) _ = c1 == c2
+  Directory m1 == Directory m2 = m1 == m2
+  _ == _ = False
 
 instance Show (FileUUIDModel wX) where
   show repo = "FileUUIDModel " ++ show (repoObjects repo)
@@ -70,13 +72,8 @@ objectMap m = ObjectMap { getObject = get, putObject = put, listObjects = list }
         put k o = return $ objectMap (M.insert k o m)
         get k = return $ M.lookup k m
 
-{-
-emptyRepo :: FileUUIDModel wX
-emptyRepo = FileUUIDModel (objectMap $ M.singleton rootId emptyDir)
--}
-
 emptyFile :: (Monad m) => Object m
-emptyFile = Blob (return B.empty) NoHash
+emptyFile = Blob (return B.empty) Nothing
 
 emptyDir :: Object m
 emptyDir = Directory M.empty
@@ -84,9 +81,6 @@ emptyDir = Directory M.empty
 ----------------------------------------------------------------------
 -- * Queries
 
-nullRepo :: FileUUIDModel wX -> Bool
-nullRepo repo = repoIds repo == [rootId]
-
 rootId :: UUID
 rootId = UUID "ROOT"
 
@@ -112,9 +106,6 @@ isEmpty (Blob f _) = B.null $ unFail f
 nonEmptyRepoObjects :: FileUUIDModel wX -> [(UUID, Object Fail)]
 nonEmptyRepoObjects = filter (not . isEmpty . snd) . repoObjects
 
-----------------------------------------------------------------------
--- * Comparing repositories
-
 ----------------------------------------------------------------------
 -- * QuickCheck generators
 
@@ -154,7 +145,7 @@ aContent = bSized 0 0.5 80 $ \k ->
                 BC.intercalate "\n" <$> vectorOf n aLine
 
 aFile :: (Monad m) => Gen (Object m)
-aFile = aContent >>= \c -> return $ Blob (return c) NoHash
+aFile = aContent >>= \c -> return $ Blob (return c) Nothing
 
 aDir :: (Monad m) => [UUID] -> [UUID] -> Gen [(UUID, Object m)]
 aDir [] _ = return []
@@ -171,11 +162,11 @@ aDir (dirid:dirids) fileids =
      return $ (dirid, Directory $ M.fromList $ names `zip` ids ++ dirnames `zip` dirids)
             : (fileids `zip` files) ++ dirs
   where subdirs [] _ _ = return []
-        subdirs tomake dirs files = do
+        subdirs (uuid:uuids) dirs files = do
           dirsplit <- choose (1, length dirs)
           filesplit <- choose (1, length files)
-          dir <- aDir (head tomake : take dirsplit dirs) (take filesplit files)
-          remaining <- subdirs (tail tomake) (drop dirsplit dirs) (drop filesplit files)
+          dir <- aDir (uuid : take dirsplit dirs) (take filesplit files)
+          remaining <- subdirs uuids (drop dirsplit dirs) (drop filesplit files)
           return $ dir ++ remaining
 
 
@@ -207,7 +198,7 @@ instance RepoModel FileUUIDModel where
                   dirsNo <- frequency [(3, return 1), (1, return 0)]
                   aRepo filesNo dirsNo
   repoApply (FileUUIDModel state) patch = FileUUIDModel <$> applyToState patch state
-  showModel = show -- ppShow
+  showModel = show
   eqModel r1 r2 = nonEmptyRepoObjects r1 == nonEmptyRepoObjects r2
 
 instance Arbitrary (Sealed FileUUIDModel) where
