@@ -36,6 +36,7 @@ module Darcs.Test.Patch.Properties.Generic
     , PatchProperty
     , MergeProperty
     , SequenceProperty
+    , propPrimPairCoverage
     ) where
 
 import Darcs.Prelude
@@ -62,6 +63,7 @@ import Darcs.Test.Patch.Arbitrary.Generic
     , MightHaveDuplicate(..)
     , TestablePrim
     )
+import Darcs.Test.Patch.Types.Pair ( Pair(..) )
 import Darcs.Test.Patch.Properties.Check ( checkAPatch, Check )
 
 import Control.Monad ( msum )
@@ -84,13 +86,14 @@ import Darcs.Patch.Witnesses.Ordered
     , (:\/:)(..)
     , FL(..)
     , RL(..)
-    , eqFL
     , mapFL
     )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.Util.Printer ( Doc, renderPS, redText, greenText, ($$), text, vcat )
 --import Darcs.ColorPrinter ( traceDoc )
 
+import Test.QuickCheck (Property, checkCoverage, cover)
+
 type PatchProperty p = forall wA wB. p wA wB -> TestResult
 -- type PairProperty p = forall wA wB. (p :> p) wA wB -> TestResult
 type MergeProperty p = forall wA wB. (FL p :\/: FL p) wA wB -> TestResult
@@ -112,12 +115,12 @@ displayPatchFL = vcat . mapFL displayPatch
 
 -- | @(AB)^ = B^A^@
 inverseComposition :: (Invert p, Eq2 p, ShowPatchBasic p)
-                   => (p :> p) wX wY -> TestResult
-inverseComposition (a :> b) =
+                   => Pair p wX wY -> TestResult
+inverseComposition (Pair (a :> b)) =
   let ab = a:>:b:>:NilFL
       iab = invert ab
       ibia = invert b:>:invert a:>:NilFL
-  in case eqFL iab ibia of
+  in case iab =\/= ibia of
     IsEq -> succeeded
     NotEq ->
       failed $ redText "ab^ /= b^a^, where"
@@ -155,8 +158,8 @@ invertRollback (WithState a x b) =
 -- | recommute   AB ↔ B′A′ if and only if B′A′ ↔ AB
 recommute :: (ShowPatchBasic p, Eq2 p, MightHaveDuplicate p)
           => CommuteFn p p
-          -> (p :> p) wA wB -> TestResult
-recommute c (x :> y) =
+          -> Pair p wA wB -> TestResult
+recommute c (Pair (x :> y)) =
     case c (x :> y) of
     Nothing -> rejected
     Just (y' :> x')
@@ -192,8 +195,8 @@ recommute c (x :> y) =
 -- | commuteInverses   AB ↔ B′A′ if and only if B⁻¹A⁻¹ ↔ A′⁻¹B′⁻¹
 commuteInverses :: (Invert p, ShowPatchBasic p, Eq2 p)
                 => CommuteFn p p
-                -> (p :> p) wA wB -> TestResult
-commuteInverses c (x :> y) =
+                -> Pair p wA wB -> TestResult
+commuteInverses c (Pair (x :> y)) =
     case c (x :> y) of
     Nothing ->
       -- check that inverse commute neither
@@ -235,11 +238,11 @@ effectPreserving
      , ShowPatchBasic p
      )
   => CommuteFn p p
-  -> WithState (p :> p) wA wB
+  -> WithState (Pair p) wA wB
   -> TestResult
-effectPreserving _ (WithState _ (x :> _) _)
+effectPreserving _ (WithState _ (Pair (x :> _)) _)
   | isEmptyHunk x = rejected
-effectPreserving c (WithState r (x :> y) r') =
+effectPreserving c (WithState r (Pair (x :> y)) r') =
   case c (x :> y) of
     Nothing -> rejected
     Just (y' :> x') ->
@@ -282,9 +285,9 @@ effectPreserving c (WithState r (x :> y) r') =
 squareCommuteLaw
   :: (Invert p, ShowPatchBasic p, Eq2 p)
   => CommuteFn p p
-  -> (p :> p) wA wB
+  -> Pair p wA wB
   -> TestResult
-squareCommuteLaw c (x :> y) =
+squareCommuteLaw c (Pair (x :> y)) =
   case c (x :> y) of
     Nothing -> rejected
     Just (y' :> x') ->
@@ -507,8 +510,8 @@ mergeCommute (x :\/: y) =
 coalesceEffectPreserving
             :: TestablePrim prim
             => (forall wX wY . (prim :> prim) wX wY -> Maybe (FL prim wX wY))
-            -> WithState (prim :> prim) wA wB -> TestResult
-coalesceEffectPreserving j (WithState r (a :> b) r') =
+            -> WithState (Pair prim) wA wB -> TestResult
+coalesceEffectPreserving j (WithState r (Pair (a :> b)) r') =
   case j (a :> b) of
        Nothing -> rejected
        Just x  -> case maybeFail $ repoApply r x of
@@ -644,3 +647,25 @@ inverseDoesntCommute x =
     Just (ix' :> x') -> failed $ redText "x:" $$ displayPatch x
       $$ redText "commutes with x^ to ix':" $$ displayPatch ix'
       $$ redText "x':" $$ displayPatch x'
+
+-- This property is just to check the coverage of pairs,
+-- it doesn't test any actual property.
+propPrimPairCoverage :: forall prim wX wY . (Eq2 prim, Commute prim) => Pair prim wX wY -> Property
+propPrimPairCoverage (Pair pq) =
+  checkCoverage $
+  -- The coverage percentages should pass reliably, but
+  -- could be dropped a bit if not.
+  let theKind = classifyCommute pq (commute pq) in
+    cover 20 (theKind == Failed) "Not Commutable" $
+    cover 60 (theKind /= Failed) "Commutable" $
+    cover 20 (theKind == Changed) "Representation Changed" $
+    True
+
+data CommuteKind = Failed | Unchanged | Changed
+  deriving (Eq, Show)
+
+classifyCommute :: Eq2 prim => (prim :> prim) wX wY -> Maybe ((prim :> prim) wX wY) -> CommuteKind
+classifyCommute _ Nothing = Failed
+classifyCommute (p :> q) (Just (q' :> p'))
+  | unsafeCompare p p' && unsafeCompare q q' = Unchanged
+  | otherwise = Changed
