@@ -11,6 +11,7 @@ import qualified Darcs.Test.Patch.Properties.Generic as W
 import Darcs.Test.Patch.Arbitrary.Generic ( MightHaveDuplicate )
 import Darcs.Test.Patch.Arbitrary.PrimV1 ()
 
+import Darcs.Test.Patch.Types.Pair ( Pair(..) )
 import Darcs.Test.Patch.WSub
 import Darcs.Test.Util.TestResult
 
@@ -35,12 +36,12 @@ mergeEitherWay = W.mergeEitherWay . toW
 commuteInverses :: (Invert wp, ShowPatchBasic wp, Eq2 wp, WSub wp p)
                  => (forall wX wY . (p :> p) wX wY -> Maybe ((p :> p) wX wY))
                  -> (p :> p) wA wB -> TestResult
-commuteInverses f = W.commuteInverses (fmap toW . f . fromW) . toW
+commuteInverses f = W.commuteInverses (fmap toW . f . fromW) . Pair . toW
 
 recommute :: (ShowPatchBasic wp, MightHaveDuplicate wp, Eq2 wp, WSub wp p)
           => (forall wX wY . ((p :> p) wX wY -> Maybe ((p :> p) wX wY)))
           -> (p :> p) wA wB -> TestResult
-recommute f = W.recommute (fmap toW . f . fromW) . toW
+recommute f = W.recommute (fmap toW . f . fromW) . Pair . toW
 
 mergeCommute :: ( MightHaveDuplicate wp
                 , ShowPatchBasic wp
@@ -69,7 +70,7 @@ showRead = W.showRead
 squareCommuteLaw :: (Invert wp, ShowPatchBasic wp, Eq2 wp, WSub wp p) =>
                              (forall wX wY . (p :> p) wX wY -> Maybe ((p :> p) wX wY))
                           -> (p :> p) wA wB -> TestResult
-squareCommuteLaw f = W.squareCommuteLaw (fmap toW . f . fromW) . toW
+squareCommuteLaw f = W.squareCommuteLaw (fmap toW . f . fromW) . Pair . toW
 
 
 coalesceCommute :: (forall wX wY . (Prim2 :> Prim2) wX wY -> Maybe (FL Prim2 wX wY))
