@@ -2,36 +2,20 @@ module Darcs.Test.Patch.RepoModel where
 
 import Darcs.Prelude
 
+import Control.Exception ( SomeException )
+
 import Darcs.Patch.Apply ( Apply, ApplyState )
 import Darcs.Patch.Witnesses.Ordered ( FL, RL )
 
 import Test.QuickCheck ( Gen )
 
-data Fail a = Failed String | OK a
-  deriving (Eq, Show)
-
-instance Functor Fail where
-  fmap _ (Failed s) = Failed s
-  fmap f (OK v) = OK (f v)
-
-instance Applicative Fail where
-  pure = OK
-  Failed s <*> _ = Failed s
-  _ <*> Failed s = Failed s
-  OK f <*> OK v = OK (f v)
-
-instance Monad Fail where
-  return = OK
-  Failed s >>= _ = Failed s
-  OK v >>= f = f v
+type Fail = Either SomeException
 
 unFail :: Fail t -> t
-unFail (OK x) = x
-unFail (Failed err) = error $ "unFail failed: " ++ err
+unFail = either (error.show) id
 
 maybeFail :: Fail a -> Maybe a
-maybeFail (OK x) = Just x
-maybeFail _ = Nothing
+maybeFail = either (const Nothing) Just
 
 class RepoModel model where
   type RepoState model :: (* -> *) -> *
