@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -Wno-orphans #-}
 module Darcs.Test.Repository.Inventory where
 
 import Darcs.Prelude
@@ -5,11 +6,10 @@ import Darcs.Prelude
 import Darcs.Repository.Inventory
     ( Inventory(..)
     , HeadInventory
-    , ValidHash(..)
+    , ValidHash
     , InventoryHash
     , PatchHash
     , PristineHash
-    , mkValidHash
     , parseInventory
     , showInventory
     , skipPristineHash
@@ -20,17 +20,21 @@ import Darcs.Repository.Inventory
     , prop_skipPokePristineHash
     )
 import Darcs.Patch.Info ( rawPatchInfo )
+import Darcs.Util.Hash ( sha256strict )
 import Darcs.Util.Printer ( renderPS )
+import Darcs.Util.ValidHash ( decodeValidHash, fromHash, fromSizeAndHash )
 
 import Darcs.Test.Patch.Info ()
 
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
+import Data.Maybe ( fromJust )
 import Test.Framework ( Test, testGroup )
 import Test.Framework.Providers.HUnit ( testCase )
 import Test.Framework.Providers.QuickCheck2 ( testProperty )
 import Test.HUnit ( Assertion, (@=?) )
 import Test.QuickCheck
+import Test.QuickCheck.Instances.ByteString ()
 
 testSuite :: Test
 testSuite = testGroup "Darcs.Repository.Inventory"
@@ -41,9 +45,6 @@ testSuite = testGroup "Darcs.Repository.Inventory"
  , testCase "example2" (testInventory rawHeadInv2 headInv2)
  ]
 
-instance Arbitrary B.ByteString where
-  arbitrary = B.pack <$> arbitrary
-
 instance Arbitrary Inventory where
   arbitrary = uncurry Inventory <$> arbitrary
 
@@ -55,9 +56,14 @@ instance Arbitrary PristineHash where
   arbitrary = arbitraryHash
 
 arbitraryHash :: ValidHash h => Gen h
-arbitraryHash = mkValidHash <$> do
-  n <- elements [64, 75] -- see D.R.Cache.okayHash
-  vectorOf n $ elements $ '-' : (['0'..'9'] ++ ['a'..'f'])
+arbitraryHash = do
+  content       <- arbitrary
+  size_prefixed <- arbitrary
+  let size = B.length content
+      hash = sha256strict content
+  if size_prefixed && size < 1000000000
+    then return (fromSizeAndHash size hash)
+    else return (fromHash hash)
 
 testInventory :: B.ByteString -> HeadInventory -> Assertion
 testInventory raw (hash,inv) = do
@@ -67,6 +73,9 @@ testInventory raw (hash,inv) = do
   rest @=? renderPS (showInventory inv)
   raw @=? renderPS (pokePristineHash hash rest)
 
+mkValidHash :: ValidHash a => String -> a
+mkValidHash = fromJust . decodeValidHash
+
 headInv1 :: HeadInventory
 headInv1 =
   ( mkValidHash "57fb9c1abbed1c0b880e2fffebe32a2163762b87e67e9bf4dcd3168e5abcad83"
