@@ -4,6 +4,8 @@ module Darcs.Patch.Format
     , FileNameFormat(..)
     ) where
 
+import Darcs.Prelude
+
 -- | Showing and reading lists of patches. This class allows us to control how
 -- lists of patches are formatted on disk. For legacy reasons V1 patches have
 -- their own special treatment (see 'ListFormat'). Other patch types use the
@@ -27,9 +29,11 @@ data ListFormat (p :: (* -> * -> *))
                         -- and flatten them out.
     | ListFormatV2      -- ^ Show lists without braces. Read with arbitrary
                         -- nested parens and flatten them out.
+    | ListFormatV3      -- ^ Temporary hack to disable use of showContextSeries
+                        -- for darcs-3 patches, until I find out how to fix this.
 
 data FileNameFormat
-    = OldFormat  -- ^ on-disk format for V1 patches
-    | NewFormat  -- ^ on-disk format for V2 patches
-    | UserFormat -- ^ display format
+    = FileNameFormatV1      -- ^ on-disk format for V1 patches
+    | FileNameFormatV2      -- ^ on-disk format for V2 patches
+    | FileNameFormatDisplay -- ^ display format
     deriving (Eq, Show)
