@@ -19,18 +19,13 @@ module Darcs.Patch.PatchInfoAnd
     ( Hopefully
     , PatchInfoAnd
     , PatchInfoAndG
-    , WPatchInfo
-    , unWPatchInfo
-    , compareWPatchInfo
     , piap
     , n2pia
     , patchInfoAndPatch
     , fmapPIAP
     , fmapFLPIAP
-    , conscientiously
     , hopefully
     , info
-    , winfo
     , hopefullyM
     , createHashed
     , extractHash
@@ -45,40 +40,39 @@ import Control.Exception ( Exception, throw )
 import System.IO.Unsafe ( unsafeInterleaveIO )
 import Data.Typeable ( Typeable )
 
-import Darcs.Util.SignalHandler ( catchNonSignal )
-import Darcs.Util.Printer ( Doc, ($$), renderString, text, vcat )
-import Darcs.Patch.Ident ( Ident(..), PatchId, IdEq2(..) )
-import Darcs.Patch.Info ( PatchInfo, showPatchInfo, displayPatchInfo, justName )
+import Darcs.Patch.Apply ( Apply(..) )
+import Darcs.Patch.Commute ( Commute(..) )
 import Darcs.Patch.Conflict ( Conflict(..) )
 import Darcs.Patch.Debug ( PatchDebug(..) )
 import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
 import Darcs.Patch.Format ( PatchListFormat )
+import Darcs.Patch.FromPrim ( PrimPatchBase(..) )
+import Darcs.Patch.Ident ( Ident(..), PatchId )
+import Darcs.Patch.Info ( PatchInfo, displayPatchInfo, justName, showPatchInfo )
+import Darcs.Patch.Inspect ( PatchInspect(..) )
 import Darcs.Patch.Merge ( CleanMerge(..), Merge(..) )
 import Darcs.Patch.Named ( Named, fmapFL_Named )
-import Darcs.Patch.Apply ( Apply(..) )
-import Darcs.Patch.Commute ( Commute(..) )
-import Darcs.Patch.Inspect ( PatchInspect(..) )
-import Darcs.Patch.FromPrim ( PrimPatchBase(..) )
 import Darcs.Patch.Read ( ReadPatch(..) )
-import Darcs.Patch.Show ( ShowPatch(..) )
 import Darcs.Patch.Repair ( Repair(..), RepairToFL )
-import Darcs.Patch.RepoType ( RepoType(..) )
-import Darcs.Patch.Show ( ShowPatchBasic(..), ShowContextPatch(..) )
+import Darcs.Patch.Show ( ShowPatch(..) )
+import Darcs.Patch.Show ( ShowContextPatch(..), ShowPatchBasic(..) )
 import Darcs.Patch.Summary ( Summary )
-import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
-import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..) )
 import Darcs.Patch.Witnesses.Ordered
-  ( (:/\:)(..)
-  , (:>)(..)
-  , (:\/:)(..)
-  , FL
-  , mapFL
-  , mapRL_RL
-  )
-import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal, mapSeal )
+    ( (:/\:)(..)
+    , (:>)(..)
+    , (:\/:)(..)
+    , FL
+    , mapFL
+    , mapRL_RL
+    )
+import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), mapSeal, seal )
 import Darcs.Patch.Witnesses.Show ( Show1, Show2 )
 import Darcs.Util.Exception ( prettyException )
+import Darcs.Util.Printer ( Doc, renderString, text, vcat, ($$) )
+import Darcs.Util.SignalHandler ( catchNonSignal )
+import Darcs.Util.ValidHash ( PatchHash )
 
 -- | @'Hopefully' p C@ @(x y)@ is @'Either' String (p C@ @(x y))@ in a
 -- form adapted to darcs patches. The @C@ @(x y)@ represents the type
@@ -89,7 +83,7 @@ import Darcs.Util.Exception ( prettyException )
 -- @Hashed hash sh@ represents an expected hashed patch with its hash.
 data Hopefully a wX wY
     = Hopefully (SimpleHopefully a wX wY)
-    | Hashed String (SimpleHopefully a wX wY)
+    | Hashed PatchHash (SimpleHopefully a wX wY)
     deriving Show
 
 -- | @SimpleHopefully@ is a variant of @Either String@ adapted for
@@ -98,70 +92,54 @@ data Hopefully a wX wY
 data SimpleHopefully a wX wY = Actually (a wX wY) | Unavailable String
     deriving Show
 
-type PatchInfoAnd rt p = PatchInfoAndG rt (Named p)
+type PatchInfoAnd p = PatchInfoAndG (Named p)
 
 -- | @'PatchInfoAnd' p wA wB@ represents a hope we have to get a
 -- patch through its info. We're not sure we have the patch, but we
 -- know its info.
-data PatchInfoAndG (rt :: RepoType) p wA wB =
+data PatchInfoAndG p wA wB =
   PIAP !PatchInfo
        (Hopefully p wA wB)
   deriving (Show)
 
--- | @'WPatchInfo' wA wB@ represents the info of a patch, marked with
--- the patch's witnesses.
-newtype WPatchInfo wA wB = WPatchInfo { unWPatchInfo :: PatchInfo }
-
--- This is actually unsafe if we ever commute patches and then compare them
--- using this function. TODO: consider adding an extra existential to WPatchInfo
--- (as with LabelledPatch in Darcs.Patch.Choices)
-compareWPatchInfo :: WPatchInfo wA wB -> WPatchInfo wC wD -> EqCheck (wA, wB) (wC, wD)
-compareWPatchInfo (WPatchInfo x) (WPatchInfo y) = if x == y then unsafeCoerceP IsEq else NotEq
-
-instance Eq2 WPatchInfo where
-   WPatchInfo x `unsafeCompare` WPatchInfo y = x == y
-
 fmapH :: (a wX wY -> b wW wZ) -> Hopefully a wX wY -> Hopefully b wW wZ
 fmapH f (Hopefully sh) = Hopefully (ff sh)
     where ff (Actually a) = Actually (f a)
           ff (Unavailable e) = Unavailable e
-fmapH f (Hashed h sh) = Hashed h (ff sh)
+fmapH f (Hashed _ sh) = Hopefully (ff sh)
     where ff (Actually a) = Actually (f a)
           ff (Unavailable e) = Unavailable e
 
-info :: PatchInfoAndG rt p wA wB -> PatchInfo
+info :: PatchInfoAndG p wA wB -> PatchInfo
 info (PIAP i _) = i
 
-patchDesc :: forall rt p wX wY . PatchInfoAnd rt p wX wY -> String
+patchDesc :: forall p wX wY . PatchInfoAnd p wX wY -> String
 patchDesc p = justName $ info p
 
-winfo :: PatchInfoAnd rt p wA wB -> WPatchInfo wA wB
-winfo (PIAP i _) = WPatchInfo i
-
 -- | @'piap' i p@ creates a PatchInfoAnd containing p with info i.
-piap :: PatchInfo -> p wA wB -> PatchInfoAndG rt p wA wB
+piap :: PatchInfo -> p wA wB -> PatchInfoAndG p wA wB
 piap i p = PIAP i (Hopefully $ Actually p)
 
 -- | @n2pia@ creates a PatchInfoAnd representing a @Named@ patch.
-n2pia :: (Ident p, PatchId p ~ PatchInfo) => p wX wY -> PatchInfoAndG rt p wX wY
+n2pia :: (Ident p, PatchId p ~ PatchInfo) => p wX wY -> PatchInfoAndG p wX wY
 n2pia x = ident x `piap` x
 
-patchInfoAndPatch :: PatchInfo -> Hopefully p wA wB -> PatchInfoAndG rt p wA wB
+patchInfoAndPatch :: PatchInfo -> Hopefully p wA wB -> PatchInfoAndG p wA wB
 patchInfoAndPatch =  PIAP
 
 fmapFLPIAP :: (FL p wX wY -> FL q wX wY)
-           -> PatchInfoAnd rt p wX wY -> PatchInfoAnd rt q wX wY
+           -> PatchInfoAnd p wX wY -> PatchInfoAnd q wX wY
 fmapFLPIAP f (PIAP i hp) = PIAP i (fmapH (fmapFL_Named f) hp)
 
 fmapPIAP :: (p wX wY -> q wX wY)
-           -> PatchInfoAndG rt p wX wY -> PatchInfoAndG rt q wX wY
+           -> PatchInfoAndG p wX wY -> PatchInfoAndG q wX wY
 fmapPIAP f (PIAP i hp) = PIAP i (fmapH f hp)
 
 -- | @'hopefully' hp@ tries to get a patch from a 'PatchInfoAnd'
 -- value. If it fails, it outputs an error \"failed to read patch:
 -- \<description of the patch>\". We get the description of the patch
 -- from the info part of 'hp'
-hopefully :: PatchInfoAndG rt p wA wB -> p wA wB
+hopefully :: PatchInfoAndG p wA wB -> p wA wB
 hopefully = conscientiously $ \e -> text "failed to read patch:" $$ e
 
 -- | Using a special exception type here means that is is treated as
@@ -180,15 +158,14 @@ instance Show PatchNotAvailable where
 -- Note: this function must be lazy in its second argument, which is why we
 -- use a lazy pattern match.
 conscientiously :: (Doc -> Doc)
-                -> PatchInfoAndG rt p wA wB -> p wA wB
+                -> PatchInfoAndG p wA wB -> p wA wB
 conscientiously er ~(PIAP pinf hp) =
     case hopefully2either hp of
       Right p -> p
       Left e -> throw $ PatchNotAvailable $ er (displayPatchInfo pinf $$ text e)
 
--- | @hopefullyM@ is a version of @hopefully@ which calls @fail@ in a
--- monad instead of erroring.
-hopefullyM :: PatchInfoAndG rt p wA wB -> Maybe (p wA wB)
+-- | Return 'Just' the patch content or 'Nothing' if it is unavailable.
+hopefullyM :: PatchInfoAndG p wA wB -> Maybe (p wA wB)
 hopefullyM (PIAP _ hp) = case hopefully2either hp of
                               Right p -> return p
                               Left _ -> Nothing
@@ -203,15 +180,15 @@ hopefully2either (Hashed _ (Unavailable e)) = Left e
 actually :: a wX wY -> Hopefully a wX wY
 actually = Hopefully . Actually
 
-createHashed :: String -> (String -> IO (Sealed (a wX))) -> IO (Sealed (Hopefully a wX))
+createHashed :: PatchHash -> (PatchHash -> IO (Sealed (a wX))) -> IO (Sealed (Hopefully a wX))
 createHashed h f = mapSeal (Hashed h) `fmap` unsafeInterleaveIO (f' `catchNonSignal` handler)
   where
   f' = do Sealed x <- f h
           return (Sealed (Actually x))
   handler e = return $ seal $ Unavailable $ prettyException e
 
-extractHash :: PatchInfoAndG rt p wA wB -> Either (p wA wB) String
-extractHash (PIAP _ (Hashed s _)) = Right s
+extractHash :: PatchInfoAndG p wA wB -> Either (p wA wB) PatchHash
+extractHash (PIAP _ (Hashed sh _)) = Right sh
 extractHash hp = Left $ conscientiously (\e -> text "unable to read patch:" $$ e) hp
 
 unavailable :: String -> Hopefully a wX wY
@@ -219,11 +196,11 @@ unavailable = Hopefully . Unavailable
 
 -- * Instances defined only for PatchInfoAnd
 
-instance Show2 p => Show1 (PatchInfoAnd rt p wX)
+instance Show2 p => Show1 (PatchInfoAnd p wX)
 
-instance Show2 p => Show2 (PatchInfoAnd rt p)
+instance Show2 p => Show2 (PatchInfoAnd p)
 
-instance RepairToFL p => Repair (PatchInfoAnd rt p) where
+instance RepairToFL p => Repair (PatchInfoAnd p) where
     applyAndTryToFix p = do mp' <- applyAndTryToFix $ hopefully p
                             case mp' of
                               Nothing -> return Nothing
@@ -231,38 +208,46 @@ instance RepairToFL p => Repair (PatchInfoAnd rt p) where
 
 -- * Instances defined for PatchInfoAndG
 
-instance PrimPatchBase p => PrimPatchBase (PatchInfoAndG rt p) where
-   type PrimOf (PatchInfoAndG rt p) = PrimOf p
+instance PrimPatchBase p => PrimPatchBase (PatchInfoAndG p) where
+   type PrimOf (PatchInfoAndG p) = PrimOf p
 
--- Equality on PatchInfoAndG is solely determined by the PatchInfo
--- It is a global invariant of darcs that once a patch is recorded,
--- it should always have the same representation in the same context.
-instance Eq2 (PatchInfoAndG rt p) where
-    unsafeCompare (PIAP i _) (PIAP i2 _) = i == i2
+getHopefully :: Hopefully p wX wY -> SimpleHopefully p wX wY
+getHopefully (Hashed _ x) = x
+getHopefully (Hopefully x) = x
 
-type instance PatchId (PatchInfoAndG rt p) = PatchInfo
+instance Eq2 p => Eq2 (SimpleHopefully p) where
+    Actually p1 `unsafeCompare` Actually p2 = p1 `unsafeCompare` p2
+    _ `unsafeCompare` _ = error "cannot compare unavailable patches"
 
-instance Ident (PatchInfoAndG rt p) where
+instance Eq2 p => Eq2 (Hopefully p) where
+    Hashed h1 _ `unsafeCompare` Hashed h2 _ = h1 == h2
+    hp1 `unsafeCompare` hp2 =
+      getHopefully hp1 `unsafeCompare` getHopefully hp2
+
+instance Eq2 p => Eq2 (PatchInfoAndG p) where
+    PIAP i1 p1 `unsafeCompare` PIAP i2 p2 = i1 == i2 && p1 `unsafeCompare` p2
+
+type instance PatchId (PatchInfoAndG p) = PatchInfo
+
+instance Ident (PatchInfoAndG p) where
     ident (PIAP i _) = i
 
-instance IdEq2 (PatchInfoAndG rt p)
+instance PatchListFormat (PatchInfoAndG p)
 
-instance PatchListFormat (PatchInfoAndG rt p)
-
-instance ShowPatchBasic p => ShowPatchBasic (PatchInfoAndG rt p) where
+instance ShowPatchBasic p => ShowPatchBasic (PatchInfoAndG p) where
     showPatch f (PIAP n p) =
       case hopefully2either p of
         Right x -> showPatch f x
         Left _ -> showPatchInfo f n
 
-instance ShowContextPatch p => ShowContextPatch (PatchInfoAndG rt p) where
-  showContextPatch f (PIAP n p) =
+instance ShowContextPatch p => ShowContextPatch (PatchInfoAndG p) where
+  showPatchWithContextAndApply f (PIAP n p) =
     case hopefully2either p of
-      Right x -> showContextPatch f x
+      Right x -> showPatchWithContextAndApply f x
       Left _ -> return $ showPatchInfo f n
 
 instance (Summary p, PatchListFormat p,
-          ShowPatch p) => ShowPatch (PatchInfoAndG rt p) where
+          ShowPatch p) => ShowPatch (PatchInfoAndG p) where
     description (PIAP n _) = displayPatchInfo n
     summary (PIAP _ p) =
       case hopefully2either p of
@@ -274,47 +259,48 @@ instance (Summary p, PatchListFormat p,
         Right x -> content x
         Left _ -> text $ "[patch content is unavailable]"
 
-instance (PatchId p ~ PatchInfo, Commute p) => Commute (PatchInfoAndG rt p) where
+instance (PatchId p ~ PatchInfo, Commute p) => Commute (PatchInfoAndG p) where
     commute (x :> y) = do y' :> x' <- commute (hopefully x :> hopefully y)
                           return $ (ident y `piap` y') :> (ident x `piap` x')
 
 instance (PatchId p ~ PatchInfo, CleanMerge p) =>
-         CleanMerge (PatchInfoAndG rt p) where
+         CleanMerge (PatchInfoAndG p) where
     cleanMerge (x :\/: y)
       | ident x == ident y = error "cannot cleanMerge identical PatchInfoAndG"
       | otherwise = do
           y' :/\: x' <- cleanMerge (hopefully x :\/: hopefully y)
           return $ (ident y `piap` y') :/\: (ident x `piap` x')
 
-instance (PatchId p ~ PatchInfo, Merge p) => Merge (PatchInfoAndG rt p) where
+instance (PatchId p ~ PatchInfo, Merge p) => Merge (PatchInfoAndG p) where
     merge (x :\/: y)
       | ident x == ident y = error "cannot merge identical PatchInfoAndG"
       | otherwise =
           case merge (hopefully x :\/: hopefully y) of
             y' :/\: x' -> (ident y `piap` y') :/\: (ident x `piap` x')
 
-instance PatchInspect p => PatchInspect (PatchInfoAndG rt p) where
+instance PatchInspect p => PatchInspect (PatchInfoAndG p) where
     listTouchedFiles = listTouchedFiles . hopefully
     hunkMatches f = hunkMatches f . hopefully
 
-instance Apply p => Apply (PatchInfoAndG rt p) where
-    type ApplyState (PatchInfoAndG rt p) = ApplyState p
+instance Apply p => Apply (PatchInfoAndG p) where
+    type ApplyState (PatchInfoAndG p) = ApplyState p
     apply = apply . hopefully
-    unapply = unapply .hopefully
+    unapply = unapply . hopefully
 
 instance ( ReadPatch p, Ident p, PatchId p ~ PatchInfo
-         ) => ReadPatch (PatchInfoAndG rt p) where
+         ) => ReadPatch (PatchInfoAndG p) where
     readPatch' = mapSeal n2pia <$> readPatch'
 
-instance Effect p => Effect (PatchInfoAndG rt p) where
+instance Effect p => Effect (PatchInfoAndG p) where
     effect = effect . hopefully
 
-instance IsHunk (PatchInfoAndG rt p) where
+instance IsHunk (PatchInfoAndG p) where
     isHunk _ = Nothing
 
-instance PatchDebug p => PatchDebug (PatchInfoAndG rt p)
+instance PatchDebug p => PatchDebug (PatchInfoAndG p)
 
-instance (Commute p, Conflict p) => Conflict (PatchInfoAnd rt p) where
+instance (Commute p, Conflict p, Summary p, PrimPatchBase p, PatchListFormat p, ShowPatch p) => Conflict (PatchInfoAnd p) where
+    isConflicted = isConflicted . hopefully
     -- Note: this relies on the laziness of 'hopefully' for efficiency
     -- and correctness in the face of lazy repositories
     resolveConflicts context patches =
