@@ -3,11 +3,12 @@ module Darcs.Patch.Prim.FileUUID.Coalesce () where
 
 import Darcs.Prelude
 
-import Darcs.Patch.Prim.Class ( PrimCanonize(..), PrimSift(..) )
-import Darcs.Patch.Prim.FileUUID.Core( Prim )
+import Darcs.Patch.Prim.Class ( PrimCoalesce(..), PrimSift(..) )
+import Darcs.Patch.Prim.FileUUID.Commute ()
+import Darcs.Patch.Prim.FileUUID.Core ( Prim )
 
 -- none of the methods are implemented
-instance PrimCanonize Prim where
+instance PrimCoalesce Prim where
   sortCoalesceFL = id -- just so that we can use it in the tests
 
 -- none of the methods are implemented
