@@ -4,7 +4,7 @@ module Darcs.Patch.Prim.V1.Apply () where
 
 import Darcs.Prelude
 
-import Control.Exception ( throw )
+import Control.Monad.Catch ( MonadThrow(throwM) )
 
 import Darcs.Patch.Apply ( Apply(..) )
 import Darcs.Patch.Repair ( RepairToFL(..) )
@@ -22,7 +22,7 @@ import Darcs.Patch.TokenReplace ( tryTokReplace )
 import Darcs.Patch.ApplyMonad ( ApplyMonadTree(..) )
 import Darcs.Util.Tree( Tree )
 
-import Darcs.Patch.Witnesses.Ordered ( FL(..), mapFL_FL, spanFL, (:>)(..) )
+import Darcs.Patch.Witnesses.Ordered ( FL(..), spanFL, (:>)(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoercePStart )
 
 import Darcs.Util.ByteString ( unlinesPS )
@@ -53,13 +53,13 @@ instance Apply Prim where
     apply (FP f (TokReplace t o n)) = mModifyFilePS f doreplace
         where doreplace fc =
                   case tryTokReplace t (BC.pack o) (BC.pack n) fc of
-                  Nothing -> throw $ userError $ "replace patch to " ++ ap2fp f
+                  Nothing -> throwM $ userError $ "replace patch to " ++ ap2fp f
                              ++ " couldn't apply."
                   Just fc' -> return fc'
     apply (FP f (Binary o n)) = mModifyFilePS f doapply
         where doapply oldf = if o == oldf
                              then return n
-                             else throw $ userError
+                             else throwM $ userError
                                   $ "binary patch to " ++ ap2fp f
                                   ++ " couldn't apply."
     apply (DP d AddDir) = mCreateDirectory d
@@ -126,25 +126,24 @@ instance RepairToFL Prim where
 
 instance PrimApply Prim where
     applyPrimFL NilFL = return ()
-    applyPrimFL (FP f h@(Hunk{}):>:the_ps)
+    applyPrimFL (h@(FP f (Hunk{})):>:the_ps)
      = case spanFL f_hunk the_ps of
            (xs :> ps') ->
-               do let foo = h :>: mapFL_FL (\(FP _ h') -> h') xs
-                  mModifyFilePS f $ hunkmod foo
+               do mModifyFilePS f $ hunkmod (h :>: xs)
                   applyPrimFL ps'
         where f_hunk (FP f' (Hunk{})) = f == f'
               f_hunk _ = False
               -- TODO there should be a HOF that abstracts
               -- over this recursion scheme
-              hunkmod :: Monad m => FL FilePatchType wX wY
+              hunkmod :: MonadThrow m => FL Prim wX wY
                       -> B.ByteString -> m B.ByteString
               hunkmod NilFL content = return content
-              hunkmod (Hunk line old new:>:hs) content =
+              hunkmod (FP _ (Hunk line old new):>:hs) content =
                   applyHunk f (line, old, new) content >>= hunkmod hs
               hunkmod _ _ = error "impossible case"
     applyPrimFL (p:>:ps) = apply p >> applyPrimFL ps
 
-applyHunk :: Monad m
+applyHunk :: MonadThrow m
           => AnchoredPath
           -> (Int, [B.ByteString], [B.ByteString])
           -> FileContents
@@ -153,7 +152,7 @@ applyHunk f h fc =
   case applyHunkLines h fc of
     Right fc' -> return fc'
     Left msg ->
-      throw $ userError $
+      throwM $ userError $
       "### Error applying:\n" ++ renderHunk h ++
       "\n### to file " ++ ap2fp f ++ ":\n" ++ BC.unpack fc ++
       "### Reason: " ++ msg
