@@ -21,6 +21,7 @@ module Darcs.Patch.Set
     , SealedPatchSet
     , Origin
     , progressPatchSet
+    , patchSetInventoryHashes
     , patchSetTags
     , emptyPatchSet
     , appendPSFL
@@ -34,7 +35,9 @@ module Darcs.Patch.Set
 
 import Darcs.Prelude
 import Data.Maybe ( catMaybes )
+import qualified Data.Set as S
 
+import Darcs.Patch.Ident ( Ident(..), PatchId )
 import Darcs.Patch.Info ( PatchInfo, piTag )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
@@ -44,11 +47,12 @@ import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Show ( Show1, Show2 )
 
 import Darcs.Util.Progress ( progress )
+import Darcs.Util.ValidHash ( InventoryHash )
 
 -- |'Origin' is a type used to represent the initial context of a repo.
 data Origin
 
-type SealedPatchSet rt p wStart = Sealed ((PatchSet rt p) wStart)
+type SealedPatchSet p wStart = Sealed ((PatchSet p) wStart)
 
 -- |The patches in a repository are stored in chunks broken up at \"clean\"
 -- tags. A tag is clean if the only patches before it in the current
@@ -65,93 +69,100 @@ type SealedPatchSet rt p wStart = Sealed ((PatchSet rt p) wStart)
 --
 -- The history is lazily loaded from disk so does not normally need to be all
 -- kept in memory.
-data PatchSet rt p wStart wY where
-    PatchSet :: RL (Tagged rt p) Origin wX -> RL (PatchInfoAnd rt p) wX wY
-             -> PatchSet rt p Origin wY
+data PatchSet p wStart wY where
+    PatchSet :: RL (Tagged p) Origin wX -> RL (PatchInfoAnd p) wX wY
+             -> PatchSet p Origin wY
 
-deriving instance Show2 p => Show (PatchSet rt p wStart wY)
+deriving instance Show2 p => Show (PatchSet p wStart wY)
 
-instance Show2 p => Show1 (PatchSet rt p wStart)
+instance Show2 p => Show1 (PatchSet p wStart)
 
-instance Show2 p => Show2 (PatchSet rt p)
+instance Show2 p => Show2 (PatchSet p)
 
+type instance PatchId (PatchSet p) = S.Set PatchInfo
 
-emptyPatchSet :: PatchSet rt p Origin Origin
+instance Ident (PatchSet p) where
+  ident = S.fromList . mapRL ident . patchSet2RL
+
+emptyPatchSet :: PatchSet p Origin Origin
 emptyPatchSet = PatchSet NilRL NilRL
 
 -- |A 'Tagged' is a single chunk of a 'PatchSet'.
 -- It has a 'PatchInfo' representing a clean tag,
 -- the hash of the previous inventory (if it exists),
 -- and the list of patches since that previous inventory.
-data Tagged rt p wX wZ where
-    Tagged :: PatchInfoAnd rt p wY wZ -> Maybe String
-           -> RL (PatchInfoAnd rt p) wX wY -> Tagged rt p wX wZ
+data Tagged p wX wZ where
+    Tagged :: RL (PatchInfoAnd p) wX wY -> PatchInfoAnd p wY wZ -> Maybe InventoryHash
+           -> Tagged p wX wZ
 
-deriving instance Show2 p => Show (Tagged rt p wX wZ)
+deriving instance Show2 p => Show (Tagged p wX wZ)
 
-instance Show2 p => Show1 (Tagged rt p wX)
+instance Show2 p => Show1 (Tagged p wX)
 
-instance Show2 p => Show2 (Tagged rt p)
+instance Show2 p => Show2 (Tagged p)
 
 
 -- |'patchSet2RL' takes a 'PatchSet' and returns an equivalent, linear 'RL' of
 -- patches.
-patchSet2RL :: PatchSet rt p wStart wX -> RL (PatchInfoAnd rt p) wStart wX
+patchSet2RL :: PatchSet p wStart wX -> RL (PatchInfoAnd p) wStart wX
 patchSet2RL (PatchSet ts ps) = concatRL (mapRL_RL ts2rl ts) +<+ ps
   where
-    ts2rl :: Tagged rt p wY wZ -> RL (PatchInfoAnd rt p) wY wZ
-    ts2rl (Tagged t _ ps2) = ps2 :<: t
+    ts2rl :: Tagged p wY wZ -> RL (PatchInfoAnd p) wY wZ
+    ts2rl (Tagged ps2 t _) = ps2 :<: t
 
 -- |'patchSet2FL' takes a 'PatchSet' and returns an equivalent, linear 'FL' of
 -- patches.
-patchSet2FL :: PatchSet rt p wStart wX -> FL (PatchInfoAnd rt p) wStart wX
+patchSet2FL :: PatchSet p wStart wX -> FL (PatchInfoAnd p) wStart wX
 patchSet2FL = reverseRL . patchSet2RL
 
 -- |'appendPSFL' takes a 'PatchSet' and a 'FL' of patches that "follow" the
 -- PatchSet, and concatenates the patches into the PatchSet.
-appendPSFL :: PatchSet rt p wStart wX -> FL (PatchInfoAnd rt p) wX wY
-           -> PatchSet rt p wStart wY
+appendPSFL :: PatchSet p wStart wX -> FL (PatchInfoAnd p) wX wY
+           -> PatchSet p wStart wY
 appendPSFL (PatchSet ts ps) newps = PatchSet ts (ps +<<+ newps)
 
 -- |Runs a progress action for each tag and patch in a given PatchSet, using
 -- the passed progress message. Does not alter the PatchSet.
-progressPatchSet :: String -> PatchSet rt p wStart wX -> PatchSet rt p wStart wX
+progressPatchSet :: String -> PatchSet p wStart wX -> PatchSet p wStart wX
 progressPatchSet k (PatchSet ts ps) =
     PatchSet (mapRL_RL progressTagged ts) (mapRL_RL prog ps)
   where
     prog = progress k
 
-    progressTagged :: Tagged rt p wY wZ -> Tagged rt p wY wZ
-    progressTagged (Tagged t h tps) = Tagged (prog t) h (mapRL_RL prog tps)
+    progressTagged :: Tagged p wY wZ -> Tagged p wY wZ
+    progressTagged (Tagged tps t h) = Tagged (mapRL_RL prog tps) (prog t) h
+
+patchSetInventoryHashes :: PatchSet p wX wY -> [Maybe InventoryHash]
+patchSetInventoryHashes (PatchSet ts _) = mapRL (\(Tagged _ _ mh) -> mh) ts
 
 -- | The tag names of /all/ tags of a given 'PatchSet'.
-patchSetTags :: PatchSet rt p wX wY -> [String]
+patchSetTags :: PatchSet p wX wY -> [String]
 patchSetTags = catMaybes . mapRL (piTag . info) . patchSet2RL
 
-inOrderTags :: PatchSet rt p wS wX -> [PatchInfo]
+inOrderTags :: PatchSet p wS wX -> [PatchInfo]
 inOrderTags (PatchSet ts _) = go ts
-  where go :: RL(Tagged rt t1) wT wY -> [PatchInfo]
-        go (ts' :<: Tagged t _ _) = info t : go ts'
+  where go :: RL(Tagged t1) wT wY -> [PatchInfo]
+        go (ts' :<: Tagged _ t _) = info t : go ts'
         go NilRL = []
 
-patchSetSnoc :: PatchSet rt p wX wY -> PatchInfoAnd rt p wY wZ -> PatchSet rt p wX wZ
+patchSetSnoc :: PatchSet p wX wY -> PatchInfoAnd p wY wZ -> PatchSet p wX wZ
 patchSetSnoc (PatchSet ts ps) p = PatchSet ts (ps :<: p)
 
 -- | Split a 'PatchSet' /before/ the latest known clean tag. The left part
 -- is what comes before the tag, the right part is the tag and its
 -- non-dependencies.
-patchSetSplit :: PatchSet rt p wX wY
-              -> (PatchSet rt p :> RL (PatchInfoAnd rt p)) wX wY
-patchSetSplit (PatchSet (ts :<: Tagged t _ ps') ps) =
+patchSetSplit :: PatchSet p wX wY
+              -> (PatchSet p :> RL (PatchInfoAnd p)) wX wY
+patchSetSplit (PatchSet (ts :<: Tagged ps' t _) ps) =
   PatchSet ts ps' :> ((NilRL :<: t) +<+ ps)
 patchSetSplit (PatchSet NilRL ps) = PatchSet NilRL NilRL :> ps
 
 -- | Drop the last @n@ patches from the given 'PatchSet'.
 patchSetDrop :: Int
-             -> PatchSet rt p wStart wX
-             -> SealedPatchSet rt p wStart
+             -> PatchSet p wStart wX
+             -> SealedPatchSet p wStart
 patchSetDrop n ps | n <= 0 = Sealed ps
-patchSetDrop n (PatchSet (ts :<: Tagged t _ ps) NilRL) =
+patchSetDrop n (PatchSet (ts :<: Tagged ps t _) NilRL) =
   patchSetDrop n $ PatchSet ts (ps :<: t)
 patchSetDrop _ (PatchSet NilRL NilRL) = Sealed $ PatchSet NilRL NilRL
 patchSetDrop n (PatchSet ts (ps :<: _)) = patchSetDrop (n - 1) $ PatchSet ts ps
