@@ -1,24 +1,66 @@
 module Darcs.Patch.Summary
-    ( plainSummary, plainSummaryPrim, plainSummaryPrims,
-      xmlSummary )
-    where
+    ( plainSummary
+    , plainSummaryFL
+    , plainSummaryPrim
+    , plainSummaryPrims
+    , xmlSummary
+    , Summary(..)
+    , ConflictState(..)
+    , IsConflictedPrim(..)
+    , listConflictedFiles
+    ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Util.Path ( fn2fp )
-import Darcs.Patch.Conflict ( Conflict(..), IsConflictedPrim(IsC), ConflictState(..) )
-import Darcs.Patch.Format ( FileNameFormat(UserFormat) )
-import Darcs.Patch.Prim.Class ( PrimDetails(..), PrimPatchBase )
+import Data.List.Ordered ( nubSort )
+import Data.Maybe ( catMaybes )
+
+import Darcs.Patch.Format ( FileNameFormat(FileNameFormatDisplay) )
+import Darcs.Patch.FromPrim ( PrimPatchBase(..) )
+import Darcs.Patch.Inspect ( PatchInspect(..) )
+import Darcs.Patch.Prim ( PrimDetails(..) )
 import Darcs.Patch.Show ( formatFileName )
 import Darcs.Patch.SummaryData ( SummDetail(..), SummOp(..) )
 import Darcs.Patch.Witnesses.Ordered ( FL, mapFL )
+import Darcs.Patch.Witnesses.Show
 
-import Darcs.Util.Printer ( Doc, empty, vcat,
-                 text,
-                 minus, plus, ($$), (<+>)
-               )
+import Darcs.Util.Path ( AnchoredPath, anchorPath )
+import Darcs.Util.Printer
+    ( Doc
+    , ($$)
+    , (<+>)
+    , empty
+    , minus
+    , plus
+    , text
+    , vcat
+    )
 
+-- | This type tags a patch with a 'ConflictState' and also hides the context
+-- witnesses (as in 'Sealed2'), so we can put them in a list.
+data IsConflictedPrim prim where
+    IsC :: !ConflictState -> !(prim wX wY) -> IsConflictedPrim prim
+data ConflictState = Okay | Conflicted | Duplicated deriving ( Eq, Ord, Show, Read)
+
+class Summary p where
+    conflictedEffect :: p wX wY -> [IsConflictedPrim (PrimOf p)]
+
+instance Summary p => Summary (FL p) where
+    conflictedEffect = concat . mapFL conflictedEffect
+
+instance Show2 prim => Show (IsConflictedPrim prim) where
+    showsPrec d (IsC cs prim) =
+        showParen (d > appPrec) $
+            showString "IsC " . showsPrec (appPrec + 1) cs .
+            showString " " . showsPrec2 (appPrec + 1) prim
+
+listConflictedFiles
+  :: (Summary p, PatchInspect (PrimOf p)) => p wX wY -> [AnchoredPath]
+listConflictedFiles =
+    nubSort . concat . catMaybes . map conflictedFiles . conflictedEffect
+  where
+    conflictedFiles (IsC Conflicted p) = Just (listTouchedFiles p)
+    conflictedFiles _ = Nothing
 
 plainSummaryPrim :: PrimDetails prim => prim wX wY -> Doc
 plainSummaryPrim = vcat . map (summChunkToLine False) . genSummary . (:[]) . IsC Okay
@@ -27,10 +69,13 @@ plainSummaryPrims :: PrimDetails prim => Bool -> FL prim wX wY -> Doc
 plainSummaryPrims machineReadable =
  vcat . map (summChunkToLine machineReadable) . genSummary . mapFL (IsC Okay)
 
-plainSummary :: (Conflict e, PrimPatchBase e) => e wX wY -> Doc
+plainSummary :: (Summary e, PrimDetails (PrimOf e)) => e wX wY -> Doc
 plainSummary = vcat . map (summChunkToLine False) . genSummary . conflictedEffect
 
-xmlSummary :: (Conflict p, PrimPatchBase p) => p wX wY -> Doc
+plainSummaryFL :: (Summary e, PrimDetails (PrimOf e)) => FL e wX wY -> Doc
+plainSummaryFL = vcat . map (summChunkToLine False) . genSummary . concat . mapFL conflictedEffect
+
+xmlSummary :: (Summary p, PrimDetails (PrimOf p)) => p wX wY -> Doc
 xmlSummary p = text "<summary>"
              $$ (vcat . map summChunkToXML . genSummary . conflictedEffect $ p)
              $$ text "</summary>"
@@ -98,7 +143,7 @@ summChunkToXML (SummChunk detail c) =
    xconf Okay t x       = text ('<':t++">") $$ x $$ text ("</"++t++">")
    xconf Conflicted t x = text ('<':t++" conflict='true'>") $$ x $$ text ("</"++t++">")
    xconf Duplicated t x = text ('<':t++" duplicate='true'>") $$ x $$ text ("</"++t++">")
-   xfn = escapeXML . dropDotSlash . fn2fp
+   xfn = escapeXML . anchorPath ""
    --
    xad 0 = empty
    xad a = text "<added_lines num='" <> text (show a) <> text "'/>"
@@ -110,18 +155,18 @@ summChunkToXML (SummChunk detail c) =
 summChunkToLine :: Bool -> SummChunk -> Doc
 summChunkToLine machineReadable (SummChunk detail c) =
   case detail of
-   SummRmDir f   -> lconf c "R" $ formatFileName UserFormat f <> text "/"
-   SummAddDir f  -> lconf c "A" $ formatFileName UserFormat f <> text "/"
-   SummFile SummRm  f _ _ _ -> lconf c "R" $ formatFileName UserFormat f
-   SummFile SummAdd f _ _ _ -> lconf c "A" $ formatFileName UserFormat f
+   SummRmDir f   -> lconf c "R" $ formatFileName FileNameFormatDisplay f <> text "/"
+   SummAddDir f  -> lconf c "A" $ formatFileName FileNameFormatDisplay f <> text "/"
+   SummFile SummRm  f _ _ _ -> lconf c "R" $ formatFileName FileNameFormatDisplay f
+   SummFile SummAdd f _ _ _ -> lconf c "A" $ formatFileName FileNameFormatDisplay f
    SummFile SummMod f r a x
-     | machineReadable -> lconf c "M" $ formatFileName UserFormat f
-     | otherwise       -> lconf c "M" $ formatFileName UserFormat f <+> rm r <+> ad a <+> rp x
+     | machineReadable -> lconf c "M" $ formatFileName FileNameFormatDisplay f
+     | otherwise       -> lconf c "M" $ formatFileName FileNameFormatDisplay f <+> rm r <+> ad a <+> rp x
    SummMv f1 f2
-     | machineReadable -> text "F " <> formatFileName UserFormat f1
-                       $$ text "T " <> formatFileName UserFormat f2
-     | otherwise       -> text " "    <> formatFileName UserFormat f1
-                       <> text " -> " <> formatFileName UserFormat f2
+     | machineReadable -> text "F " <> formatFileName FileNameFormatDisplay f1
+                       $$ text "T " <> formatFileName FileNameFormatDisplay f2
+     | otherwise       -> text " "    <> formatFileName FileNameFormatDisplay f1
+                       <> text " -> " <> formatFileName FileNameFormatDisplay f2
    SummNone -> case c of
                Okay -> empty
                _    -> lconf c ""  empty
@@ -138,8 +183,3 @@ summChunkToLine machineReadable (SummChunk detail c) =
    rm a = minus <> text (show a)
    rp 0 = empty
    rp a = text "r" <> text (show a)
-
-dropDotSlash :: FilePath -> FilePath
-dropDotSlash ('.':'/':str) = dropDotSlash str
-dropDotSlash str = str
-
