@@ -1,10 +1,8 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.V1.Apply () where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Patch.Annotate ( Annotate(..) )
 import Darcs.Patch.Apply ( ApplyState, Apply, apply )
 import Darcs.Patch.Prim ( PrimPatch, applyPrimFL )
 import Darcs.Patch.Repair ( RepairToFL, applyAndTryToFixFL,
@@ -23,6 +21,3 @@ instance PrimPatch prim => Apply (RepoPatchV1 prim) where
 instance PrimPatch prim => RepairToFL (RepoPatchV1 prim) where
     applyAndTryToFixFL (PP x) = mapMaybeSnd (mapFL_FL PP) `fmap` applyAndTryToFixFL x
     applyAndTryToFixFL x = do apply x; return Nothing
-
-instance (PrimPatch prim, Annotate prim) => Annotate (RepoPatchV1 prim) where
-    annotate = annotate . effect
