@@ -16,6 +16,7 @@
 --  Boston, MA 02110-1301, USA.
 
 {-# OPTIONS_GHC -fno-warn-orphans #-}
+{-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
 
 module Darcs.Patch.V1.Commute
     (
@@ -30,10 +31,11 @@ import Darcs.Prelude
 import Control.Monad ( MonadPlus, mplus, msum, mzero, guard )
 import Control.Applicative ( Alternative(..) )
 import Data.Maybe ( fromMaybe )
+import Safe ( headErr )
 
+import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.Commute ( selfCommuter )
 import Darcs.Patch.CommuteFn ( commuterIdFL, commuterFLId )
-import Darcs.Util.Path ( AnchoredPath )
 import Darcs.Patch.Invert ( invertRL )
 import Darcs.Patch.Merge ( CleanMerge(..), Merge(..) )
 import Darcs.Patch.Commute ( Commute(..) )
@@ -52,15 +54,15 @@ import Darcs.Patch.Conflict
 import Darcs.Patch.Unwind ( Unwind(..), Unwound(..), mkUnwound )
 import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
-import Darcs.Patch.Prim ( PrimPatch, is_filepatch )
+import Darcs.Patch.Prim ( PrimPatch )
 import Darcs.Patch.Permutations
     ( headPermutationsRL
     , simpleHeadPermutationsFL
     , removeFL
+    , nubFL
     )
 import Darcs.Util.Printer ( renderString, text, vcat, ($$) )
 import Darcs.Patch.V1.Show ( showPatch_ )
-import Data.List ( nub )
 import Data.List.Ordered ( nubSort )
 import Darcs.Patch.Summary
     ( Summary(..)
@@ -102,7 +104,7 @@ instance  Monad Perhaps where
     (Succeeded x) >>= k =  k x
     Failed   >>= _      =  Failed
     Unknown  >>= _      =  Unknown
-    return              =  Succeeded
+    return              =  pure
 
 instance Alternative Perhaps where
     empty = Unknown
@@ -139,15 +141,6 @@ cleverCommute c (p1 :> p2) = case c (p1 :> p2) of
                  Failed -> Failed
                  Unknown -> Unknown
 
--- | If we have two Filepatches which modify different files, we can return a
--- result early, since the patches trivially commute.
-speedyCommute :: PrimPatch prim => CommuteFunction prim
-speedyCommute (p1 :> p2)
-    | Just m1 <- isFilepatchMerger p1
-    , Just m2 <- isFilepatchMerger p2
-    , m1 /= m2 = Succeeded (unsafeCoerceP p2 :> unsafeCoerceP p1)
-    | otherwise = Unknown
-
 everythingElseCommute :: forall prim . PrimPatch prim => CommuteFunction prim
 everythingElseCommute (PP p1 :> PP p2) = toPerhaps $ do
     p2' :> p1' <- commute (p1 :> p2)
@@ -199,8 +192,7 @@ instance PrimPatch prim => Merge (RepoPatchV1 prim) where
 
 instance PrimPatch prim => Commute (RepoPatchV1 prim) where
     commute x = toMaybe $ msum
-                  [speedyCommute x,
-                   (cleverCommute mergerCommute) x,
+                  [(cleverCommute mergerCommute) x,
                    everythingElseCommute x
                   ]
 
@@ -215,15 +207,6 @@ instance PrimPatch prim => PatchInspect (RepoPatchV1 prim) where
     hunkMatches f c@(Regrem{}) = hunkMatches f $ invert c
     hunkMatches f (PP p) = hunkMatches f p
 
-isFilepatchMerger :: PrimPatch prim => RepoPatchV1 prim wX wY -> Maybe AnchoredPath
-isFilepatchMerger (PP p) = is_filepatch p
-isFilepatchMerger (Merger _ _ p1 p2) = do
-     f1 <- isFilepatchMerger p1
-     f2 <- isFilepatchMerger p2
-     if f1 == f2 then return f1 else Nothing
-isFilepatchMerger (Regrem und unw p1 p2)
-    = isFilepatchMerger (Merger und unw p1 p2)
-
 commuteRecursiveMerger :: PrimPatch prim
     => (RepoPatchV1 prim :> RepoPatchV1 prim) wX wY -> Perhaps ((RepoPatchV1 prim :> RepoPatchV1 prim) wX wY)
 commuteRecursiveMerger (pA :> p@(Merger _ _ p1 p2)) = toPerhaps $
@@ -345,12 +328,11 @@ putBefore p1 (p2:>:p2s) =
 putBefore _ NilFL = Just (unsafeCoerceP NilFL)
 
 instance PrimPatch prim => CommuteNoConflicts (RepoPatchV1 prim) where
-  commuteNoConflicts x =
-    toMaybe $ msum [ speedyCommute x
-                   , everythingElseCommute x
-                   ]
+  commuteNoConflicts x = toMaybe $ everythingElseCommute x
 
 instance PrimPatch prim => Conflict (RepoPatchV1 prim) where
+  isConflicted (PP _) = False
+  isConflicted _ = True
   resolveConflicts _ = map mangleOrFail . combineConflicts resolveOne
     where
       resolveOne p | isMerger p = [publicUnravel p]
@@ -389,7 +371,7 @@ dropAllInverses (p :>: ps) =
   fromMaybe (p :>: ps') $ removeFL (invert p) ps'
 
 unravel :: PrimPatch prim => RepoPatchV1 prim wX wY -> [Sealed (FL prim wX)]
-unravel p = nub $ map (mapSeal (dropAllInverses . concatFL . mapFL_FL effect)) $
+unravel p = nubFL $ map (mapSeal (dropAllInverses . concatFL . mapFL_FL effect)) $
             getSupers $ map (mapSeal reverseRL) $ unseal (newUr p) $ unwind p
 
 getSupers :: PrimPatch prim
@@ -409,7 +391,7 @@ Sealed x `isSuperpatchOf` Sealed y = x `iso` y
           _ `iso` NilFL = True
           NilFL `iso` _ = False
           a `iso` (b:>:bs) =
-              head $ ([as `iso` bs | (ah :>: as) <- simpleHeadPermutationsFL a, IsEq <- [ah =\/= b]] :: [Bool]) ++ [False]
+              headErr $ ([as `iso` bs | (ah :>: as) <- simpleHeadPermutationsFL a, IsEq <- [ah =\/= b]] :: [Bool]) ++ [False]
 
 -- | merger takes two patches, (which have been determined to conflict) and
 -- constructs a Merger patch to represent the conflict. @p1@ is considered to
@@ -440,7 +422,8 @@ instance PrimPatch prim => Effect (RepoPatchV1 prim) where
     effect p@(Regrem{}) = invert $ effect $ invert p
     effect (PP p) = p :>: NilFL
 
-instance IsHunk prim => IsHunk (RepoPatchV1 prim) where
+instance (PrimPatch prim, ApplyState prim ~ ApplyState (RepoPatchV1 prim)) =>
+         IsHunk (RepoPatchV1 prim) where
     isHunk p = do PP p' <- return p
                   isHunk p'
 
