@@ -1,14 +1,13 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.V1.Read () where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Darcs.Patch.Invert ( invert )
 import Darcs.Patch.Prim ( PrimPatch )
 import Darcs.Patch.Read ( ReadPatch(..) )
-import Darcs.Patch.ReadMonads ( ParserM, choice, string,
-                                lexChar, myLex', skipSpace )
+import Darcs.Util.Parser ( Parser, choice, string,
+                                lexChar, lexWord, skipSpace )
 
 import Darcs.Patch.V1.Core ( RepoPatchV1(..) )
 import Darcs.Patch.V1.Commute ( merger )
@@ -27,9 +26,9 @@ instance PrimPatch prim => ReadPatch (RepoPatchV1 prim) where
             , liftM seal $ skipSpace >> readMerger False
             , liftM (mapSeal PP) readPatch'
             ]
-readMerger :: (ParserM m, PrimPatch prim) => Bool -> m (RepoPatchV1 prim wX wY)
+readMerger :: (PrimPatch prim) => Bool -> Parser (RepoPatchV1 prim wX wY)
 readMerger b = do string s
-                  g <- myLex'
+                  g <- lexWord
                   lexChar '('
                   Sealed p1 <- readPatch'
                   Sealed p2 <- readPatch'
