@@ -1,9 +1,5 @@
 module Darcs.Patch.Witnesses.Show
-    ( ShowDict(..)
-    , showD
-    , showListD
-    , showsPrecD
-    , Show1(..)
+    ( Show1(..)
     , Show2(..)
     , show1
     , showsPrec1
@@ -13,29 +9,24 @@ module Darcs.Patch.Witnesses.Show
     , appPrec
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Darcs.Util.Show ( appPrec )
 
-data ShowDict a where
-    ShowDictClass :: Show a => ShowDict a
-    ShowDictRecord :: (Int -> a -> ShowS) -> (a -> String) -> ([a] -> ShowS) -> ShowDict a
+import Data.Constraint ( Dict(..) )
+
+type ShowDict a = Dict (Show a)
 
 showsPrecD :: ShowDict a -> Int -> a -> ShowS
-showsPrecD ShowDictClass       = showsPrec
-showsPrecD (ShowDictRecord showsPrecR _ _) = showsPrecR
+showsPrecD Dict       = showsPrec
 
 showD :: ShowDict a -> a -> String
-showD ShowDictClass       = show
-showD (ShowDictRecord _ showR _) = showR
-
-showListD :: ShowDict a -> [a] -> ShowS
-showListD ShowDictClass       = showList
-showListD (ShowDictRecord _ _ showListR) = showListR
+showD Dict       = show
 
 class Show1 a where
-    showDict1 :: ShowDict (a wX)
+  showDict1 :: Dict (Show (a wX))
+  default showDict1 :: Show (a wX) => ShowDict (a wX)
+  showDict1 = Dict
 
 showsPrec1 :: Show1 a => Int -> a wX -> ShowS
 showsPrec1 = showsPrecD showDict1
@@ -44,7 +35,9 @@ show1 :: Show1 a => a wX -> String
 show1 = showD showDict1
 
 class Show2 a where
-    showDict2 :: ShowDict (a wX wY)
+  showDict2 :: ShowDict (a wX wY)
+  default showDict2 :: Show (a wX wY) => ShowDict (a wX wY)
+  showDict2 = Dict
 
 showsPrec2 :: Show2 a => Int -> a wX wY -> ShowS
 showsPrec2 = showsPrecD showDict2
