@@ -2,7 +2,6 @@ module Darcs.Patch.Witnesses.Unsafe
     ( unsafeCoerceP
     , unsafeCoercePStart
     , unsafeCoercePEnd
-    , unsafeCoerceP2
     , unsafeCoerceP1
     ) where
 
@@ -17,8 +16,5 @@ unsafeCoercePStart = unsafeCoerce
 unsafeCoercePEnd :: a wX wY1 -> a wX wY2
 unsafeCoercePEnd = unsafeCoerce
 
-unsafeCoerceP2 :: t wW wX wY wZ -> t wA wB wC wD
-unsafeCoerceP2 = unsafeCoerce
-
 unsafeCoerceP1 :: a wX -> a wY
 unsafeCoerceP1 = unsafeCoerce
