@@ -18,6 +18,7 @@
 
 module Darcs.Repository
     ( Repository
+    , AccessType(..)
     , repoLocation
     , repoFormat
     , repoPristineType
@@ -35,7 +36,6 @@ module Darcs.Repository
     , maybeIdentifyRepository
     , identifyRepositoryFor
     , ReadingOrWriting(..)
-    , withRecorded
     , withRepoLock
     , withRepoLockCanFail
     , withRepository
@@ -45,19 +45,21 @@ module Darcs.Repository
     , amInRepository
     , amNotInRepository
     , amInHashedRepository
-    , replacePristine
-    , readRepo
+    , writePristine
+    , readPatches
     , prefsUrl
     , addToPending
-    , addPendingDiffToPending
+    , unsafeAddToPending
     , tentativelyAddPatch
+    , tentativelyAddPatches
     , tentativelyRemovePatches
-    , tentativelyAddToPending
-    , readTentativeRepo
+    , setTentativePending
+    , tentativelyRemoveFromPW
     , withManualRebaseUpdate
     , tentativelyMergePatches
     , considerMergeToWorking
     , revertRepositoryChanges
+    , UpdatePending(..)
     , finalizeRepositoryChanges
     , createRepository
     , createRepositoryV1
@@ -66,38 +68,31 @@ module Darcs.Repository
     , cloneRepository
     , applyToWorking
     , createPristineDirectoryTree
-    , createPartialsPristineDirectoryTree
     , reorderInventory
     , cleanRepository
     , PatchSet
     , SealedPatchSet
     , PatchInfoAnd
-    , setScriptsExecutable
+    , setAllScriptsExecutable
     , setScriptsExecutablePatches
-    , testTentative
     , modifyCache
     -- * Recorded and unrecorded and pending.
-    , readRecorded
+    , readPristine
     , readUnrecorded
     , unrecordedChanges
     , readPendingAndWorking
     , filterOutConflicts
-    , readRecordedAndPending
-    -- * Index.
-    , readIndex
-    , invalidateIndex
+    , readPristineAndPending
     ) where
 
 import Darcs.Repository.State
-    ( readRecorded
+    ( readPristine
     , readUnrecorded
     , unrecordedChanges
     , readPendingAndWorking
-    , readIndex
-    , invalidateIndex
-    , readRecordedAndPending
+    , readPristineAndPending
     , filterOutConflicts
-    , addPendingDiffToPending
+    , unsafeAddToPending
     , addToPending
     )
 
@@ -113,26 +108,25 @@ import Darcs.Repository.Identify
     , amInHashedRepository
     )
 import Darcs.Repository.Hashed
-    ( readRepo
-    , readTentativeRepo
+    ( readPatches
     , tentativelyAddPatch
+    , tentativelyAddPatches
     , tentativelyRemovePatches
-    , revertRepositoryChanges
-    , finalizeRepositoryChanges
     , reorderInventory
     )
 import Darcs.Repository.Pristine
-    ( withRecorded
-    , createPristineDirectoryTree
-    , createPartialsPristineDirectoryTree
+    ( createPristineDirectoryTree
+    , writePristine
+    )
+import Darcs.Repository.Transaction
+    ( revertRepositoryChanges
+    , finalizeRepositoryChanges
     )
 import Darcs.Repository.Traverse ( cleanRepository )
-import Darcs.Repository.Pending
-    ( tentativelyAddToPending
-    )
+import Darcs.Repository.Pending ( setTentativePending, tentativelyRemoveFromPW )
 import Darcs.Repository.Working
     ( applyToWorking
-    , setScriptsExecutable
+    , setAllScriptsExecutable
     , setScriptsExecutablePatches
     )
 import Darcs.Repository.Job
@@ -144,11 +138,10 @@ import Darcs.Repository.Job
     , withUMaskFlag
     )
 import Darcs.Repository.Rebase ( withManualRebaseUpdate )
-import Darcs.Repository.Test ( testTentative )
 import Darcs.Repository.Merge( tentativelyMergePatches
                              , considerMergeToWorking
                              )
-import Darcs.Repository.Cache
+import Darcs.Util.Cache
     ( Cache
     , CacheLoc(..)
     , CacheType(..)
@@ -160,6 +153,7 @@ import Darcs.Repository.Cache
     )
 import Darcs.Repository.InternalTypes
     ( Repository
+    , AccessType(..)
     , PristineType(..)
     , modifyCache
     , repoLocation
@@ -169,7 +163,6 @@ import Darcs.Repository.InternalTypes
     )
 import Darcs.Repository.Clone
     ( cloneRepository
-    , replacePristine
     )
 import Darcs.Repository.Create
     ( createRepository
@@ -177,6 +170,7 @@ import Darcs.Repository.Create
     , createRepositoryV2
     , EmptyRepository(..)
     )
+import Darcs.Repository.Flags ( UpdatePending(..) )
 
 import Darcs.Patch.Set ( PatchSet, SealedPatchSet )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd )
