@@ -19,14 +19,14 @@
 {-# LANGUAGE MultiParamTypeClasses #-}
 
 module Darcs.Repository.ApplyPatches
-    ( applyPatches
-    , runTolerantly
+    ( runTolerantly
     , runSilently
     , DefaultIO, runDefault
     ) where
 
 import Control.Exception ( IOException, SomeException, catch )
 import Control.Monad ( unless )
+import Control.Monad.Catch ( MonadThrow )
 import qualified Data.ByteString as B ( empty, null, readFile )
 import Data.Char ( toLower )
 import Data.List ( isSuffixOf )
@@ -39,87 +39,57 @@ import System.Directory
     , renamePath
     )
 import System.IO ( hPutStrLn, stderr )
-import System.IO.Error ( catchIOError, isDoesNotExistError, isPermissionError )
+import System.IO.Error
+    ( catchIOError
+    , isAlreadyExistsError
+    , isDoesNotExistError
+    , isPermissionError
+    )
 
 import Darcs.Prelude
 
-import Darcs.Patch.Apply ( Apply(..) )
 import Darcs.Patch.ApplyMonad ( ApplyMonad(..), ApplyMonadTree(..) )
-import Darcs.Patch.Info ( displayPatchInfo )
-import Darcs.Patch.MonadProgress ( MonadProgress(..), ProgressAction(..) )
-import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, hopefully, info )
-import Darcs.Patch.Witnesses.Ordered ( FL(..), mapFL )
 import Darcs.Repository.Prefs ( changePrefval )
 import Darcs.Util.Exception ( prettyException )
-import Darcs.Util.External ( backupByCopying, backupByRenaming )
+import Darcs.Util.File ( backupByCopying, backupByRenaming )
 import Darcs.Util.Lock ( writeAtomicFilePS )
-import Darcs.Util.Path ( AnchoredPath, anchorPath )
-import Darcs.Util.Printer ( hPutDocLn, renderString )
-import Darcs.Util.Printer ( text, ($$) )
-import Darcs.Util.Progress ( beginTedious, endTedious, finishedOneIO, tediousSize )
+import Darcs.Util.Path ( AnchoredPath, realPath )
 import Darcs.Util.Tree ( Tree )
 
-applyPatches :: (MonadProgress m, ApplyMonad (ApplyState p) m, Apply p)
-             => FL (PatchInfoAnd rt p) wX wY -> m ()
-applyPatches ps = runProgressActions "Applying patch" (mapFL doApply ps)
-  where
-    doApply hp = ProgressAction { paAction = apply (hopefully hp)
-                                , paMessage = displayPatchInfo (info hp)
-                                , paOnError = text "Unapplicable patch:" $$
-                                              displayPatchInfo (info hp)
-                                }
-
-ap2fp :: AnchoredPath -> FilePath
-ap2fp = anchorPath ""
-
 newtype DefaultIO a = DefaultIO { runDefaultIO :: IO a }
-    deriving (Functor, Applicative, Monad)
-
-instance MonadProgress DefaultIO where
-  runProgressActions _ [] = return ()
-  runProgressActions what items = DefaultIO $ do
-    do beginTedious what
-       tediousSize what (length items)
-       mapM_ go items
-       endTedious what
-    where go item =
-            do finishedOneIO what (renderString $ paMessage item)
-               runDefaultIO (paAction item) `catch` \e ->
-                 do hPutDocLn stderr $ paOnError item
-                    ioError e
+    deriving (Functor, Applicative, Monad, MonadThrow)
 
 instance ApplyMonad Tree DefaultIO where
-    type ApplyMonadBase DefaultIO = IO
 
 instance ApplyMonadTree DefaultIO where
-    mDoesDirectoryExist = DefaultIO . doesDirectoryExist . ap2fp
+    mDoesDirectoryExist = DefaultIO . doesDirectoryExist . realPath
     mChangePref a b c = DefaultIO $ changePrefval a b c
-    mModifyFilePS f j = DefaultIO $ B.readFile (ap2fp f) >>= runDefaultIO . j >>= writeAtomicFilePS (ap2fp f)
-    mCreateDirectory = DefaultIO . createDirectory . ap2fp
+    mModifyFilePS f j = DefaultIO $ B.readFile (realPath f) >>= runDefaultIO . j >>= writeAtomicFilePS (realPath f)
+    mCreateDirectory = DefaultIO . createDirectory . realPath
     mCreateFile f = DefaultIO $
-                    do exf <- doesFileExist (ap2fp f)
-                       if exf then fail $ "File '"++ap2fp f++"' already exists!"
-                              else do exd <- doesDirectoryExist $ ap2fp f
-                                      if exd then fail $ "File '"++ap2fp f++"' already exists!"
-                                             else writeAtomicFilePS (ap2fp f) B.empty
+                    do exf <- doesFileExist (realPath f)
+                       if exf then fail $ "File '"++realPath f++"' already exists!"
+                              else do exd <- doesDirectoryExist $ realPath f
+                                      if exd then fail $ "File '"++realPath f++"' already exists!"
+                                             else writeAtomicFilePS (realPath f) B.empty
     mRemoveFile f = DefaultIO $
-                    do let fp = ap2fp f
+                    do let fp = realPath f
                        x <- B.readFile fp
                        unless (B.null x) $
                             fail $ "Cannot remove non-empty file "++fp
                        removeFile fp
-    mRemoveDirectory = DefaultIO . removeDirectory . ap2fp
+    mRemoveDirectory = DefaultIO . removeDirectory . realPath
     mRename a b = DefaultIO $ renamePath x y
-      where x = ap2fp a
-            y = ap2fp b
+      where x = realPath a
+            y = realPath b
 
-class (Functor m, Monad m) => TolerantMonad m where
+class (Functor m, MonadThrow m) => TolerantMonad m where
     warning :: IO () -> m ()
     runIO :: m a -> IO a
     runTM :: IO a -> m a
 
 newtype TolerantIO a = TIO { runTIO :: IO a }
-    deriving (Functor, Applicative, Monad)
+    deriving (Functor, Applicative, Monad, MonadThrow)
 
 instance TolerantMonad TolerantIO where
     warning io = TIO $ io `catch` \e -> hPutStrLn stderr $ "Warning: " ++ prettyException e
@@ -127,7 +97,7 @@ instance TolerantMonad TolerantIO where
     runTM = TIO
 
 newtype SilentIO a = SIO { runSIO :: IO a }
-    deriving (Functor, Applicative, Monad)
+    deriving (Functor, Applicative, Monad, MonadThrow)
 
 instance TolerantMonad SilentIO where
     warning io = SIO $ io `catch` \(_ :: SomeException) -> return ()
@@ -137,6 +107,8 @@ instance TolerantMonad SilentIO where
 newtype TolerantWrapper m a = TolerantWrapper { runTolerantWrapper :: m a }
     deriving (Functor, Applicative, Monad, TolerantMonad)
 
+deriving instance MonadThrow m => MonadThrow (TolerantWrapper m)
+
 -- | Apply patches, emitting warnings if there are any IO errors
 runTolerantly :: TolerantWrapper TolerantIO a -> IO a
 runTolerantly = runTIO . runTolerantWrapper
@@ -155,7 +127,6 @@ runDefault action =
       "\npatches in your repo, and perhaps 'darcs repair' to fix them."
 
 instance TolerantMonad m => ApplyMonad Tree (TolerantWrapper m) where
-    type ApplyMonadBase (TolerantWrapper m) = IO
 
 instance TolerantMonad m => ApplyMonadTree (TolerantWrapper m) where
     mDoesDirectoryExist d = runTM $ runDefaultIO $ mDoesDirectoryExist d
@@ -170,25 +141,27 @@ instance TolerantMonad m => ApplyMonadTree (TolerantWrapper m) where
                                  (\(e :: IOException) ->
                                    if "(Directory not empty)" `isSuffixOf` show e
                                    then ioError $ userError $
-                                            "Not deleting " ++ ap2fp d ++ " because it is not empty."
+                                            "Not deleting " ++ realPath d ++ " because it is not empty."
                                    else ioError $ userError $
-                                            "Not deleting " ++ ap2fp d ++ " because:\n" ++ show e)
+                                            "Not deleting " ++ realPath d ++ " because:\n" ++ show e)
     mRename a b = warning $ catch
                           (let do_backup = if map toLower x == map toLower y
-                                           then backupByCopying (ap2fp b) -- avoid making the original vanish
-                                           else backupByRenaming (ap2fp b)
+                                           then backupByCopying (realPath b) -- avoid making the original vanish
+                                           else backupByRenaming (realPath b)
                            in do_backup >> runDefaultIO (mRename a b))
                           (\e -> case () of
                                  _ | isPermissionError e -> ioError $ userError $
                                        couldNotRename ++ "."
                                    | isDoesNotExistError e -> ioError $ userError $
                                        couldNotRename ++ " because " ++ x ++ " does not exist."
+                                   | isAlreadyExistsError e -> ioError $ userError $
+                                       couldNotRename ++ " because " ++ y ++ " already exists."
                                    | otherwise -> ioError e
                           )
        where
-        x = ap2fp a
-        y = ap2fp b
+        x = realPath a
+        y = realPath b
         couldNotRename = "Could not rename " ++ x ++ " to " ++ y
 
 backup :: AnchoredPath -> IO ()
-backup f = backupByRenaming (ap2fp f)
+backup f = backupByRenaming (realPath f)
