@@ -1,79 +1,74 @@
 module Darcs.Repository.Clone
     ( cloneRepository
-    , replacePristine
     ) where
 
 import Darcs.Prelude
 
 import Control.Exception ( catch, SomeException )
-import Control.Monad ( unless, void, when )
+import Control.Monad ( forM, unless, void, when )
 import qualified Data.ByteString.Char8 as BC
 import Data.List( intercalate )
 import Data.Maybe( catMaybes )
-import System.FilePath( (</>) )
+import Safe ( tailErr )
+import System.FilePath.Posix ( (</>) )
 import System.Directory
     ( removeFile
     , listDirectory
     )
-import System.IO ( stderr )
 
 import Darcs.Repository.Create
     ( EmptyRepository(..)
     , createRepository
-    , writePristine
     )
-import Darcs.Repository.State ( invalidateIndex )
 import Darcs.Repository.Identify ( identifyRepositoryFor, ReadingOrWriting(..) )
 import Darcs.Repository.Pristine
-    ( ApplyDir(..)
-    , applyToTentativePristineCwd
+    ( applyToTentativePristine
     , createPristineDirectoryTree
+    , writePristine
     )
 import Darcs.Repository.Hashed
     ( copyHashedInventory
-    , finalizeRepositoryChanges
-    , finalizeTentativeChanges
-    , readRepo
-    , revertRepositoryChanges
-    , revertTentativeChanges
+    , readPatches
     , tentativelyRemovePatches
     , writeTentativeInventory
     )
+import Darcs.Repository.Transaction
+    ( finalizeRepositoryChanges
+    , revertRepositoryChanges
+    )
 import Darcs.Repository.Working
-    ( setScriptsExecutable
+    ( setAllScriptsExecutable
     , setScriptsExecutablePatches )
 import Darcs.Repository.InternalTypes
     ( Repository
+    , AccessType(..)
     , repoLocation
     , repoFormat
     , repoCache
     , modifyCache
     )
 import Darcs.Repository.Job ( withUMaskFlag )
-import Darcs.Repository.Cache
-    ( unionRemoteCaches
-    , unionCaches
+import Darcs.Util.Cache
+    ( filterRemoteCaches
     , fetchFileUsingCache
     , speculateFileUsingCache
-    , HashedDir(..)
-    , repo2cache
     , dropNonRepos
     )
 
 import Darcs.Repository.ApplyPatches ( runDefault )
 import Darcs.Repository.Inventory
-    ( peekPristineHash
-    , getValidHash
+    ( PatchHash
+    , encodeValidHash
+    , peekPristineHash
     )
 import Darcs.Repository.Format
     ( RepoProperty ( HashedInventory, Darcs2, Darcs3 )
     , RepoFormat
     , formatHas
-    , readProblem
     )
 import Darcs.Repository.Prefs ( addRepoSource, deleteSources )
 import Darcs.Repository.Match ( getOnePatchset )
-import Darcs.Util.External
+import Darcs.Util.File
     ( copyFileOrUrl
     , Cachable(..)
     , gzFetchFilePS
@@ -87,25 +82,24 @@ import Darcs.Repository.Packs
     , fetchAndUnpackPatches
     , packsDir
     )
+import Darcs.Repository.Paths ( hashedInventoryPath, pristineDirPath )
 import Darcs.Repository.Resolution
     ( StandardResolution(..)
     , patchsetConflictResolutions
     , announceConflicts
     )
 import Darcs.Repository.Working ( applyToWorking )
-import Darcs.Util.Lock ( appendTextFile, withNewDirectory )
+import Darcs.Util.Lock ( writeTextFile, withNewDirectory )
 import Darcs.Repository.Flags
     ( UpdatePending(..)
     , UseCache(..)
     , RemoteDarcs (..)
     , remoteDarcs
-    , Compression (..)
     , CloneKind (..)
     , Verbosity (..)
     , DryRun (..)
     , UMask (..)
     , SetScriptsExecutable (..)
-    , RemoteRepos (..)
     , SetDefault (..)
     , InheritDefault (..)
     , WithWorkingDir (..)
@@ -113,26 +107,29 @@ import Darcs.Repository.Flags
     , WithPatchIndex (..)
     , PatchFormat (..)
     , AllowConflicts(..)
-    , ExternalMerge(..)
+    , ResolveConflicts(..)
+    , WithPrefsTemplates(..)
     )
 
-import Darcs.Patch ( RepoPatch, IsRepoType, description )
+import Darcs.Patch ( RepoPatch, description )
 import Darcs.Patch.Depends ( findUncommon )
-import Darcs.Patch.Set ( patchSet2RL
-                       , patchSet2FL
-                       , progressPatchSet
-                       )
+import Darcs.Patch.Invertible ( mkInvertible )
+import Darcs.Patch.Set
+    ( Origin
+    , patchSet2FL
+    , patchSet2RL
+    , patchSetInventoryHashes
+    , progressPatchSet
+    )
 import Darcs.Patch.Match ( MatchFlag(..), patchSetMatch )
 import Darcs.Patch.Progress ( progressRLShowTags, progressFL )
 import Darcs.Patch.Apply ( Apply(..) )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.Patch.Witnesses.Ordered
-    ( FL(..)
+    ( (:\/:)(..)
+    , FL(..)
     , RL(..)
-    , (:\/:)(..)
     , lengthFL
-    , bunchFL
-    , mapFL
     , mapRL
     , lengthRL
     , nullFL
@@ -141,13 +138,13 @@ import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, extractHash )
 
 import Darcs.Util.Tree( Tree, emptyTree )
 
-import Darcs.Util.Download ( maxPipelineLength )
 import Darcs.Util.Exception ( catchall )
 import Darcs.Util.English ( englishNum, Noun(..) )
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.URL ( isValidLocalPath )
 import Darcs.Util.SignalHandler ( catchInterrupt, withSignalsBlocked )
-import Darcs.Util.Printer ( Doc, ($$), hPutDocLn, hsep, putDocLn, text )
+import Darcs.Util.Ssh ( resetSshConnections )
+import Darcs.Util.Printer ( Doc, ($$), hsep, putDocLn, text )
 import Darcs.Util.Printer.Color ( unsafeRenderStringColored )
 import Darcs.Util.Progress
     ( debugMessage
@@ -166,7 +163,6 @@ cloneRepository ::
     -> CloneKind
     -> UMask -> RemoteDarcs
     -> SetScriptsExecutable
-    -> RemoteRepos
     -> SetDefault
     -> InheritDefault
     -> [MatchFlag]
@@ -175,10 +171,11 @@ cloneRepository ::
     -> WithPatchIndex   -- use patch index
     -> Bool   -- use packs
     -> ForgetParent
+    -> WithPrefsTemplates
     -> IO ()
-cloneRepository repourl mysimplename v useCache cloneKind um rdarcs sse remoteRepos
+cloneRepository repourl mysimplename v useCache cloneKind um rdarcs sse
                 setDefault inheritDefault matchFlags rfsource withWorkingDir
-                usePatchIndex usePacks forget =
+                usePatchIndex usePacks forget withPrefsTemplates =
   withUMaskFlag um $ withNewDirectory mysimplename $ do
       let patchfmt
             | formatHas Darcs3 rfsource = PatchFormat3
@@ -186,30 +183,29 @@ cloneRepository repourl mysimplename v useCache cloneKind um rdarcs sse remoteRe
             | otherwise                 = PatchFormat1
       EmptyRepository _toRepo <-
         createRepository patchfmt withWorkingDir
-          (if cloneKind == LazyClone then NoPatchIndex else usePatchIndex) useCache
+          (if cloneKind == LazyClone then NoPatchIndex else usePatchIndex)
+          useCache withPrefsTemplates
       debugMessage "Finished initializing new repository."
-      addRepoSource repourl NoDryRun remoteRepos setDefault inheritDefault False
+      addRepoSource repourl NoDryRun setDefault inheritDefault False
 
-      debugMessage "Identifying and copying repository..."
+      debugMessage "Identifying remote repository..."
       fromRepo <- identifyRepositoryFor Reading _toRepo useCache repourl
       let fromLoc = repoLocation fromRepo
-      let rffrom = repoFormat fromRepo
-      case readProblem rffrom of
-        Just e ->  fail $ "Incompatibility with repository " ++ fromLoc ++ ":\n" ++ e
-        Nothing -> return ()
+
       debugMessage "Copying prefs..."
       copyFileOrUrl (remoteDarcs rdarcs)
         (joinUrl [fromLoc, darcsdir, "prefs", "prefs"])
         (darcsdir </> "prefs/prefs") (MaxAge 600) `catchall` return ()
-      debugMessage "Copying sources..."
-      cache <- unionRemoteCaches (repoCache _toRepo) (repoCache fromRepo) fromLoc
-      appendTextFile (darcsdir </> "prefs/sources")
-                     (show $ repo2cache fromLoc `unionCaches` dropNonRepos cache)
-      debugMessage "Done copying and filtering sources."
-      -- put remote source last
-      _toRepo <- return $
-        modifyCache (const $ cache `unionCaches` repo2cache fromLoc) _toRepo
-      if formatHas HashedInventory rffrom then do
+
+      debugMessage "Filtering remote sources..."
+      cache <- filterRemoteCaches (repoCache fromRepo)
+      _toRepo <- return $ modifyCache (const cache) _toRepo
+      writeTextFile
+        (darcsdir </> "prefs/sources")
+        (unlines [show $ dropNonRepos cache])
+      debugMessage $ "Considering sources:\n"++show (repoCache _toRepo)
+
+      if formatHas HashedInventory (repoFormat fromRepo) then do
        debugMessage "Copying basic repository (hashed_inventory and pristine)"
        if usePacks && (not . isValidLocalPath) fromLoc
          then copyBasicRepoPacked    fromRepo _toRepo v rdarcs withWorkingDir
@@ -222,17 +218,17 @@ cloneRepository repourl mysimplename v useCache cloneKind um rdarcs sse remoteRe
            then copyCompleteRepoPacked    fromRepo _toRepo v cloneKind
            else copyCompleteRepoNotPacked fromRepo _toRepo v cloneKind
       else
-       -- old-fashioned repositories are cloned diferently since
+       -- old-fashioned repositories are cloned differently since
        -- we need to copy all patches first and then build pristine
        copyRepoOldFashioned fromRepo _toRepo v withWorkingDir
-      when (sse == YesSetScriptsExecutable) setScriptsExecutable
+      when (sse == YesSetScriptsExecutable) setAllScriptsExecutable
       case patchSetMatch matchFlags of
        Nothing -> return ()
        Just psm -> do
         putInfo v $ text "Going to specified version..."
         -- the following is necessary to be able to read _toRepo's patches
-        _toRepo <- revertRepositoryChanges _toRepo NoUpdatePending
-        patches <- readRepo _toRepo
+        _toRepo <- revertRepositoryChanges _toRepo
+        patches <- readPatches _toRepo
         Sealed context <- getOnePatchset _toRepo psm
         to_remove :\/: only_in_context <- return $ findUncommon patches context
         case only_in_context of
@@ -243,11 +239,9 @@ cloneRepository repourl mysimplename v useCache cloneKind um rdarcs sse remoteRe
               , show num_to_remove
               , englishNum num_to_remove (Noun "patch") ""
               ]
-            invalidateIndex _toRepo
             _toRepo <-
-              tentativelyRemovePatches _toRepo GzipCompression NoUpdatePending to_remove
-            _toRepo <-
-              finalizeRepositoryChanges _toRepo NoUpdatePending GzipCompression
+              tentativelyRemovePatches _toRepo NoUpdatePending to_remove
+            _toRepo <- finalizeRepositoryChanges _toRepo NoDryRun
             runDefault (unapply to_remove) `catch` \(e :: SomeException) ->
                 fail ("Couldn't undo patch in working tree.\n" ++ show e)
             when (sse == YesSetScriptsExecutable) $ setScriptsExecutablePatches to_remove
@@ -259,24 +253,24 @@ cloneRepository repourl mysimplename v useCache cloneKind um rdarcs sse remoteRe
               $$ description only_in_context
       when (forget == YesForgetParent) deleteSources
       -- check for unresolved conflicts
-      patches <- readRepo _toRepo
+      patches <- readPatches _toRepo
       let conflicts = patchsetConflictResolutions patches
-      _ <- announceConflicts "clone" YesAllowConflictsAndMark NoExternalMerge conflicts
+      _ <- announceConflicts "clone" (YesAllowConflicts MarkConflicts) conflicts
       Sealed mangled_res <- return $ mangled conflicts
       unless (nullFL mangled_res) $
         withSignalsBlocked $ void $ applyToWorking _toRepo v mangled_res
 
 putInfo :: Verbosity -> Doc -> IO ()
 putInfo Quiet _ = return ()
-putInfo _ d = hPutDocLn stderr d
+putInfo _ d = putDocLn d
 
 putVerbose :: Verbosity -> Doc -> IO ()
 putVerbose Verbose d = putDocLn d
 putVerbose _ _ = return ()
 
-copyBasicRepoNotPacked  :: forall rt p wR wU wT.
-                           Repository rt p wR wU wT -- remote
-                        -> Repository rt p wR wU wT -- existing empty local
+copyBasicRepoNotPacked  :: forall p wU wR.
+                           Repository 'RO p wU wR -- remote
+                        -> Repository 'RO p wU wR -- existing empty local
                         -> Verbosity
                         -> RemoteDarcs
                         -> WithWorkingDir
@@ -287,9 +281,9 @@ copyBasicRepoNotPacked fromRepo toRepo verb rdarcs withWorkingDir = do
   putVerbose verb $ text "Writing pristine and working tree contents..."
   createPristineDirectoryTree toRepo "." withWorkingDir
 
-copyCompleteRepoNotPacked :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-                        => Repository rt p wR wU wT -- remote
-                        -> Repository rt p wR wU wT -- existing basic local
+copyCompleteRepoNotPacked :: forall rt p wU wR. (RepoPatch p, ApplyState p ~ Tree)
+                        => Repository 'RO p wU wR -- remote
+                        -> Repository rt p wU wR -- existing basic local
                         -> Verbosity
                         -> CloneKind
                         -> IO ()
@@ -298,13 +292,13 @@ copyCompleteRepoNotPacked _ toRepo verb cloneKind = do
        allowCtrlC cloneKind cleanup $ do
          fetchPatchesIfNecessary toRepo
          pi <- doesPatchIndexExist (repoLocation toRepo)
-         ps <- readRepo toRepo
+         ps <- readPatches toRepo
          when pi $ createPIWithInterrupt toRepo ps
 
 copyBasicRepoPacked ::
-  forall rt p wR wU wT.
-     Repository rt p wR wU wT -- remote
-  -> Repository rt p wR wU wT -- existing empty local repository
+  forall p wU wR.
+     Repository 'RO p wU wR -- remote
+  -> Repository 'RO p wU wR -- existing empty local repository
   -> Verbosity
   -> RemoteDarcs
   -> WithWorkingDir
@@ -313,9 +307,9 @@ copyBasicRepoPacked fromRepo toRepo verb rdarcs withWorkingDir =
   do let fromLoc = repoLocation fromRepo
      let hashURL = joinUrl [fromLoc, darcsdir, packsDir, "pristine"]
      mPackHash <- (Just <$> gzFetchFilePS hashURL Uncachable) `catchall` (return Nothing)
-     let hiURL = joinUrl [fromLoc, darcsdir, "hashed_inventory"]
+     let hiURL = fromLoc </> hashedInventoryPath
      i <- gzFetchFilePS hiURL Uncachable
-     let currentHash = BC.pack $ getValidHash $ peekPristineHash i
+     let currentHash = BC.pack $ encodeValidHash $ peekPristineHash i
      let copyNormally = copyBasicRepoNotPacked fromRepo toRepo verb rdarcs withWorkingDir
      case mPackHash of
       Just packHash | packHash == currentHash
@@ -331,25 +325,25 @@ copyBasicRepoPacked fromRepo toRepo verb rdarcs withWorkingDir =
                     copyNormally
 
 copyBasicRepoPacked2 ::
-  forall rt p wR wU wT.
-     Repository rt p wR wU wT -- remote
-  -> Repository rt p wR wU wT -- existing empty local repository
+  forall rt p wU wR.
+     Repository 'RO p wU wR -- remote
+  -> Repository rt p wU wR -- existing empty local repository
   -> Verbosity
   -> WithWorkingDir
   -> IO ()
 copyBasicRepoPacked2 fromRepo toRepo verb withWorkingDir = do
   putVerbose verb $ text "Cloning packed basic repository."
   -- unpack inventory & pristine cache
-  cleanDir $ darcsdir </> "pristine.hashed"
-  removeFile $ darcsdir </> "hashed_inventory"
+  cleanDir pristineDirPath
+  removeFile hashedInventoryPath
   fetchAndUnpackBasic (repoCache toRepo) (repoLocation fromRepo)
   putInfo verb $ text "Done fetching and unpacking basic pack."
   createPristineDirectoryTree toRepo "." withWorkingDir
 
 copyCompleteRepoPacked ::
-  forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-  => Repository rt p wR wU wT -- remote
-  -> Repository rt p wR wU wT -- existing basic local repository
+  forall rt p wU wR. (RepoPatch p, ApplyState p ~ Tree)
+  => Repository 'RO p wU wR -- remote
+  -> Repository rt p wU wR -- existing basic local repository
   -> Verbosity
   -> CloneKind
   -> IO ()
@@ -362,83 +356,83 @@ copyCompleteRepoPacked from to verb cloneKind =
       copyCompleteRepoNotPacked from to verb cloneKind
 
 copyCompleteRepoPacked2 ::
-  forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-  => Repository rt p wR wU wT
-  -> Repository rt p wR wU wT
+  forall rt p wU wR. (RepoPatch p, ApplyState p ~ Tree)
+  => Repository 'RO p wU wR
+  -> Repository rt p wU wR
   -> Verbosity
   -> CloneKind
   -> IO ()
 copyCompleteRepoPacked2 fromRepo toRepo verb cloneKind = do
-  us <- readRepo toRepo
+  us <- readPatches toRepo
   -- get old patches
   let cleanup = putInfo verb $ text "Using lazy repository."
   allowCtrlC cloneKind cleanup $ do
     putVerbose verb $ text "Using patches pack."
-    fetchAndUnpackPatches (mapRL hashedPatchFileName $ patchSet2RL us)
-      (repoCache toRepo) (repoLocation fromRepo)
+    is <-
+      forM (patchSetInventoryHashes us) $
+        maybe (fail "unexpected unhashed inventory") return
+    hs <-
+      forM (mapRL hashedPatchHash $ patchSet2RL us) $
+        maybe (fail "unexpected unhashed patch") return
+    fetchAndUnpackPatches is hs (repoCache toRepo) (repoLocation fromRepo)
     pi <- doesPatchIndexExist (repoLocation toRepo)
-    when pi $ createPIWithInterrupt toRepo us -- TODO or do another readRepo?
+    when pi $ createPIWithInterrupt toRepo us -- TODO or do another readPatches?
 
 cleanDir :: FilePath -> IO ()
 cleanDir d = mapM_ (\x -> removeFile $ d </> x) =<< listDirectory d
 
-copyRepoOldFashioned :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-                        => Repository rt p wR wU wT  -- remote repo
-                        -> Repository rt p wR wU wT  -- local empty repo
+copyRepoOldFashioned :: forall p wU wR. (RepoPatch p, ApplyState p ~ Tree)
+                        => Repository 'RO p wU wR  -- remote repo
+                        -> Repository 'RO p Origin Origin -- local empty repo
                         -> Verbosity
                         -> WithWorkingDir
                         -> IO ()
-copyRepoOldFashioned fromrepository _toRepo verb withWorkingDir = do
-  revertTentativeChanges
-  patches <- readRepo fromrepository
+copyRepoOldFashioned fromRepo _toRepo verb withWorkingDir = do
+  _toRepo <- revertRepositoryChanges _toRepo
+  _ <- writePristine _toRepo emptyTree
+  patches <- readPatches fromRepo
   let k = "Copying patch"
   beginTedious k
   tediousSize k (lengthRL $ patchSet2RL patches)
   let patches' = progressPatchSet k patches
-  writeTentativeInventory (repoCache _toRepo) GzipCompression patches'
+  writeTentativeInventory _toRepo patches'
   endTedious k
-  finalizeTentativeChanges _toRepo GzipCompression
-  -- apply all patches into current hashed repository
-  _toRepo <- revertRepositoryChanges _toRepo NoUpdatePending
-  local_patches <- readRepo _toRepo
-  replacePristine _toRepo emptyTree
+  local_patches <- readPatches _toRepo
   let patchesToApply = progressFL "Applying patch" $ patchSet2FL local_patches
-  sequence_ $ mapFL (applyToTentativePristineCwd ApplyNormal) $ bunchFL 100 patchesToApply
-  _toRepo <- finalizeRepositoryChanges _toRepo NoUpdatePending GzipCompression
-  putVerbose verb $ text "Writing pristine and working tree contents..."
+  applyToTentativePristine _toRepo (mkInvertible patchesToApply)
+  _toRepo <- finalizeRepositoryChanges _toRepo NoDryRun
+  putVerbose verb $ text "Writing the working tree..."
   createPristineDirectoryTree _toRepo "." withWorkingDir
 
 -- | This function fetches all patches that the given repository has
 --   with fetchFileUsingCache.
-fetchPatchesIfNecessary :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p)
-                        => Repository rt p wR wU wT
+fetchPatchesIfNecessary :: forall rt p wU wR. RepoPatch p
+                        => Repository rt p wU wR
                         -> IO ()
 fetchPatchesIfNecessary toRepo =
-  do  ps <- readRepo toRepo
-      pipelineLength <- maxPipelineLength
+  do  ps <- readPatches toRepo
       let patches = patchSet2RL ps
           ppatches = progressRLShowTags "Copying patches" patches
-          (first, other) = splitAt (pipelineLength - 1) $ tail $ hashes patches
-          speculate | pipelineLength > 1 = [] : first : map (:[]) other
-                    | otherwise = []
+          (first, other) = splitAt (100 - 1) $ tailErr $ hashes patches
+          speculate = [] : first : map (:[]) other
       mapM_ fetchAndSpeculate $ zip (hashes ppatches) (speculate ++ repeat [])
-  where hashes :: forall wX wY . RL (PatchInfoAnd rt p) wX wY -> [String]
-        hashes = catMaybes . mapRL (either (const Nothing) Just . extractHash)
-        fetchAndSpeculate :: (String, [String]) -> IO ()
+  where hashes :: forall wX wY . RL (PatchInfoAnd p) wX wY -> [PatchHash]
+        hashes = catMaybes . mapRL hashedPatchHash
+        fetchAndSpeculate :: (PatchHash, [PatchHash]) -> IO ()
         fetchAndSpeculate (f, ss) = do
-          _ <- fetchFileUsingCache c HashedPatchesDir f
-          mapM_ (speculateFileUsingCache c HashedPatchesDir) ss
+          _ <- fetchFileUsingCache c f
+          mapM_ (speculateFileUsingCache c) ss
         c = repoCache toRepo
 
--- | Replace the existing pristine with a new one (loaded up in a Tree object).
-replacePristine :: Repository rt p wR wU wT -> Tree IO -> IO ()
-replacePristine = writePristine . repoLocation
-
 allowCtrlC :: CloneKind -> IO () -> IO () -> IO ()
-allowCtrlC CompleteClone _       action = action
-allowCtrlC _             cleanup action = action `catchInterrupt` cleanup
+allowCtrlC CompleteClone _ action = action
+allowCtrlC _ cleanup action =
+  action `catchInterrupt` do
+    debugMessage "Cleanup after SIGINT in allowCtrlC"
+    -- the SIGINT has also killed our running ssh connections,
+    -- this will cause them to be restarted
+    resetSshConnections
+    cleanup
 
-hashedPatchFileName :: PatchInfoAnd rt p wA wB -> String
-hashedPatchFileName x = case extractHash x of
-  Left _ -> fail "unexpected unhashed patch"
-  Right h -> h
+hashedPatchHash :: PatchInfoAnd p wA wB -> Maybe PatchHash
+hashedPatchHash = either (const Nothing) Just . extractHash
