@@ -1,7 +1,5 @@
 module Darcs.Repository.Flags
-    (
-      Compression (..)
-    , RemoteDarcs (..)
+    ( RemoteDarcs (..)
     , remoteDarcs
     , Reorder (..)
     , Verbosity (..)
@@ -13,26 +11,24 @@ module Darcs.Repository.Flags
     , LookForReplaces (..)
     , DiffAlgorithm (..)
     , LookForMoves (..)
+    , DiffOpts (..)
     , RunTest (..)
     , SetScriptsExecutable (..)
     , LeaveTestDir (..)
-    , RemoteRepos (..)
     , SetDefault (..)
     , InheritDefault (..)
     , UseIndex (..)
-    , ScanKnown (..)
     , CloneKind (..)
     , AllowConflicts (..)
-    , ExternalMerge (..)
+    , ResolveConflicts (..)
     , WorkRepo (..)
     , WantGuiPause (..)
     , WithPatchIndex (..)
     , WithWorkingDir (..)
     , ForgetParent (..)
     , PatchFormat (..)
-    , IncludeBoring (..)
-    , HooksConfig (..)
-    , HookConfig (..)
+    , WithPrefsTemplates (..)
+    , OptimizeDeep (..)
     ) where
 
 import Darcs.Prelude
@@ -40,14 +36,9 @@ import Darcs.Prelude
 import Darcs.Util.Diff ( DiffAlgorithm(..) )
 import Darcs.Util.Global ( defaultRemoteDarcsCmd )
 
-
 data Verbosity = Quiet | NormalVerbosity | Verbose
     deriving ( Eq, Show )
 
-data Compression = NoCompression
-                 | GzipCompression
-    deriving ( Eq, Show )
-
 data WithPatchIndex = YesPatchIndex | NoPatchIndex
     deriving ( Eq, Show )
 
@@ -74,7 +65,7 @@ data DryRun = YesDryRun | NoDryRun
 data UMask = YesUMask String | NoUMask
     deriving ( Eq, Show )
 
-data LookForAdds = YesLookForAdds | NoLookForAdds
+data LookForAdds = NoLookForAdds | YesLookForAdds | EvenLookForBoring
     deriving ( Eq, Show )
 
 data LookForReplaces = YesLookForReplaces | NoLookForReplaces
@@ -83,8 +74,13 @@ data LookForReplaces = YesLookForReplaces | NoLookForReplaces
 data LookForMoves = YesLookForMoves | NoLookForMoves
     deriving ( Eq, Show )
 
-data IncludeBoring = YesIncludeBoring | NoIncludeBoring
-    deriving ( Eq, Show )
+data DiffOpts = DiffOpts
+  { withIndex :: UseIndex
+  , lookForAdds :: LookForAdds
+  , lookForReplaces :: LookForReplaces
+  , lookForMoves :: LookForMoves
+  , diffAlg :: DiffAlgorithm
+  } deriving Show
 
 data RunTest = YesRunTest | NoRunTest
     deriving ( Eq, Show )
@@ -95,9 +91,6 @@ data SetScriptsExecutable = YesSetScriptsExecutable | NoSetScriptsExecutable
 data LeaveTestDir = YesLeaveTestDir | NoLeaveTestDir
     deriving ( Eq, Show )
 
-data RemoteRepos = RemoteRepos [String]
-    deriving ( Eq, Show )
-
 data SetDefault = YesSetDefault Bool | NoSetDefault Bool
     deriving ( Eq, Show )
 
@@ -106,21 +99,16 @@ data InheritDefault = YesInheritDefault | NoInheritDefault
 
 data UseIndex = UseIndex | IgnoreIndex deriving ( Eq, Show )
 
-data ScanKnown = ScanKnown -- ^Just files already known to darcs
-               | ScanAll -- ^All files, i.e. look for new ones
-               | ScanBoring -- ^All files, even boring ones
-    deriving ( Eq, Show )
-
 -- Various kinds of getting repositories
 data CloneKind = LazyClone       -- ^Just copy pristine and inventories
                | NormalClone     -- ^First do a lazy clone then copy everything
                | CompleteClone   -- ^Same as Normal but omit telling user they can interrumpt
     deriving ( Eq, Show )
 
-data AllowConflicts = NoAllowConflicts | YesAllowConflicts | YesAllowConflictsAndMark
+data AllowConflicts = NoAllowConflicts | YesAllowConflicts ResolveConflicts
     deriving ( Eq, Show )
 
-data ExternalMerge = YesExternalMerge String | NoExternalMerge
+data ResolveConflicts = NoResolveConflicts | MarkConflicts | ExternalMerge String
     deriving ( Eq, Show )
 
 data WorkRepo = WorkRepoDir String | WorkRepoPossibleURL String | WorkRepoCurrentDir
@@ -138,12 +126,8 @@ data ForgetParent = YesForgetParent | NoForgetParent
 data PatchFormat = PatchFormat1 | PatchFormat2 | PatchFormat3
     deriving ( Eq, Show )
 
-data HooksConfig = HooksConfig
-  { pre :: HookConfig
-  , post :: HookConfig
-  }
+data WithPrefsTemplates =  WithPrefsTemplates | NoPrefsTemplates
+    deriving ( Eq, Show )
 
-data HookConfig = HookConfig
-  { cmd :: Maybe String
-  , prompt :: Bool
-  }
+data OptimizeDeep = OptimizeShallow | OptimizeDeep
+    deriving ( Eq, Show )
