@@ -27,14 +27,14 @@ module Darcs.Repository.State
     -- * Diffs
     , unrecordedChanges
     -- * Trees
-    , readRecorded, readUnrecorded, readRecordedAndPending, readWorking
+    , readPristine, readUnrecorded, readPristineAndPending, readWorking
     , readPendingAndWorking, readUnrecordedFiltered
     -- * Index
-    , readIndex, updateIndex, invalidateIndex, UseIndex(..), ScanKnown(..)
+    , readIndex, updateIndex
     -- * Utilities
     , filterOutConflicts
     -- * Pending-related functions that depend on repo state
-    , addPendingDiffToPending, addToPending
+    , unsafeAddToPending, addToPending
     ) where
 
 import Darcs.Prelude
@@ -45,25 +45,21 @@ import Control.Exception ( catch, IOException )
 import Data.Ord ( comparing )
 import Data.List ( sortBy, union, delete )
 
-import System.Directory( doesFileExist, doesDirectoryExist, renameFile )
-import System.FilePath ( (<.>), (</>) )
-import System.IO ( hPutStrLn, stderr )
-import System.IO.Error ( catchIOError )
+import System.Directory( doesFileExist, renameFile )
+import System.FilePath ( (<.>) )
 
-import qualified Data.ByteString as B
-    ( ByteString, readFile, writeFile, empty, concat )
-import qualified Data.ByteString.Char8 as BC
-    ( pack, unpack )
+import qualified Data.ByteString as B ( ByteString, concat )
+import qualified Data.ByteString.Char8 as BC ( pack, unpack )
 import qualified Data.ByteString.Lazy as BL ( toChunks )
 
-import Darcs.Patch ( RepoPatch, PrimOf, sortCoalesceFL
+import Darcs.Patch ( RepoPatch, PrimOf, canonizeFL
                    , PrimPatch, maybeApplyToTree
                    , tokreplace, forceTokReplace, move )
 import Darcs.Patch.Named ( anonymous )
 import Darcs.Patch.Apply ( ApplyState, applyToTree, effectOnPaths )
-import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+)
+import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+), consGapFL
                                      , (:>)(..), reverseRL, reverseFL
-                                     , mapFL, concatFL, toFL, nullFL )
+                                     , mapFL, concatFL, joinGapsFL, nullFL )
 import Darcs.Patch.Witnesses.Eq ( EqCheck(IsEq, NotEq) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePEnd )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal, unFreeLeft, mapSeal
@@ -74,22 +70,27 @@ import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia )
 import Darcs.Patch.FileHunk ( FileHunk(..), IsHunk(..) )
 import Darcs.Patch.TokenReplace ( breakToTokens, defaultToks )
 
-import Darcs.Repository.Flags ( UseIndex(..), ScanKnown(..), DiffAlgorithm(..)
-                              , UpdatePending(..), LookForMoves(..), LookForReplaces(..) )
-
-import Darcs.Repository.InternalTypes ( Repository, repoFormat, repoLocation )
+import Darcs.Repository.Flags
+    ( DiffAlgorithm(..)
+    , LookForMoves(..)
+    , LookForReplaces(..)
+    , LookForAdds(..)
+    , UseIndex(..)
+    , DiffOpts(..)
+    )
+import Darcs.Repository.InternalTypes
+    ( AccessType(..)
+    , Repository
+    , repoFormat
+    , repoLocation
+    )
 import Darcs.Repository.Format(formatHas, RepoProperty(NoWorkingDir))
 import qualified Darcs.Repository.Pending as Pending
 import Darcs.Repository.Prefs ( filetypeFunction, isBoring )
+import Darcs.Repository.Pristine ( readPristine )
 import Darcs.Repository.Diff ( treeDiff )
-import Darcs.Repository.Inventory ( peekPristineHash, getValidHash )
 import Darcs.Repository.Paths
-    ( pristineDirPath
-    , hashedInventoryPath
-    , oldPristineDirPath
-    , oldCurrentDirPath
-    , patchesDirPath
-    , indexPath
+    ( indexPath
     , indexInvalidPath
     )
 
@@ -103,13 +104,10 @@ import Darcs.Util.Path
     , parents
     , movedirfilename
     )
-import Darcs.Util.Hash( Hash( NoHash ) )
 import Darcs.Util.Tree( Tree, restrict, FilterTree, expand, emptyTree, overlay, find
                       , ItemType(..), itemType, readBlob, modifyTree, findFile, TreeItem(..)
                       , makeBlobBS, expandPath )
 import qualified Darcs.Util.Tree.Plain as PlainTree ( readPlainTree )
-import Darcs.Util.Tree.Hashed
-    ( darcsTreeHash, readDarcsHashed, decodeDarcsHash, decodeDarcsSize )
 import Darcs.Util.Index
     ( Index
     , indexFormatValid
@@ -126,6 +124,9 @@ import Darcs.Util.Index ( listFileIDs, getFileID )
 import Control.Monad ( unless )
 import Darcs.Util.Path ( displayPath )
 import Darcs.Util.Tree ( list )
+#else
+import System.IO ( hPutStrLn, stderr )
+import System.IO.Error ( catchIOError )
 #endif
 
 newtype TreeFilter m = TreeFilter { applyTreeFilter :: forall tr . FilterTree tr m => tr m -> tr m }
@@ -135,7 +136,7 @@ newtype TreeFilter m = TreeFilter { applyTreeFilter :: forall tr . FilterTree tr
 -- constructed filter will take pending into account, so the subpaths will be
 -- translated correctly relative to pending move patches.
 restrictSubpaths :: (RepoPatch p, ApplyState p ~ Tree)
-                 => Repository rt p wR wU wT -> [AnchoredPath]
+                 => Repository rt p wU wR -> [AnchoredPath]
                  -> IO (TreeFilter m)
 restrictSubpaths repo paths = do
   Sealed pending <- Pending.readPending repo
@@ -146,7 +147,7 @@ restrictSubpaths repo paths = do
 -- abiguous typing of @p@.
 restrictSubpathsAfter :: (RepoPatch p, ApplyState p ~ Tree)
                       => FL (PrimOf p) wR wP
-                      -> Repository rt p wR wU wT
+                      -> Repository rt p wU wR
                       -> [AnchoredPath]
                       -> IO (TreeFilter m)
 restrictSubpathsAfter pending _repo paths = do
@@ -158,7 +159,7 @@ restrictSubpathsAfter pending _repo paths = do
 -- note we assume pending starts at the recorded state
 maybeRestrictSubpaths :: (RepoPatch p, ApplyState p ~ Tree)
                       => FL (PrimOf p) wR wP
-                      -> Repository rt p wR wU wT
+                      -> Repository rt p wU wR
                       -> Maybe [AnchoredPath]
                       -> IO (TreeFilter m)
 maybeRestrictSubpaths pending repo =
@@ -173,12 +174,14 @@ maybeRestrictSubpaths pending repo =
 restrictBoring :: Tree m -> IO (TreeFilter m)
 restrictBoring guide = do
   boring <- isBoring
-  let exclude p = inDarcsdir p || boring (realPath p)
+  let exclude p t = inDarcsdir p || boring (appendSlash t (realPath p))
+      appendSlash TreeType fp = fp ++ "/"
+      appendSlash BlobType fp = fp
       restrictTree :: FilterTree t m => t m -> t m
       restrictTree =
-        Tree.filter $ \p _ ->
+        Tree.filter $ \p i ->
           case find guide p of
-            Nothing -> not (exclude p)
+            Nothing -> not (exclude p (itemType i))
             _ -> True
   return (TreeFilter restrictTree)
 
@@ -223,38 +226,34 @@ This also depends on the options given:
   replaces, since this implies that the index is out of date.
 -}
 unrecordedChanges :: (RepoPatch p, ApplyState p ~ Tree)
-                  => (UseIndex, ScanKnown, DiffAlgorithm)
-                  -> LookForMoves
-                  -> LookForReplaces
-                  -> Repository rt p wR wU wR
+                  => DiffOpts
+                  -> Repository rt p wU wR
                   -> Maybe [AnchoredPath] -> IO (FL (PrimOf p) wR wU)
-unrecordedChanges dopts lfm lfr r paths = do
-  (pending :> working) <- readPendingAndWorking dopts lfm lfr r paths
-  return $ sortCoalesceFL (pending +>+ working)
+unrecordedChanges dopts@DiffOpts{..} r paths = do
+  (pending :> working) <- readPendingAndWorking dopts r paths
+  return $ canonizeFL diffAlg (pending +>+ working)
 
 -- Implementation note: it is important to do things in the right order: we
 -- first have to read the pending patch, then detect moves, then detect adds,
 -- then detect replaces.
 readPendingAndWorking :: (RepoPatch p, ApplyState p ~ Tree)
-                      => (UseIndex, ScanKnown, DiffAlgorithm)
-                      -> LookForMoves
-                      -> LookForReplaces
-                      -> Repository rt p wR wU wR
+                      => DiffOpts
+                      -> Repository rt p wU wR
                       -> Maybe [AnchoredPath]
                       -> IO ((FL (PrimOf p) :> FL (PrimOf p)) wR wU)
-readPendingAndWorking _ _ _ r _ | formatHas NoWorkingDir (repoFormat r) = do
+readPendingAndWorking _ r _ | formatHas NoWorkingDir (repoFormat r) = do
   IsEq <- return $ workDirLessRepoWitness r
   return (NilFL :> NilFL)
-readPendingAndWorking (useidx, scan, diffalg) lfm lfr repo mbpaths = do
+readPendingAndWorking DiffOpts{..} repo mbpaths = do
   debugMessage "readPendingAndWorking: start"
   (pending_tree, working_tree, (pending :> moves)) <-
-    readPendingAndMovesAndUnrecorded repo useidx scan lfm mbpaths
+    readPendingAndMovesAndUnrecorded repo withIndex lookForAdds lookForMoves mbpaths
   debugMessage "readPendingAndWorking: after readPendingAndMovesAndUnrecorded"
   (pending_tree_with_replaces, Sealed replaces) <-
-    getReplaces lfr diffalg repo pending_tree working_tree
+    getReplaces lookForReplaces diffAlg repo pending_tree working_tree
   debugMessage "readPendingAndWorking: after getReplaces"
   ft <- filetypeFunction
-  wrapped_diff <- treeDiff diffalg ft pending_tree_with_replaces working_tree
+  wrapped_diff <- treeDiff diffAlg ft pending_tree_with_replaces working_tree
   case unFreeLeft wrapped_diff of
     Sealed diff -> do
       debugMessage "readPendingAndWorking: done"
@@ -262,14 +261,14 @@ readPendingAndWorking (useidx, scan, diffalg) lfm lfr repo mbpaths = do
 
 readPendingAndMovesAndUnrecorded
   :: (RepoPatch p, ApplyState p ~ Tree)
-  => Repository rt p wR wU wR
+  => Repository rt p wU wR
   -> UseIndex
-  -> ScanKnown
+  -> LookForAdds
   -> LookForMoves
   -> Maybe [AnchoredPath]
   -> IO ( Tree IO             -- pristine with (pending + moves)
         , Tree IO             -- working
-        , (FL (PrimOf p) :> FL (PrimOf p)) wR wU -- pending :> moves
+        , (FL (PrimOf p) :> FL (PrimOf p)) wR wM -- pending :> moves
         )
 readPendingAndMovesAndUnrecorded repo useidx scan lfm mbpaths = do
   debugMessage "readPendingAndMovesAndUnrecorded: start"
@@ -295,70 +294,50 @@ readPendingAndMovesAndUnrecorded repo useidx scan lfm mbpaths = do
   return
     (pending_tree_with_moves, working_tree, unsafeCoercePEnd (pending :> moves))
 
--- | @filteredWorking useidx scan relevant index pending_tree@ reads the
+-- | @filteredWorking useidx scan relevant from_index pending_tree@ reads the
 -- working tree and filters it according to options and @relevant@ file paths.
 -- The @pending_tree@ is understood to have @relevant@ already applied and is
--- used (only) if @useidx == 'IgnoreIndex'@ and @scan /= 'ScanBoring'@ to act as
+-- used (only) if @useidx == 'IgnoreIndex'@ and @scan /= 'EvenLookForBoring'@ to act as
 -- a guide for filtering the working tree.
-filteredWorking :: Repository rt p wR wU wR
+filteredWorking :: Repository rt p wU wR
                 -> UseIndex
-                -> ScanKnown
+                -> LookForAdds
                 -> TreeFilter IO
                 -> Tree IO
                 -> Tree IO
                 -> IO (Tree IO)
-filteredWorking repo useidx scan relevant index pending_tree =
-  applyTreeFilter restrictDarcsdir <$> applyTreeFilter relevant <$> do
+filteredWorking repo useidx scan relevant from_index pending_tree =
+  applyTreeFilter restrictDarcsdir <$> applyTreeFilter relevant <$>
     case useidx of
       UseIndex ->
         case scan of
-          ScanKnown -> return index
-          ScanAll -> do
-            nonboring <- restrictBoring index
+          NoLookForAdds -> return from_index
+          YesLookForAdds -> do
+            nonboring <- restrictBoring from_index
             plain <- applyTreeFilter nonboring <$> readPlainTree repo
-            return $ plain `overlay` index
-          ScanBoring -> do
+            return $ plain `overlay` from_index
+          EvenLookForBoring -> do
             plain <- readPlainTree repo
-            return $ plain `overlay` index
-      IgnoreIndex ->
+            return $ plain `overlay` from_index
+      IgnoreIndex -> do
+        working <- readPlainTree repo
         case scan of
-          ScanKnown -> do
+          NoLookForAdds -> do
             guide <- expand pending_tree
-            restrict guide <$> readPlainTree repo
-          ScanAll -> do
+            return $ restrict guide working
+          YesLookForAdds -> do
             guide <- expand pending_tree
             nonboring <- restrictBoring guide
-            applyTreeFilter nonboring <$> readPlainTree repo
-          ScanBoring -> readPlainTree repo
+            return $ applyTreeFilter nonboring working
+          EvenLookForBoring -> return working
 
--- | Witnesses the fact that in the absence of a working tree, we
--- pretend that the working dir updates magically to the tentative state.
-workDirLessRepoWitness :: Repository rt p wR wU wT -> EqCheck wU wT
+-- | Witnesses the fact that in the absence of a working tree, the
+-- unrecorded state cannot differ from the record state.
+workDirLessRepoWitness :: Repository rt p wU wR -> EqCheck wU wR
 workDirLessRepoWitness r
  | formatHas NoWorkingDir (repoFormat r) = unsafeCoerceP IsEq
  | otherwise                             = NotEq
 
--- | Obtains a Tree corresponding to the "recorded" state of the repository:
--- this is the same as the pristine cache, which is the same as the result of
--- applying all the repository's patches to an empty directory.
-readRecorded :: Repository rt p wR wU wT -> IO (Tree IO)
-readRecorded _repo = do
-  hashed <- doesFileExist hashedInventoryPath
-  if hashed
-     then do inv <- B.readFile hashedInventoryPath
-             let pris = peekPristineHash inv
-                 hash = decodeDarcsHash $ BC.pack $ getValidHash pris
-                 size = decodeDarcsSize $ BC.pack $ getValidHash pris
-             when (hash == NoHash) $
-                 fail $ "Bad pristine root: " ++ getValidHash pris
-             readDarcsHashed pristineDirPath (size, hash)
-     else do have_pristine <- doesDirectoryExist $ oldPristineDirPath
-             have_current <- doesDirectoryExist $ oldCurrentDirPath
-             case (have_pristine, have_current) of
-               (True, _) -> PlainTree.readPlainTree $ oldPristineDirPath
-               (False, True) -> PlainTree.readPlainTree $ oldCurrentDirPath
-               (_, _) -> fail "No pristine tree is available!"
-
 -- | Obtains a Tree corresponding to the "unrecorded" state of the repository:
 -- the modified files of the working tree plus the "pending" patch.
 -- The optional list of paths allows to restrict the query to a subtree.
@@ -367,20 +346,20 @@ readRecorded _repo = do
 -- parts of the index do not need to go through an up-to-date check (which
 -- involves a relatively expensive lstat(2) per file.
 readUnrecorded :: (RepoPatch p, ApplyState p ~ Tree)
-               => Repository rt p wR wU wR
+               => Repository rt p wU wR
                -> UseIndex
                -> Maybe [AnchoredPath]
                -> IO (Tree IO)
 readUnrecorded repo useidx mbpaths = do
 #if TEST_INDEX
-  t1 <- expand =<< readUnrecordedFiltered repo useidx ScanKnown NoLookForMoves mbpaths
+  t1 <- expand =<< readUnrecordedFiltered repo useidx NoLookForAdds NoLookForMoves mbpaths
   (pending_tree, Sealed pending) <- readPending repo
   relevant <- maybeRestrictSubpaths pending repo mbpaths
   t2 <- readIndexOrPlainTree repo useidx relevant pending_tree
   assertEqualTrees "indirect" t1 "direct" t2
   return t1
 #else
-  expand =<< readUnrecordedFiltered repo useidx ScanKnown NoLookForMoves mbpaths
+  expand =<< readUnrecordedFiltered repo useidx NoLookForAdds NoLookForMoves mbpaths
 #endif
 
 #if TEST_INDEX
@@ -397,7 +376,7 @@ showTree name tree = unlines (name : map (("  "++) . displayPath . fst) (list tr
 #endif
 
 readIndexOrPlainTree :: (ApplyState p ~ Tree, RepoPatch p)
-                     => Repository rt p wR wU wR
+                     => Repository rt p wU wR
                      -> UseIndex
                      -> TreeFilter IO
                      -> Tree IO
@@ -425,14 +404,14 @@ readIndexOrPlainTree repo IgnoreIndex treeFilter pending_tree = do
   expand =<< applyTreeFilter treeFilter . restrict guide <$> readPlainTree repo
 #endif
 
--- | A variant of 'readUnrecorded' that takes the UseIndex and ScanKnown
+-- | A variant of 'readUnrecorded' that takes the UseIndex and LookForAdds
 -- options into account, similar to 'readPendingAndWorking'. We are only
 -- interested in the resulting tree, not the patch, so the 'DiffAlgorithm' option
 -- is irrelevant.
 readUnrecordedFiltered :: (RepoPatch p, ApplyState p ~ Tree)
-                       => Repository rt p wR wU wR
+                       => Repository rt p wU wR
                        -> UseIndex
-                       -> ScanKnown
+                       -> LookForAdds
                        -> LookForMoves
                        -> Maybe [AnchoredPath] -> IO (Tree IO)
 readUnrecordedFiltered repo useidx scan lfm mbpaths = do
@@ -448,60 +427,55 @@ readWorking relevant =
    PlainTree.readPlainTree ".")
 
 -- | Obtains the recorded 'Tree' with the pending patch applied.
-readRecordedAndPending :: (RepoPatch p, ApplyState p ~ Tree)
-                       => Repository rt p wR wU wR -> IO (Tree IO)
-readRecordedAndPending repo = fst `fmap` readPending repo
+readPristineAndPending :: (RepoPatch p, ApplyState p ~ Tree)
+                       => Repository rt p wU wR -> IO (Tree IO)
+readPristineAndPending repo = fst `fmap` readPending repo
 
 -- | Obtains the recorded 'Tree' with the pending patch applied, plus
 --   the pending patch itself. The pending patch should start at the
 --   recorded state (we even verify that it applies, and degrade to
 --   renaming pending and starting afresh if it doesn't).
 readPending :: (RepoPatch p, ApplyState p ~ Tree)
-            => Repository rt p wR wU wR
+            => Repository rt p wU wR
             -> IO (Tree IO, Sealed (FL (PrimOf p) wR))
 readPending repo = do
-  pristine <- readRecorded repo
+  pristine <- readPristine repo
   Sealed pending <- Pending.readPending repo
-  catch ((\t -> (t, seal pending)) <$> applyToTree pending pristine) $
-    \(err :: IOException) -> do
-       putStrLn $ "Yikes, pending has conflicts! " ++ show err
-       putStrLn "Stashing the buggy pending as _darcs/patches/pending_buggy"
-       renameFile (patchesDirPath </> "pending")
-                  (patchesDirPath </> "pending_buggy")
-       return (pristine, seal NilFL)
-
--- | Mark the existing index as invalid. This has to be called whenever the
--- listing of pristine+pending changes and will cause darcs to update the index.
--- This will happen either when we call updateIndex in finalizeRepositoryChanges
--- or else when we try to read the index the next time.
--- (NB. This is about files added and removed from
--- pristine: changes to file content in either pristine or working are handled
--- transparently by the index reading code.)
-invalidateIndex :: t -> IO ()
-invalidateIndex _ = B.writeFile indexInvalidPath B.empty
+  catch ((\t -> (t, seal pending)) <$> applyToTree pending pristine) $ \(e::IOException) -> do
+    fail $
+      "Cannot apply pending patch, please run `darcs repair`\n"
+      ++ show e
 
+-- | Open the index or re-create it in case it is invalid or non-existing.
 readIndex :: (RepoPatch p, ApplyState p ~ Tree)
-          => Repository rt p wR wU wR -> IO Index
+          => Repository rt p wU wR -> IO Index
 readIndex repo = do
   okay <- checkIndex
   if not okay
     then internalUpdateIndex repo
-    else openIndex indexPath darcsTreeHash
+    else openIndex indexPath
 
 -- | Update the index so that it matches pristine+pending. If the index does
 -- not exist or is invalid, create a new one. Returns the updated index.
 internalUpdateIndex :: (RepoPatch p, ApplyState p ~ Tree)
-            => Repository rt p wR wU wR -> IO Index
+            => Repository rt p wU wR -> IO Index
 internalUpdateIndex repo = do
-  pris <- readRecordedAndPending repo
-  idx <- updateIndexFrom indexPath darcsTreeHash pris
+  pris <-
+    readPristineAndPending repo
+    `catch` \(_::IOException) -> readPristine repo
+  idx <- updateIndexFrom indexPath pris
   removeFileMayNotExist indexInvalidPath
   return idx
 
 -- | Update the index so that it matches pristine+pending. If the index does
 -- not exist or is invalid, create a new one.
+--
+-- This has to be called whenever the listing of pristine+pending changes. Note
+-- that this only concerns files added and removed or renamed: changes to file
+-- content in either pristine or working are handled transparently by the index
+-- reading code.
 updateIndex :: (RepoPatch p, ApplyState p ~ Tree)
-            => Repository rt p wR wU wR -> IO ()
+            => Repository rt p wU wR -> IO ()
 updateIndex repo = do
   -- call checkIndex to throw away the index if it is invalid;
   -- this can happen if we are called with --ignore-times
@@ -510,8 +484,9 @@ updateIndex repo = do
   void $ internalUpdateIndex repo
 
 -- | Check if we have a valid index. This means that the index file exists, is
--- readable, and can be mmapped, /and/ that indexInvalidPath does /not/ exist.
--- We do not yet remove indexInvalidPath in case updating the index fails.
+-- readable, and can be mmapped. For compatibility with older darcs versions we
+-- also check that indexInvalidPath does not exist. We do not yet remove
+-- indexInvalidPath in case updating the index fails.
 checkIndex :: IO Bool
 checkIndex = do
   invalid <- doesFileExist $ indexInvalidPath
@@ -526,19 +501,22 @@ checkIndex = do
 -- conflict with the recorded or unrecorded changes in a repo
 filterOutConflicts
   :: (RepoPatch p, ApplyState p ~ Tree)
-  => Repository rt p wR wU wR     -- ^Repository itself, used for grabbing
+  => Repository rt p wU wR     -- ^Repository itself, used for grabbing
                                   --  unrecorded changes
-  -> FL (PatchInfoAnd rt p) wX wR -- ^Recorded patches from repository, starting from
+  -> UseIndex                     -- ^Whether to use the index when reading
+                                  --  the working state
+  -> FL (PatchInfoAnd p) wX wR -- ^Recorded patches from repository, starting from
                                   --  same context as the patches to filter
-  -> FL (PatchInfoAnd rt p) wX wZ -- ^Patches to filter
-  -> IO (Bool, Sealed (FL (PatchInfoAnd rt p) wX))
+  -> FL (PatchInfoAnd p) wX wZ -- ^Patches to filter
+  -> IO (Bool, Sealed (FL (PatchInfoAnd p) wX))
                                   -- ^True iff any patches were removed,
                                   --  possibly filtered patches
-filterOutConflicts repository us them
+filterOutConflicts repository useidx us them
      = do -- Note: use of anonymous is benign here since we only try to merge cleanly
           unrec <- fmap n2pia . anonymous
-                     =<< unrecordedChanges (UseIndex, ScanKnown, MyersDiff)
-                          NoLookForMoves NoLookForReplaces repository Nothing
+                     =<< unrecordedChanges
+                          (DiffOpts useidx NoLookForAdds NoLookForReplaces
+                          NoLookForMoves MyersDiff) repository Nothing
           them' :> rest <-
             return $ partitionConflictingFL them (us +>+ unrec :>: NilFL)
           return (check rest, Sealed them')
@@ -548,10 +526,10 @@ filterOutConflicts repository us them
 
 -- | Automatically detect file moves using the index.
 -- TODO: This function lies about the witnesses.
-getMoves :: forall rt p wR wU wB prim.
+getMoves :: forall rt p wU wR wB prim.
             (RepoPatch p, ApplyState p ~ Tree, prim ~ PrimOf p)
          => LookForMoves
-         -> Repository rt p wR wU wR
+         -> Repository rt p wU wR
          -> Maybe [AnchoredPath]
          -> IO (FL prim wB wB)
 getMoves NoLookForMoves _ _ = return NilFL
@@ -561,7 +539,7 @@ getMoves YesLookForMoves repository files =
     mkMovesFL [] = NilFL
     mkMovesFL ((a,b,_):xs) = move a b :>: mkMovesFL xs
 
-    getMovedFiles :: Repository rt p wR wU wR
+    getMovedFiles :: Repository rt p wU wR
                   -> Maybe [AnchoredPath]
                   -> IO [(AnchoredPath, AnchoredPath, ItemType)]
     getMovedFiles repo fs = do
@@ -635,11 +613,11 @@ getMoves YesLookForMoves repository files =
 -- | Search for possible replaces between the recordedAndPending state
 -- and the unrecorded (or working) state. Return a Sealed FL list of
 -- replace patches to be applied to the recordedAndPending state.
-getReplaces :: forall rt p wR wU wT
+getReplaces :: forall rt p wU wR
              . (RepoPatch p, ApplyState p ~ Tree)
             => LookForReplaces
             -> DiffAlgorithm
-            -> Repository rt p wR wU wT
+            -> Repository rt p wU wR
             -> Tree IO -- ^ pending tree (including possibly detected moves)
             -> Tree IO -- ^ working tree
             -> IO (Tree IO, -- new pending tree
@@ -654,7 +632,7 @@ getReplaces YesLookForReplaces diffalg _repo pending working = do
       flip runStateT pending $
         forM replaces $ \(path, a, b) ->
           doReplace defaultToks path (BC.unpack a) (BC.unpack b)
-    return (new_pending, mapSeal concatFL $ toFL patches)
+    return (new_pending, mapSeal concatFL $ unFreeLeft $ joinGapsFL patches)
   where
     modifiedTokens :: PrimOf p wX wY -> [(AnchoredPath, B.ByteString, B.ByteString)]
     modifiedTokens p = case isHunk p of
@@ -681,7 +659,7 @@ getReplaces YesLookForReplaces diffalg _repo pending working = do
           Nothing -> getForceReplace path toks old new
           Just pend' -> do
             put pend'
-            return $ joinGap (:>:) (freeGap replacePatch) (emptyGap NilFL)
+            return $ consGapFL replacePatch (emptyGap NilFL)
       where
         replacePatch = tokreplace path toks old new
 
@@ -714,20 +692,15 @@ getReplaces YesLookForReplaces diffalg _repo pending working = do
                 put tree''
                 return patches
 
-
 -- | Add an 'FL' of patches started from the pending state to the pending patch.
--- TODO: add witnesses for pending so we can make the types precise: currently
--- the passed patch can be applied in any context, not just after pending.
-addPendingDiffToPending :: (RepoPatch p, ApplyState p ~ Tree)
-                        => Repository rt p wR wU wR
-                        -> FreeLeft (FL (PrimOf p)) -> IO ()
-addPendingDiffToPending repo newP = do
+unsafeAddToPending :: (RepoPatch p, ApplyState p ~ Tree)
+                   => Repository 'RW p wU wR
+                   -> FreeLeft (FL (PrimOf p)) -> IO ()
+unsafeAddToPending repo newP = do
     (_, Sealed toPend) <- readPending repo
-    invalidateIndex repo
     case unFreeLeft newP of
         (Sealed p) -> do
-            recordedState <- readRecorded repo
-            Pending.makeNewPending repo YesUpdatePending (toPend +>+ p) recordedState
+            Pending.writeTentativePending repo (toPend +>+ p)
 
 -- | Add an 'FL' of patches starting from the working state to the pending patch,
 -- including as much extra context as is necessary (context meaning
@@ -735,17 +708,15 @@ addPendingDiffToPending repo newP = do
 -- changes between pending and working as is possible, and including anything
 -- that doesn't commute, and the patch itself in the new pending patch.
 addToPending :: (RepoPatch p, ApplyState p ~ Tree)
-             => Repository rt p wR wU wR
-             -> UseIndex -> FL (PrimOf p) wU wY -> IO ()
-addToPending repo useidx p = do
-   (toPend :> toUnrec) <- readPendingAndWorking (useidx, ScanKnown, MyersDiff)
-      NoLookForMoves NoLookForReplaces repo Nothing
-   invalidateIndex repo
+             => Repository 'RW p wU wR
+             -> DiffOpts
+             -> FL (PrimOf p) wU wY -> IO ()
+addToPending repo dopts p = do
+   (toPend :> toUnrec) <-
+      readPendingAndWorking dopts repo Nothing
    case genCommuteWhatWeCanRL commuteFL (reverseFL toUnrec :> p) of
        (toP' :> p'  :> _excessUnrec) -> do
-           recordedState <- readRecorded repo
-           Pending.makeNewPending repo YesUpdatePending
-            (toPend +>+ reverseRL toP' +>+ p') recordedState
+           Pending.writeTentativePending repo (toPend +>+ reverseRL toP' +>+ p')
 
-readPlainTree :: Repository rt p wR wU wT -> IO (Tree IO)
+readPlainTree :: Repository rt p wU wR -> IO (Tree IO)
 readPlainTree repo  = PlainTree.readPlainTree (repoLocation repo)
