@@ -26,16 +26,15 @@ import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashed
 import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.UI.External ( viewDocWith )
 import Darcs.UI.Flags ( DarcsFlag, useCache, patchIndexYes, pathsFromArgs )
-import Darcs.UI.Options ( (^), odesc, ocheck
-                        , defaultFlags, parseFlags, (?) )
+import Darcs.UI.Options ( parseFlags, (?), (^) )
 import qualified Darcs.UI.Options.All as O
 
-import Darcs.Repository.State ( readRecorded )
+import Darcs.Repository.State ( readPristine )
 import Darcs.Repository
     ( withRepository
     , withRepoLockCanFail
     , RepoJob(..)
-    , readRepo
+    , readPatches
     )
 import Darcs.Repository.PatchIndex ( attemptCreatePatchIndex )
 import Darcs.Patch.Set ( patchSet2RL )
@@ -79,10 +78,7 @@ annotate = DarcsCommand
     , commandPrereq = amInHashedRepository
     , commandCompleteArgs = knownFileArgs
     , commandArgdefaults = nodefaults
-    , commandAdvancedOptions = odesc annotateAdvancedOpts
-    , commandBasicOptions = odesc annotateBasicOpts
-    , commandDefaults = defaultFlags annotateOpts
-    , commandCheckOptions = ocheck annotateOpts
+    , commandOptions = annotateOpts
     }
   where
     annotateBasicOpts = O.machineReadable ^ O.matchUpToOne ^ O.repoDir
@@ -96,23 +92,24 @@ annotateCmd fps opts args = do
     [path] -> do
       when (patchIndexYes ? opts == O.YesPatchIndex)
         $ withRepoLockCanFail (useCache ? opts)
-        $ RepoJob (\repo -> readRepo repo >>= attemptCreatePatchIndex repo)
+        $ RepoJob (\repo -> readPatches repo >>= attemptCreatePatchIndex repo)
       annotateCmd' opts path
     _ -> die "Error: annotate requires a single filepath argument"
 
 annotateCmd' :: [DarcsFlag] -> AnchoredPath -> IO ()
 annotateCmd' opts fixed_path = withRepository (useCache ? opts) $ RepoJob $ \repository -> do
   let matchFlags = parseFlags O.matchUpToOne opts
-  r <- readRepo repository
-  recorded <- readRecorded repository
+  r <- readPatches repository
+  recorded <- readPristine repository
   (patches, initial, path) <-
     case patchSetMatch matchFlags of
       Just psm -> do
         Sealed x <- getOnePatchset repository psm
-        let (_, [path'], _) =
-              withFileNames Nothing [fixed_path] (rollbackToPatchSetMatch psm r)
-        initial <- snd `fmap` virtualTreeIO (rollbackToPatchSetMatch psm r) recorded
-        return (seal $ patchSet2RL x, initial, path')
+        case withFileNames Nothing [fixed_path] (rollbackToPatchSetMatch psm r) of
+          (_, [path'], _) -> do
+            initial <- snd `fmap` virtualTreeIO (rollbackToPatchSetMatch psm r) recorded
+            return (seal $ patchSet2RL x, initial, path')
+          _ -> error "impossible"
       Nothing ->
         return (seal $ patchSet2RL r, recorded, fixed_path)
 
