@@ -23,83 +23,86 @@ module Darcs.UI.Commands.Record
     ) where
 
 import Darcs.Prelude
-import Data.Foldable ( traverse_ )
 
 import Control.Exception ( handleJust )
-import Control.Monad ( when, unless, void )
+import Control.Monad ( unless, void, when )
 import Data.Char ( ord )
-import System.Exit ( exitFailure, exitSuccess, ExitCode(..) )
+import Data.Foldable ( traverse_ )
 import System.Directory ( removeFile )
+import System.Exit ( ExitCode(..), exitFailure, exitSuccess )
 
-import Darcs.Patch.PatchInfoAnd ( n2pia )
-import Darcs.Repository
-    ( Repository
-    , withRepoLock
-    , RepoJob(..)
-    , tentativelyAddPatch
-    , finalizeRepositoryChanges
-    , invalidateIndex
-    , readPendingAndWorking
-    , readRecorded
-    )
-import Darcs.Repository.Pending ( tentativelyRemoveFromPW )
-
-import Darcs.Patch ( IsRepoType, RepoPatch, PrimOf, sortCoalesceFL )
-import Darcs.Patch.Named ( infopatch, adddeps )
-import Darcs.Patch.Witnesses.Ordered
-    ( FL(..), (:>)(..), nullFL, (+>+) )
+import Darcs.Patch ( PrimOf, RepoPatch, canonizeFL, summaryFL )
+import Darcs.Patch.Apply ( ApplyState )
+import Darcs.Patch.Depends ( contextPatches )
 import Darcs.Patch.Info ( PatchInfo, patchinfo )
-import Darcs.Patch.Apply( ApplyState )
+import Darcs.Patch.Named ( adddeps, infopatch )
+import Darcs.Patch.PatchInfoAnd ( n2pia )
+import Darcs.Patch.Progress ( progressFL )
 import Darcs.Patch.Split ( primSplitter )
-import Darcs.UI.SelectChanges
-    (  WhichChanges(..)
-    , selectionConfigPrim
-    , runInvertibleSelection
-    , askAboutDepends
+import Darcs.Patch.Witnesses.Ordered ( (:>)(..), FL(..), nullFL, (+>+) )
+import Darcs.Repository
+    ( RepoJob(..)
+    , Repository
+    , AccessType(..)
+    , finalizeRepositoryChanges
+    , readPendingAndWorking
+    , readPristine
+    , readPatches
+    , tentativelyAddPatch
+    , tentativelyRemoveFromPW
+    , withRepoLock
     )
-import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions(..) )
-import Darcs.Util.Path ( AnchoredPath, displayPath, AbsolutePath )
+import Darcs.Repository.Flags ( UpdatePending(..) )
 import Darcs.UI.Commands
-    ( DarcsCommand(..), withStdOpts
-    , nodefaults
+    ( DarcsCommand(..)
+    , amInHashedRepository
     , commandAlias
+    , nodefaults
     , setEnvDarcsFiles
     , setEnvDarcsPatches
-    , amInHashedRepository
+    , withStdOpts
+    )
+import Darcs.UI.Commands.Util
+    ( announceFiles
+    , filterExistingPaths
+    , testTentativeAndMaybeExit
     )
-import Darcs.UI.Commands.Util ( announceFiles, filterExistingPaths,
-                                testTentativeAndMaybeExit )
 import Darcs.UI.Completion ( modifiedFileArgs )
 import Darcs.UI.Flags
     ( DarcsFlag
+    , diffingOpts
     , fileHelpAuthor
     , getAuthor
     , getDate
-    , diffOpts
-    , scanKnown
     , pathSetFromArgs
     )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, oparse, defaultFlags )
-import Darcs.UI.PatchHeader ( getLog )
+import Darcs.UI.Options ( Config, (?), (^) )
 import qualified Darcs.UI.Options.All as O
-import Darcs.Repository.Flags ( UpdatePending (..), DryRun(NoDryRun), ScanKnown(..) )
+import Darcs.UI.PatchHeader ( getLog )
+import Darcs.UI.SelectChanges
+    ( WhichChanges(..)
+    , askAboutDepends
+    , runInvertibleSelection
+    , selectionConfigPrim
+    )
+import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions(..) )
 import Darcs.Util.Exception ( clarifyErrors )
-import Darcs.Util.Prompt ( promptYorn )
-import Darcs.Util.Progress ( debugMessage )
 import Darcs.Util.Global ( darcsLastMessage )
-import Darcs.Patch.Progress ( progressFL )
+import Darcs.Util.Path ( AbsolutePath, AnchoredPath, displayPath )
 import Darcs.Util.Printer
     ( Doc
-    , ($+$)
-    , (<+>)
     , formatWords
     , pathlist
     , putDocLn
     , text
     , vcat
     , vsep
+    , ($+$)
+    , (<+>)
     )
-import Darcs.Util.Tree( Tree )
+import Darcs.Util.Progress ( debugMessage )
+import Darcs.Util.Prompt ( promptYorn )
+import Darcs.Util.Tree ( Tree )
 
 recordHelp :: Doc
 recordHelp =
@@ -171,61 +174,6 @@ recordHelp =
     ]
   ])
 
-recordBasicOpts :: DarcsOption a
-                   (Maybe String
-                    -> Maybe String
-                    -> O.TestChanges
-                    -> Maybe Bool
-                    -> Bool
-                    -> Bool
-                    -> Maybe O.AskLongComment
-                    -> O.LookFor
-                    -> Maybe String
-                    -> O.WithContext
-                    -> O.DiffAlgorithm
-                    -> a)
-recordBasicOpts
-    = O.patchname
-    ^ O.author
-    ^ O.testChanges
-    ^ O.interactive
-    ^ O.pipe
-    ^ O.askDeps
-    ^ O.askLongComment
-    ^ O.lookfor
-    ^ O.repoDir
-    ^ O.withContext
-    ^ O.diffAlgorithm
-
-recordAdvancedOpts :: DarcsOption a
-                      (O.Logfile -> O.Compression -> O.UseIndex -> O.UMask -> O.SetScriptsExecutable -> O.IncludeBoring -> a)
-recordAdvancedOpts = O.logfile ^ O.compress ^ O.useIndex ^ O.umask ^ O.setScriptsExecutable ^ O.includeBoring
-
-data RecordConfig = RecordConfig
-    { patchname :: Maybe String
-    , author :: Maybe String
-    , testChanges :: O.TestChanges
-    , interactive :: Maybe Bool
-    , pipe :: Bool
-    , askDeps :: Bool
-    , askLongComment :: Maybe O.AskLongComment
-    , lookfor :: O.LookFor
-    , _workingRepoDir :: Maybe String
-    , withContext :: O.WithContext
-    , diffAlgorithm :: O.DiffAlgorithm
-    , verbosity :: O.Verbosity
-    , logfile :: O.Logfile
-    , compress :: O.Compression
-    , useIndex :: O.UseIndex
-    , umask :: O.UMask
-    , sse :: O.SetScriptsExecutable
-    , includeBoring :: O.IncludeBoring
-    , useCache :: O.UseCache
-    }
-
-recordConfig :: [DarcsFlag] -> RecordConfig
-recordConfig = oparse (recordBasicOpts ^ O.verbosity ^ recordAdvancedOpts ^ O.useCache) RecordConfig
-
 record :: DarcsCommand
 record = DarcsCommand
     { commandProgramName = "darcs"
@@ -238,55 +186,63 @@ record = DarcsCommand
     , commandPrereq = amInHashedRepository
     , commandCompleteArgs = modifiedFileArgs
     , commandArgdefaults = nodefaults
-    , commandAdvancedOptions = odesc recordAdvancedOpts
-    , commandBasicOptions = odesc recordBasicOpts
-    , commandDefaults = defaultFlags recordOpts
-    , commandCheckOptions = ocheck recordOpts
+    , commandOptions = allOpts
     }
   where
-    recordOpts = recordBasicOpts `withStdOpts` recordAdvancedOpts
+    basicOpts
+      = O.patchname
+      ^ O.author
+      ^ O.testChanges
+      ^ O.interactive
+      ^ O.pipe
+      ^ O.askDeps
+      ^ O.askLongComment
+      ^ O.lookforadds
+      ^ O.lookforreplaces
+      ^ O.lookformoves
+      ^ O.repoDir
+      ^ O.diffAlgorithm
+    advancedOpts
+      = O.logfile
+      ^ O.umask
+      ^ O.setScriptsExecutable
+    allOpts = basicOpts `withStdOpts` advancedOpts
 
 -- | commit is an alias for record
 commit :: DarcsCommand
 commit = commandAlias "commit" Nothing record
 
-reportNonExisting :: ScanKnown -> ([AnchoredPath], [AnchoredPath]) -> IO ()
-reportNonExisting scan (paths_only_in_working, _) = do
-  unless (scan /= ScanKnown || null paths_only_in_working) $  putDocLn $
+reportNonExisting :: O.LookForAdds -> ([AnchoredPath], [AnchoredPath]) -> IO ()
+reportNonExisting lfa (paths_only_in_working, _) = do
+  unless (lfa /= O.NoLookForAdds || null paths_only_in_working) $  putDocLn $
     "These paths are not yet in the repository and will be added:" <+>
     pathlist (map displayPath paths_only_in_working)
 
 recordCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-recordCmd fps flags args = do
-    let cfg = recordConfig flags
-    checkNameIsNotOption (patchname cfg) (isInteractive cfg)
-    withRepoLock NoDryRun (useCache cfg) YesUpdatePending (umask cfg) $ RepoJob $ \(repository :: Repository rt p wR wU wR) -> do
-      let scan = scanKnown (O.adds (lookfor cfg)) (includeBoring cfg)
+recordCmd fps cfg args = do
+    checkNameIsNotOption (O.patchname ? cfg) (isInteractive cfg)
+    withRepoLock (O.useCache ? cfg) (O.umask ? cfg) $ RepoJob $ \(repository :: Repository 'RW p wU wR) -> do
       existing_files <- do
         files <- pathSetFromArgs fps args
         files' <-
           traverse
-            (filterExistingPaths
-              repository (verbosity cfg) (useIndex cfg) scan (O.moves (lookfor cfg)))
-            files
-        when (verbosity cfg /= O.Quiet) $
-            traverse_ (reportNonExisting scan) files'
+            (filterExistingPaths repository (O.verbosity ? cfg) (diffingOpts cfg)) files
+        when (O.verbosity ? cfg /= O.Quiet) $
+            traverse_ (reportNonExisting (O.lookforadds ? cfg)) files'
         let files'' = fmap snd files'
         when (files'' == Just []) $
             fail "None of the files you specified exist."
         return files''
-      announceFiles (verbosity cfg) existing_files "Recording changes in"
+      announceFiles (O.verbosity ? cfg) existing_files "Recording changes in"
       debugMessage "About to get the unrecorded changes."
-      changes <- readPendingAndWorking (diffingOpts cfg)
-                   (O.moves (lookfor cfg)) (O.replaces (lookfor cfg))
-                   repository existing_files
-      debugMessage "I've got unrecorded changes."
+      changes <-
+        readPendingAndWorking (diffingOpts cfg) repository existing_files
       case changes of
-          NilFL :> NilFL | not (askDeps cfg) -> do
+          NilFL :> NilFL | not (O.askDeps ? cfg) -> do
               -- We need to grab any input waiting for us, since we
               -- might break scripts expecting to send it to us; we
               -- don't care what that input is, though.
-              void (getDate (pipe cfg))
+              void (getDate (O.pipe ? cfg))
               putStrLn "No changes!"
               exitFailure
           _ -> doRecord repository cfg existing_files changes
@@ -296,42 +252,53 @@ checkNameIsNotOption :: Maybe String -> Bool -> IO ()
 checkNameIsNotOption Nothing     _      = return ()
 checkNameIsNotOption _           False  = return ()
 checkNameIsNotOption (Just name) True   =
-    when (length name == 1 || (length name == 2 && head name == '-')) $ do
+  case name of
+    [_] -> warnPatchName
+    ('-':_) -> warnPatchName
+    _ -> return ()
+  where
+    warnPatchName = do
         confirmed <- promptYorn $ "You specified " ++ show name ++ " as the patch name. Is that really what you want?"
         unless confirmed $ putStrLn "Okay, aborting the record." >> exitFailure
 
-doRecord :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-         => Repository rt p wR wU wR -> RecordConfig -> Maybe [AnchoredPath]
+doRecord :: (RepoPatch p, ApplyState p ~ Tree)
+         => Repository 'RW p wU wR -> Config -> Maybe [AnchoredPath]
          -> (FL (PrimOf p) :> FL (PrimOf p)) wR wU -> IO ()
 doRecord repository cfg files pw@(pending :> working) = do
-    date <- getDate (pipe cfg)
-    my_author <- getAuthor (author cfg) (pipe cfg)
+    debugMessage "I've got unrecorded changes."
+    date <- getDate (O.pipe ? cfg)
+    my_author <- getAuthor (O.author ? cfg) (O.pipe ? cfg)
     debugMessage "I'm slurping the repository."
-    pristine <- readRecorded repository
     debugMessage "About to select changes..."
-    (chs :> _ ) <- runInvertibleSelection (sortCoalesceFL $ pending +>+ working) $
+    let da = O.diffAlgorithm ? cfg
+    (chs :> _ ) <- runInvertibleSelection (canonizeFL da $ pending +>+ working) $
                       selectionConfigPrim
                           First "record" (patchSelOpts cfg)
-                          (Just (primSplitter (diffAlgorithm cfg)))
-                          files (Just pristine)
-    when (not (askDeps cfg) && nullFL chs) $
+                          (Just (primSplitter (O.diffAlgorithm ? cfg)))
+                          files
+    when (not (O.askDeps ? cfg) && nullFL chs) $
               do putStrLn "Ok, if you don't want to record anything, that's fine!"
                  exitSuccess
     handleJust onlySuccessfulExits (\_ -> return ()) $
-             do deps <- if askDeps cfg
-                        then askAboutDepends repository chs (patchSelOpts cfg) []
+             do deps <- if O.askDeps ? cfg
+                        then do
+                          _ :> patches <- contextPatches <$> readPatches repository
+                          askAboutDepends patches chs (patchSelOpts cfg) []
                         else return []
-                when (askDeps cfg) $ debugMessage "I've asked about dependencies."
+                when (O.askDeps ? cfg) $ debugMessage "I've asked about dependencies."
                 if nullFL chs && null deps
                   then putStrLn "Ok, if you don't want to record anything, that's fine!"
                   else do setEnvDarcsFiles chs
-                          (name, my_log, logf) <- getLog (patchname cfg) (pipe cfg) (logfile cfg) (askLongComment cfg) Nothing chs
+                          (name, my_log, logf) <-
+                            getLog (O.patchname ? cfg) (O.pipe ? cfg)
+                              (O.logfile ? cfg) (O.askLongComment ? cfg)
+                              Nothing (summaryFL chs)
                           debugMessage ("Patch name as received from getLog: " ++ show (map ord name))
                           doActualRecord repository cfg name date my_author my_log logf deps chs pw
 
-doActualRecord :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-               => Repository rt p wR wU wR
-               -> RecordConfig
+doActualRecord :: (RepoPatch p, ApplyState p ~ Tree)
+               => Repository 'RW p wU wR
+               -> Config
                -> String -> String -> String
                -> [String] -> Maybe String
                -> [PatchInfo] -> FL (PrimOf p) wR wX
@@ -340,23 +307,21 @@ doActualRecord _repository cfg name date my_author my_log logf deps chs
       (pending :> working) = do
     debugMessage "Writing the patch file..."
     myinfo <- patchinfo date name my_author my_log
-    let mypatch = infopatch myinfo $ progressFL "Writing changes:" chs
+    let mypatch = infopatch myinfo $ progressFL "Writing changes" chs
     let pia = n2pia $ adddeps mypatch deps
     _repository <-
-      tentativelyAddPatch _repository (compress cfg) (verbosity cfg)
-        NoUpdatePending pia
-    invalidateIndex _repository
-    debugMessage "Applying to pristine..."
-    testTentativeAndMaybeExit _repository (verbosity cfg) (testChanges cfg)
-      (sse cfg) (isInteractive cfg) ("you have a bad patch: '" ++ name ++ "'")
+      tentativelyAddPatch _repository NoUpdatePending pia
+    tp <- readPristine _repository
+    testTentativeAndMaybeExit tp cfg
+      ("you have a bad patch: '" ++ name ++ "'")
       "record it" (Just failuremessage)
     tentativelyRemoveFromPW _repository chs pending working
     _repository <-
-      finalizeRepositoryChanges _repository YesUpdatePending (compress cfg)
+      finalizeRepositoryChanges _repository (O.dryRun ? cfg)
       `clarifyErrors` failuremessage
     debugMessage "Syncing timestamps..."
     removeLogFile logf
-    unless (verbosity cfg == O.Quiet) $
+    unless (O.verbosity ? cfg == O.Quiet) $
       putDocLn $ text $ "Finished recording patch '" ++ name ++ "'"
     setEnvDarcsPatches (pia :>: NilFL)
   where
@@ -375,18 +340,14 @@ onlySuccessfulExits :: ExitCode -> Maybe ()
 onlySuccessfulExits ExitSuccess = Just ()
 onlySuccessfulExits _ = Nothing
 
-patchSelOpts :: RecordConfig -> S.PatchSelectionOptions
+patchSelOpts :: Config -> S.PatchSelectionOptions
 patchSelOpts cfg = S.PatchSelectionOptions
-    { S.verbosity = verbosity cfg
+    { S.verbosity = O.verbosity ? cfg
     , S.matchFlags = []
     , S.interactive = isInteractive cfg
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.withSummary = O.NoSummary -- option not supported, use default
-    , S.withContext = withContext cfg
     }
 
-diffingOpts :: RecordConfig -> (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
-diffingOpts cfg = diffOpts (useIndex cfg) (O.adds (lookfor cfg)) O.NoIncludeBoring (diffAlgorithm cfg)
-
-isInteractive :: RecordConfig -> Bool
-isInteractive = maybe True id . interactive
+isInteractive :: Config -> Bool
+isInteractive cfg = maybe True id (O.interactive ? cfg)
