@@ -43,7 +43,7 @@ import Darcs.UI.Commands.Clone ( otherHelpInheritDefault )
 import Darcs.UI.Commands.Util ( printDryRunMessageAndExit, checkUnrelatedRepos )
 import Darcs.UI.Flags
     ( DarcsFlag
-    , willRemoveLogFile, changesReverse, dryRun, useCache, remoteRepos, setDefault
+    , willRemoveLogFile, changesReverse, dryRun, useCache, setDefault
     , fixUrl
     , getCc
     , getAuthor
@@ -60,27 +60,25 @@ import Darcs.UI.Flags
     , minimize
     , editDescription
     )
-import Darcs.UI.Options
-    ( (^), odesc, ocheck
-    , defaultFlags, parseFlags, (?)
-    )
+import Darcs.UI.Options ( (?), (^) )
 import qualified Darcs.UI.Options.All as O
 
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, hopefully, patchDesc )
 import Darcs.Repository
     ( Repository
+    , AccessType(..)
     , repoLocation
     , PatchSet
     , identifyRepositoryFor
     , ReadingOrWriting(..)
     , withRepository
     , RepoJob(..)
-    , readRepo
-    , readRecorded
+    , readPatches
+    , readPristine
     , prefsUrl )
 import Darcs.Patch.Set ( Origin )
 import Darcs.Patch.Apply( ApplyState )
-import Darcs.Patch ( IsRepoType, RepoPatch, description, applyToTree, effect, invert )
+import Darcs.Patch ( RepoPatch, description, applyToTree, effect, invert )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Patch.Witnesses.Ordered
@@ -91,9 +89,13 @@ import Darcs.Patch.Bundle
     , minContext
     , readContextFile
     )
-import Darcs.Repository.Prefs ( addRepoSource, getPreflist )
+import Darcs.Repository.Prefs
+    ( Pref(Defaultrepo, Email, Post, Repos)
+    , addRepoSource
+    , getPreflist
+    )
 import Darcs.Repository.Flags ( DryRun(..) )
-import Darcs.Util.External ( fetchFilePS, Cachable(..) )
+import Darcs.Util.File ( fetchFilePS, Cachable(..) )
 import Darcs.UI.External
     ( signString
     , sendEmailDoc
@@ -103,9 +105,9 @@ import Darcs.UI.External
     )
 import Darcs.Util.ByteString ( mmapFilePS, isAscii )
 import qualified Data.ByteString.Char8 as BC (unpack)
+import Darcs.Util.File ( withOpenTemp )
 import Darcs.Util.Lock
-    ( withOpenTemp
-    , writeDocBinFile
+    ( writeDocBinFile
     , readDocBinFile
     , removeFileMayNotExist
     )
@@ -137,11 +139,10 @@ import Darcs.Util.SignalHandler ( catchInterrupt )
 patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity ? flags
-    , S.matchFlags = parseFlags O.matchSeveral flags
+    , S.matchFlags = O.matchSeveral ? flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps ? flags
     , S.withSummary = O.withSummary ? flags
-    , S.withContext = O.NoContext
     }
 
 send :: DarcsCommand
@@ -154,12 +155,9 @@ send = DarcsCommand
     , commandExtraArgHelp = ["[REPOSITORY]"]
     , commandCommand = sendCmd
     , commandPrereq = amInHashedRepository
-    , commandCompleteArgs = prefArgs "repos"
+    , commandCompleteArgs = prefArgs Repos
     , commandArgdefaults = defaultRepo
-    , commandAdvancedOptions = odesc sendAdvancedOpts
-    , commandBasicOptions = odesc sendBasicOpts
-    , commandDefaults = defaultFlags sendOpts
-    , commandCheckOptions = ocheck sendOpts
+    , commandOptions = sendOpts
     }
   where
     sendBasicOpts
@@ -183,17 +181,16 @@ send = DarcsCommand
       ^ O.allowUnrelatedRepos
     sendAdvancedOpts
       = O.logfile
-      ^ O.remoteRepos
       ^ O.sendToContext 
       ^ O.changesReverse
-      ^ O.network
+      ^ O.remoteDarcs
     sendOpts = sendBasicOpts `withStdOpts` sendAdvancedOpts
 
 sendCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 sendCmd fps opts [""] = sendCmd fps opts []
 sendCmd (_,o) opts [unfixedrepodir] =
  withRepository (useCache ? opts) $ RepoJob $
-  \(repository :: Repository rt p wR wU wR) -> do
+  \(repository :: Repository 'RO p wU wR) -> do
   when (O.mail ? opts && dryRun ? opts == O.NoDryRun) $ do
     -- If --mail is used and the user has not provided a --sendmail-command
     -- and we can detect that the system has no default way to send emails, 
@@ -202,8 +199,8 @@ sendCmd (_,o) opts [unfixedrepodir] =
     when (isNothing sm_cmd) checkDefaultSendmail
   case O.sendToContext ? opts of
     Just contextfile -> do
-        wtds <- decideOnBehavior opts (Nothing :: Maybe (Repository rt p wR wU wR))
-        ref <- readRepo repository
+        wtds <- decideOnBehavior opts (Nothing :: Maybe (Repository rt p wU wR))
+        ref <- readPatches repository
         Sealed them <- readContextFile ref (toFilePath contextfile)
         sendToThem repository opts wtds "CONTEXT" them
     Nothing -> do
@@ -212,29 +209,29 @@ sendCmd (_,o) opts [unfixedrepodir] =
         here <- getCurrentDirectory
         when (repodir == toFilePath here) $
            fail cannotSendToSelf
-        old_default <- getPreflist "defaultrepo"
+        old_default <- getPreflist Defaultrepo
         when (old_default == [repodir]) $
             putInfo opts (creatingPatch repodir)
         repo <- identifyRepositoryFor Reading repository (useCache ? opts) repodir
-        them <- readRepo repo
-        addRepoSource repodir (dryRun ? opts) (remoteRepos ? opts)
-            (setDefault False opts) (O.inheritDefault ? opts) (isInteractive True opts)
+        them <- readPatches repo
+        addRepoSource repodir (dryRun ? opts) (setDefault False opts)
+          (O.inheritDefault ? opts) (isInteractive True opts)
         wtds <- decideOnBehavior opts (Just repo)
         sendToThem repository opts wtds repodir them
 sendCmd _ _ _ = error "impossible case"
 
-sendToThem :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-           => Repository rt p wR wU wT -> [DarcsFlag] -> [WhatToDo] -> String
-           -> PatchSet rt p Origin wX -> IO ()
+sendToThem :: (RepoPatch p, ApplyState p ~ Tree)
+           => Repository rt p wU wR -> [DarcsFlag] -> [WhatToDo] -> String
+           -> PatchSet p Origin wX -> IO ()
 sendToThem repo opts wtds their_name them = do
-  us <- readRepo repo
+  us <- readPatches repo
   common :> us' <- return $ findCommonWithThem us them
   checkUnrelatedRepos (O.allowUnrelatedRepos ? opts) us them
   case us' of
       NilFL -> do putInfo opts nothingSendable
                   exitSuccess
       _     -> putVerbose opts $ selectionIs (mapFL description us')
-  pristine <- readRecorded repo
+  pristine <- readPristine repo
   let direction = if changesReverse ? opts then FirstReversed else First
       selection_config = selectionConfig direction "send" (patchSelOpts opts) Nothing Nothing
   (to_be_sent :> _) <- runSelection us' selection_config
@@ -260,21 +257,21 @@ sendToThem repo opts wtds their_name them = do
   here   <- getCurrentDirectory
   let make_fname (tb:>:_) = getUniqueDPatchName $ patchDesc tb
       make_fname _ = error "impossible case"
-  fname <- make_fname to_be_sent
+  let fname = make_fname to_be_sent
   let outname = case getOutput opts fname of
                     Just f  -> Just f
                     Nothing | O.mail ? opts -> Nothing
-                            | not $ null [ p | Post p <- wtds] -> Nothing
-                            | otherwise        -> Just (makeAbsoluteOrStd here fname)
+                            | not $ null [ p | PostHttp p <- wtds] -> Nothing
+                            | otherwise        -> Just (makeAbsoluteOrStd here <$> fname)
   case outname of
-    Just fname' -> writeBundleToFile opts to_be_sent bundle fname' wtds their_name
-    Nothing     -> sendBundle opts to_be_sent bundle fname wtds their_name
+    Just fname' -> fname' >>= \f -> writeBundleToFile opts to_be_sent bundle f wtds their_name
+    Nothing     -> fname >>= \f -> sendBundle opts to_be_sent bundle f wtds their_name
 
 
-prepareBundle :: forall rt p wX wY wZ. (RepoPatch p, ApplyState p ~ Tree)
-              => [DarcsFlag] -> PatchSet rt p Origin wZ
-              -> Either (FL (PatchInfoAnd rt p) wX wY)
-                        (Tree IO, (FL (PatchInfoAnd rt p) :\/: FL (PatchInfoAnd rt p)) wX wY)
+prepareBundle :: forall p wX wY wZ. (RepoPatch p, ApplyState p ~ Tree)
+              => [DarcsFlag] -> PatchSet p Origin wZ
+              -> Either (FL (PatchInfoAnd p) wX wY)
+                        (Tree IO, (FL (PatchInfoAnd p) :\/: FL (PatchInfoAnd p)) wX wY)
               -> IO Doc
 prepareBundle opts common e = do
   unsig_bundle <-
@@ -287,78 +284,88 @@ prepareBundle opts common e = do
        Left to_be_sent -> makeBundle Nothing
                                       (unsafeCoerceP common)
                                       (mapFL_FL hopefully to_be_sent)
-  signString (parseFlags O.sign opts) unsig_bundle
+  signString (O.sign ? opts) unsig_bundle
 
-sendBundle :: forall rt p wX wY . (RepoPatch p, ApplyState p ~ Tree)
-           => [DarcsFlag] -> FL (PatchInfoAnd rt p) wX wY
-             -> Doc -> String -> [WhatToDo] -> String -> IO ()
-sendBundle opts to_be_sent bundle fname wtds their_name=
-         let
-           auto_subject :: forall pp wA wB . FL (PatchInfoAnd rt pp) wA wB -> String
-           auto_subject (p:>:NilFL)  = "darcs patch: " ++ trim (patchDesc p) 57
-           auto_subject (p:>:ps) = "darcs patch: " ++ trim (patchDesc p) 43 ++
-                            " (and " ++ show (lengthFL ps) ++ " more)"
-           auto_subject _ = error "Tried to get a name from empty patch list."
-           trim st n = if length st <= n then st
-                       else take (n-3) st ++ "..."
-           in do
-           thetargets <- getTargets wtds
-           from <- getAuthor (author ? opts) False
-           let thesubject = fromMaybe (auto_subject to_be_sent) $ getSubject opts
-           (mailcontents, mailfile, mailcharset) <- getDescription opts their_name to_be_sent
-
-           let warnMailBody = case mailfile of
-                                  Just mf -> putDocLn $ emailBackedUp mf
-                                  Nothing -> return ()
-
-               warnCharset msg = do
-                 confirmed <- promptYorn $ promptCharSetWarning msg
-                 unless confirmed $ do
-                    putDocLn charsetAborted
-                    warnMailBody
-                    exitSuccess
-
-           thecharset <- case charset ? opts of
-                              -- Always trust provided charset
-                              providedCset@(Just _) -> return providedCset
-                              Nothing ->
-                                case mailcharset of
-                                Nothing -> do
-                                  warnCharset charsetCouldNotGuess
-                                  return mailcharset
-                                Just "utf-8" -> return mailcharset
-                                -- Trust other cases (us-ascii)
-                                Just _ -> return mailcharset
-
-           let body = makeEmail their_name
-                        (maybe [] (\x -> [("In-Reply-To", x), ("References", x)]) . getInReplyTo $ opts)
-                        (Just mailcontents)
-                        thecharset
-                        bundle
-                        (Just fname)
-               contentAndBundle = Just (mailcontents, bundle)
-
-               sendmail =
-                (do
-                 let to = generateEmailToString thetargets
-                 sm_cmd <- getSendmailCmd opts
-                 sendEmailDoc from to thesubject (getCc opts)
-                               sm_cmd contentAndBundle body
-                 putInfo opts (success to (getCc opts)))
-                 `onException` warnMailBody
-
-           when (null [ p | Post p <- thetargets]) sendmail
-           nbody <- withOpenTemp $ \ (fh,fn) -> do
-               let to = generateEmailToString thetargets
-               generateEmail fh from to thesubject (getCc opts) body
-               hClose fh
-               mmapFilePS fn
-           forM_ [ p | Post p <- thetargets]
-             (\url -> do
-                putInfo opts $ postingPatch url
-                postUrl url nbody "message/rfc822")
-             `catch` (\(_ :: IOException) -> sendmail)
-           cleanup opts mailfile
+sendBundle
+  :: forall p wX wY
+   . (RepoPatch p, ApplyState p ~ Tree)
+  => [DarcsFlag]
+  -> FL (PatchInfoAnd p) wX wY
+  -> Doc
+  -> String
+  -> [WhatToDo]
+  -> String
+  -> IO ()
+sendBundle opts to_be_sent bundle fname wtds their_name = do
+  let auto_subject :: forall pp wA wB. FL (PatchInfoAnd pp) wA wB -> String
+      auto_subject (p :>: NilFL) = "darcs patch: " ++ trim (patchDesc p) 57
+      auto_subject (p :>: ps) =
+        "darcs patch: " ++
+        trim (patchDesc p) 43 ++ " (and " ++ show (lengthFL ps) ++ " more)"
+      auto_subject _ = error "Tried to get a name from empty patch list."
+      trim st n =
+        if length st <= n
+          then st
+          else take (n - 3) st ++ "..."
+  thetargets <- getTargets wtds
+  from <- getAuthor (author ? opts) False
+  let thesubject = fromMaybe (auto_subject to_be_sent) $ getSubject opts
+  (mailcontents, mailfile, mailcharset) <-
+    getDescription opts their_name to_be_sent
+  let warnMailBody =
+        case mailfile of
+          Just mf -> putDocLn $ emailBackedUp mf
+          Nothing -> return ()
+      warnCharset msg = do
+        confirmed <- promptYorn $ promptCharSetWarning msg
+        unless confirmed $ do
+          putDocLn charsetAborted
+          warnMailBody
+          exitSuccess
+  thecharset <-
+    case charset ? opts of
+      -- Always trust provided charset
+      providedCset@(Just _) ->
+        return providedCset
+      Nothing ->
+        case mailcharset of
+          Nothing -> do
+            warnCharset charsetCouldNotGuess
+            return mailcharset
+          Just "utf-8" ->
+            return mailcharset
+          -- Trust other cases (us-ascii)
+          Just _ -> return mailcharset
+  let body =
+        makeEmail
+          their_name
+          (maybe [] (\x -> [("In-Reply-To", x), ("References", x)]) .
+           getInReplyTo $
+           opts)
+          (Just mailcontents) thecharset bundle (Just fname)
+      contentAndBundle = Just (mailcontents, bundle)
+      sendmail =
+        (do let to = generateEmailToString thetargets
+            sm_cmd <- getSendmailCmd opts
+            sendEmailDoc from to thesubject (getCc opts) sm_cmd
+              contentAndBundle body
+            putInfo opts (success to (getCc opts)))
+        `onException`
+        warnMailBody
+  when (null [p | PostHttp p <- thetargets]) sendmail
+  nbody <-
+    withOpenTemp $ \(fh, fn) -> do
+      let to = generateEmailToString thetargets
+      generateEmail fh from to thesubject (getCc opts) body
+      hClose fh
+      mmapFilePS fn
+  forM_
+    [p | PostHttp p <- thetargets]
+    (\url -> do
+       putInfo opts $ postingPatch url
+       postUrl url nbody "message/rfc822") `catch`
+    (\(_ :: IOException) -> sendmail)
+  cleanup opts mailfile
 
 generateEmailToString :: [WhatToDo] -> String
 generateEmailToString = intercalate " , " . filter (/= "") . map extractEmail
@@ -371,8 +378,8 @@ cleanup opts (Just mailfile) = when (isNothing (hasLogfile opts) || willRemoveLo
                                       removeFileMayNotExist mailfile
 cleanup _ Nothing = return ()
 
-writeBundleToFile :: forall rt p wX wY . (RepoPatch p, ApplyState p ~ Tree)
-                  => [DarcsFlag] -> FL (PatchInfoAnd rt p) wX wY -> Doc ->
+writeBundleToFile :: forall p wX wY . (RepoPatch p, ApplyState p ~ Tree)
+                  => [DarcsFlag] -> FL (PatchInfoAnd p) wX wY -> Doc ->
                     AbsolutePathOrStd -> [WhatToDo] -> String -> IO ()
 writeBundleToFile opts to_be_sent bundle fname wtds their_name =
     do (d,f,_) <- getDescription opts their_name to_be_sent
@@ -385,10 +392,10 @@ writeBundleToFile opts to_be_sent bundle fname wtds their_name =
        cleanup opts f
 
 data WhatToDo
-    = Post String        -- ^ POST the patch via HTTP
+    = PostHttp String    -- ^ POST the patch via HTTP
     | SendMail String    -- ^ send patch via email
 
-decideOnBehavior :: [DarcsFlag] -> Maybe (Repository rt p wR wU wT) -> IO [WhatToDo]
+decideOnBehavior :: [DarcsFlag] -> Maybe (Repository rt p wU wR) -> IO [WhatToDo]
 decideOnBehavior opts remote_repo =
     case the_targets of
     [] -> do wtds <- case remote_repo of
@@ -401,26 +408,26 @@ decideOnBehavior opts remote_repo =
     where the_targets = collectTargets opts
           check_post the_remote_repo =
                        do p <- ((readPost . BC.unpack) `fmap`
-                                fetchFilePS (prefsUrl (repoLocation the_remote_repo) ++ "/post")
+                                fetchFilePS (prefsUrl (repoLocation the_remote_repo) Post)
                                 (MaxAge 600)) `catchall` return []
                           emails <- who_to_email the_remote_repo
                           return (p++emails)
           readPost = map parseLine . lines where
-              parseLine t = maybe (Post t) SendMail $ stripPrefix "mailto:" t
+              parseLine t = maybe (PostHttp t) SendMail $ stripPrefix "mailto:" t
           who_to_email repo =
               do email <- (BC.unpack `fmap`
-                           fetchFilePS (prefsUrl (repoLocation repo) ++ "/email")
+                           fetchFilePS (prefsUrl (repoLocation repo) Email)
                                        (MaxAge 600))
                           `catchall` return ""
                  if '@' `elem` email then return . map SendMail $ lines email
                                      else return []
           announce_recipients emails =
             let pn (SendMail s) = s
-                pn (Post p) = p
+                pn (PostHttp p) = p
                 msg = willSendTo (dryRun ? opts) (map pn emails)
             in case dryRun ? opts of
                 O.YesDryRun -> putInfo opts msg
-                O.NoDryRun  -> when (null the_targets && isNothing (getOutput opts "")) $
+                O.NoDryRun  -> when (null the_targets && isNothing (getOutput opts (return ""))) $
                                 putInfo opts msg
 
 getTargets :: [WhatToDo] -> IO [WhatToDo]
@@ -429,11 +436,11 @@ getTargets wtds = return wtds
 
 collectTargets :: [DarcsFlag] -> [WhatToDo]
 collectTargets flags = [ f t | t <- O._to (O.headerFields ? flags) ] where
-    f url | "http:" `isPrefixOf` url = Post url
+    f url | "http:" `isPrefixOf` url = PostHttp url
     f em = SendMail em
 
 getDescription :: RepoPatch p
-               => [DarcsFlag] -> String -> FL (PatchInfoAnd rt p) wX wY -> IO (Doc, Maybe String, Maybe String)
+               => [DarcsFlag] -> String -> FL (PatchInfoAnd p) wX wY -> IO (Doc, Maybe String, Maybe String)
 getDescription opts their_name patches =
     case get_filename of
         Just file -> do
