@@ -20,15 +20,15 @@ module Darcs.UI.Commands.TransferMode ( transferMode ) where
 
 import Darcs.Prelude
 
+import System.Directory ( withCurrentDirectory )
 import Control.Exception ( catch )
 import System.IO ( stdout, hFlush )
 
-import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Exception ( prettyException )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInRepository )
 import Darcs.UI.Completion ( noArgs )
 import Darcs.UI.Flags ( DarcsFlag )
-import Darcs.UI.Options ( oid, odesc, ocheck, defaultFlags )
+import Darcs.UI.Options ( oid )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Util.Printer ( Doc, text )
 import Darcs.Util.Progress ( setProgressMode )
@@ -65,10 +65,7 @@ transferMode = DarcsCommand
     , commandCommand = transferModeCmd
     , commandPrereq = amInRepository
     , commandArgdefaults = nodefaults
-    , commandAdvancedOptions = []
-    , commandBasicOptions = odesc transferModeBasicOpts
-    , commandDefaults = defaultFlags transferModeOpts
-    , commandCheckOptions = ocheck transferModeOpts
+    , commandOptions = transferModeOpts
     }
   where
     transferModeBasicOpts = O.repoDir
