@@ -22,17 +22,10 @@
 -- renamed and the re-exports from "Darcs.UI.Options.All" removed.
 module Darcs.UI.Flags
     ( F.DarcsFlag
-    , remoteDarcs
     , diffingOpts
-    , diffOpts
-    , scanKnown
     , wantGuiPause
     , isInteractive
     , willRemoveLogFile
-    , includeBoring
-    , lookForAdds
-    , lookForMoves
-    , lookForReplaces
     , setDefault
     , allowConflicts
     , hasXmlOutput
@@ -41,7 +34,6 @@ module Darcs.UI.Flags
     , verbose
     , enumeratePatches
 
-    , fixRemoteRepos
     , fixUrl
     , pathsFromArgs
     , pathSetFromArgs
@@ -62,12 +54,10 @@ module Darcs.UI.Flags
     , withNewRepo
 
     -- * Re-exports
-    , O.compress
     , O.diffAlgorithm
     , O.reorder
     , O.minimize
     , O.editDescription
-    , O.externalMerge
     , O.maxCount
     , O.matchAny
     , O.withContext
@@ -82,12 +72,10 @@ module Darcs.UI.Flags
     , O.useIndex
     , O.umask
     , O.dryRun
-    , O.runTest
     , O.testChanges
     , O.setScriptsExecutable
     , O.withWorkingDir
     , O.leaveTestDir
-    , O.remoteRepos
     , O.cloneKind
     , O.patchIndexNo
     , O.patchIndexYes
@@ -111,19 +99,17 @@ import Data.Maybe
     , isNothing
     , catMaybes
     )
-import Control.Monad ( unless )
-import System.Directory ( doesDirectoryExist, createDirectory )
+import Control.Monad ( void, unless )
+import System.Directory ( createDirectory, doesDirectoryExist, withCurrentDirectory )
 import System.FilePath.Posix ( (</>) )
 import System.Environment ( lookupEnv )
+import System.Posix.Files ( getSymbolicLinkStatus )
 
--- Use of RemoteRepo data constructor is harmless here, if not ideal.
--- See haddocks for fixRemoteRepos below for details.
-import qualified Darcs.UI.Options.Flags as F ( DarcsFlag(RemoteRepo) )
+import qualified Darcs.UI.Options.Flags as F ( DarcsFlag )
 import Darcs.UI.Options ( Config, (?), (^), oparse, parseFlags, unparseOpt )
 import qualified Darcs.UI.Options.All as O
 
-import Darcs.Util.Exception ( catchall )
-import Darcs.Util.File ( withCurrentDirectory )
+import Darcs.Util.Exception ( catchall, ifDoesNotExistError )
 import Darcs.Util.Prompt
     ( askUser
     , askUserListItem
@@ -131,7 +117,8 @@ import Darcs.Util.Prompt
 import Darcs.Util.Lock ( writeTextFile )
 import Darcs.Repository.Flags ( WorkRepo(..) )
 import Darcs.Repository.Prefs
-    ( getPreflist
+    ( Pref(Author)
+    , getPreflist
     , getGlobal
     , globalPrefsDirDoc
     , globalPrefsDir
@@ -141,13 +128,14 @@ import Darcs.Util.IsoDate ( getIsoDateTime, cleanLocalDate )
 import Darcs.Util.Path
     ( AbsolutePath
     , AbsolutePathOrStd
-    , toFilePath
-    , makeSubPathOf
-    , ioAbsolute
-    , makeAbsoluteOrStd
     , AnchoredPath
     , floatSubPath
     , inDarcsdir
+    , ioAbsolute
+    , makeAbsolute
+    , makeAbsoluteOrStd
+    , makeRelativeTo
+    , toFilePath
     )
 import Darcs.Util.Printer ( pathlist, putDocLn, text, ($$), (<+>) )
 import Darcs.Util.Printer.Color ( ePutDocLn )
@@ -159,25 +147,17 @@ verbose = (== O.Verbose) . parseFlags O.verbosity
 quiet :: Config -> Bool
 quiet = (== O.Quiet) . parseFlags O.verbosity
 
-remoteDarcs :: Config -> O.RemoteDarcs
-remoteDarcs = O.remoteDarcs . parseFlags O.network
-
 enumeratePatches :: Config -> Bool
 enumeratePatches = (== O.YesEnumPatches) . parseFlags O.enumPatches
 
-diffOpts :: O.UseIndex -> O.LookForAdds -> O.IncludeBoring -> O.DiffAlgorithm -> (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
-diffOpts use_index look_for_adds include_boring diff_alg =
-    (use_index, scanKnown look_for_adds include_boring, diff_alg)
-
--- | Non-trivial interaction between options.
-scanKnown :: O.LookForAdds -> O.IncludeBoring -> O.ScanKnown
-scanKnown O.NoLookForAdds _ = O.ScanKnown
-scanKnown O.YesLookForAdds O.NoIncludeBoring = O.ScanAll
-scanKnown O.YesLookForAdds O.YesIncludeBoring = O.ScanBoring
-
-diffingOpts :: Config -> (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
-diffingOpts flags = diffOpts (O.useIndex ? flags) (lookForAdds flags)
-    (parseFlags O.includeBoring flags) (O.diffAlgorithm ? flags)
+diffingOpts :: Config -> O.DiffOpts
+diffingOpts flags =
+  O.DiffOpts
+    (O.useIndex ? flags)
+    (O.lookforadds ? flags)
+    (O.lookforreplaces ? flags)
+    (O.lookformoves ? flags)
+    (O.diffAlgorithm ? flags)
 
 -- | This will become dis-entangled as soon as we inline these functions.
 wantGuiPause :: Config -> O.WantGuiPause
@@ -187,7 +167,10 @@ wantGuiPause fs =
     else O.NoWantGuiPause
   where
     hasDiffCmd = isJust . O.diffCmd . parseFlags O.extDiff
-    hasExternalMerge = (/= O.NoExternalMerge) . parseFlags O.externalMerge
+    hasExternalMerge flags =
+      case O.conflictsNo ? flags of
+        Just (O.YesAllowConflicts (O.ExternalMerge _)) -> True
+        _ -> False
     hasPause = (== O.YesWantGuiPause) . parseFlags O.pauseForGui
 
 -- | Non-trivial interaction between options. Explicit @-i@ or @-a@ dominates,
@@ -206,20 +189,6 @@ isInteractive def = oparse (O.dryRunXml ^ O.changesFormat ^ O.interactive) decid
 willRemoveLogFile :: Config -> Bool
 willRemoveLogFile = O._rmlogfile . parseFlags O.logfile
 
-includeBoring :: Config -> Bool
-includeBoring cfg = case parseFlags O.includeBoring cfg of
-  O.NoIncludeBoring -> False
-  O.YesIncludeBoring -> True
-
-lookForAdds :: Config -> O.LookForAdds
-lookForAdds = O.adds . parseFlags O.lookfor
-
-lookForReplaces :: Config -> O.LookForReplaces
-lookForReplaces = O.replaces . parseFlags O.lookfor
-
-lookForMoves :: Config -> O.LookForMoves
-lookForMoves = O.moves . parseFlags O.lookfor
-
 setDefault :: Bool -> Config -> O.SetDefault
 setDefault defYes = maybe def noDef . parseFlags O.setDefault where
   def = if defYes then O.YesSetDefault False else O.NoSetDefault False
@@ -228,21 +197,14 @@ setDefault defYes = maybe def noDef . parseFlags O.setDefault where
 allowConflicts :: Config -> O.AllowConflicts
 allowConflicts = maybe O.NoAllowConflicts id . parseFlags O.conflictsNo
 
--- | Ugly. The alternative is to put the remoteRepos accessor into the IO monad,
--- which is hardly better.
--- However, accessing the flag list directly here is benign, as we only map
--- over the list and don't change the order.
-fixRemoteRepos :: AbsolutePath -> Config -> IO Config
-fixRemoteRepos d = mapM fixRemoteRepo where
-  fixRemoteRepo (F.RemoteRepo p) = F.RemoteRepo `fmap` fixUrl d p
-  fixRemoteRepo f = return f
-
--- | 'fixUrl' takes a String that may be a file path or a URL.
--- It returns either the URL, or an absolute version of the path.
+-- | The first argument is an 'AbsolutePath', the second a 'String' that may be
+-- a file path or a URL. It returns either the URL, or an absolute version of
+-- the path, interpreted relative to the first argument.
 fixUrl :: AbsolutePath -> String -> IO String
-fixUrl d f = if isValidLocalPath f
-                then toFilePath `fmap` withCurrentDirectory d (ioAbsolute f)
-                else return f
+fixUrl d f =
+  if isValidLocalPath f
+    then withCurrentDirectory (toFilePath d) (toFilePath <$> ioAbsolute f)
+    else return f
 
 -- TODO move the following four functions somewhere else,
 -- they have nothing to do with flags
@@ -301,12 +263,19 @@ maybeFixSubPaths (r, o) fs = do
     -- special case here because fixit otherwise converts
     -- "" to (SubPath "."), which is a valid path
     fixit "" = return Nothing
-    fixit p = do ap <- withCurrentDirectory o $ ioAbsolute p
-                 case makeSubPathOf r ap of
-                   Just sp -> return $ Just $ floatSubPath sp
-                   Nothing -> do
-                     absolutePathByRepodir <- withCurrentDirectory r $ ioAbsolute p
-                     return $ floatSubPath <$> makeSubPathOf r absolutePathByRepodir
+    fixit p = do
+      -- raise an exception if the given path has a trailing pathSeparator
+      -- but refers to an existing non-directory
+      ifDoesNotExistError () $ void (getSymbolicLinkStatus p)
+      msp <- makeRelativeTo r (makeAbsolute o p)
+      case msp of
+        Just sp -> return $ floatIt sp
+        Nothing -> do
+          msp' <- makeRelativeTo r (makeAbsolute r p)
+          case msp' of
+            Nothing -> return Nothing
+            Just sp' -> return $ floatIt sp'
+    floatIt = either (const Nothing) Just . floatSubPath
 
 -- | 'getRepourl' takes a list of flags and returns the url of the
 -- repository specified by @Repodir \"directory\"@ in that list of flags, if any.
@@ -415,8 +384,8 @@ promptAuthor store alwaysAsk = do
 -- possibilities when reading from global preferences.
 getEasyAuthor :: IO [String]
 getEasyAuthor =
-  firstNotNullIO [ (take 1 . nonblank) `fmap` getPreflist "author"
-                 , nonblank    `fmap` getGlobal "author"
+  firstNotNullIO [ (take 1 . nonblank) `fmap` getPreflist Author
+                 , nonblank    `fmap` getGlobal Author
                  , maybeToList `fmap` lookupEnv "DARCS_EMAIL"
                  , maybeToList `fmap` lookupEnv "EMAIL"
                  ]
@@ -460,11 +429,13 @@ getSendmailCmd fs =
     Nothing -> lookupEnv "SENDMAIL"
     justcmd -> return justcmd
 
--- | Accessor for output option
-getOutput :: Config -> FilePath -> Maybe AbsolutePathOrStd
+-- | Accessor for output option. Takes and returns IO actions
+-- so that the default value is only calculated if needed,
+-- as it might involve filesystem actions that can fail.
+getOutput :: Config -> IO FilePath -> Maybe (IO AbsolutePathOrStd)
 getOutput fs fp = fmap go (parseFlags O.output fs) where
-  go (O.Output ap)         = ap
-  go (O.OutputAutoName ap) = makeAbsoluteOrStd ap fp
+  go (O.Output ap)         = return ap
+  go (O.OutputAutoName ap) = makeAbsoluteOrStd ap <$> fp
 
 -- |'getSubject' takes a list of flags and returns the subject of the mail
 -- to be sent by @darcs send@. Looks for a subject specified by
