@@ -5,6 +5,7 @@ module Darcs.UI.Options
     , DarcsOptDescr
     , optDescr
     , Config
+    , withDashes
     ) where
 
 import Darcs.Prelude
@@ -14,7 +15,7 @@ import System.Console.GetOpt ( OptDescr )
 
 import Darcs.UI.Options.All ( DarcsOption )
 import Darcs.UI.Options.Core
-import Darcs.UI.Options.Util ( DarcsOptDescr, Flag, PrimDarcsOption )
+import Darcs.UI.Options.Util ( DarcsOptDescr, Flag, PrimDarcsOption, withDashes )
 import Darcs.Util.Path ( AbsolutePath )
 
 -- | Instantiate a 'DarcsOptDescr' with an 'AbsolutePath'
