@@ -37,6 +37,8 @@ import Darcs.UI.Options.Iso
 
 -- * Option specifications
 
+data OptMsg = OptWarning String | OptError String
+
 {-| A type for option specifications.
 
 It consists of four components: a parser, an unparser, a checker, and a list
@@ -130,7 +132,7 @@ data OptSpec d f a b = OptSpec
   , oparse :: b -> ([f] -> a)
     -- ^ Convert flag list to option value, in CPS. Note: as a pure
     -- function, it is not supposed to fail.
-  , ocheck :: [f] -> [String]
+  , ocheck :: [f] -> [OptMsg]
     -- ^ Check for erros in a flag list, returns error messages.
   , odesc :: [d f]
     -- ^ Descriptions, one for each flag that makes up the option.
@@ -284,7 +286,7 @@ instance Semigroup (PrimOptSpec d f a [v]) where
 
 -- | See 'oappend' and 'oempty'.
 instance Monoid (PrimOptSpec d f a [v]) where
-  mappend = oappend
+  mappend = (<>)
   mempty = oempty
 
 -- | Parse a list of flags against a primitive option spec, returning the
