@@ -20,13 +20,14 @@ data DarcsFlag = Version | ExactVersion | ListCommands
                | Subject String | InReplyTo String | Charset String
                | SendmailCmd String | Author String | SelectAuthor | PatchName String
                | OnePatch String | SeveralPatch String
-               | OneHash String
                | AfterPatch String | UpToPatch String
-               | AfterHash String | UpToHash String
-               | TagName String | LastN String | MaxCount String
+               | OnePattern String | SeveralPattern String
+               | AfterPattern String | UpToPattern String
+               | OneHash String | AfterHash String | UpToHash String
+               | OneTag String | SeveralTag String | AfterTag String | UpToTag String
+               | LastN String | MaxCount String
                | IndexRange String | OneIndex String
                | NumberPatches
-               | OneTag String | AfterTag String | UpToTag String
                | GenContext | Context AbsolutePath | Count
                | LogFile AbsolutePath | RmLogFile | DontRmLogFile
                | DistName String | DistZip | All
@@ -55,7 +56,7 @@ data DarcsFlag = Version | ExactVersion | ListCommands
                | AllowWindowsReserved | DontAllowWindowsReserved
                | DontGrabDeps | DontPromptForDependencies | PromptForDependencies
                | Compress | NoCompress | UnCompress
-               | WorkRepoDir String | WorkRepoUrl String | RemoteRepo String
+               | WorkRepoDir String | WorkRepoUrl String
                | NewRepo String
                | NotInRemote (Maybe String)
                | Reply String | ApplyAs String
@@ -69,14 +70,13 @@ data DarcsFlag = Version | ExactVersion | ListCommands
                | DiffFlags String
                | XMLOutput
                | ForceReplace
-               | OnePattern String | SeveralPattern String
-               | AfterPattern String | UpToPattern String
                | NonApply | NonVerify | NonForce
                | DryRun
                | InheritDefault | NoInheritDefault
                | SetDefault | NoSetDefault
                | Disable | SetScriptsExecutable | DontSetScriptsExecutable
                | Once | Linear | Backoff | Bisect
+               | ShrinkFailure | NoShrinkFailure
                | Hashed -- deprecated flag, here to output an error message
                | UseFormat1 | UseFormat2 | UseFormat3
                | UseNoWorkingDir | UseWorkingDir
@@ -92,10 +92,12 @@ data DarcsFlag = Version | ExactVersion | ListCommands
                | NoCache
                | AllowUnrelatedRepos
                | Check | Repair | JustThisRepo
-               | ReadMarks String | WriteMarks String
+               | ReadMarks AbsolutePath | WriteMarks AbsolutePath
                | NullFlag
                | NoAmendUnrecord | AmendUnrecord
                | PatchIndexFlag
                | NoPatchIndexFlag
                | EnumPatches | NoEnumPatches
+               | WithPrefsTemplates | NoPrefsTemplates
+               | OptimizeDeep | OptimizeShallow
                  deriving ( Eq, Show )
