@@ -128,7 +128,7 @@ toHash = OptSpec {..} where
   oparse k fs = k [ UpToHash s | F.UpToHash s <- fs ]
   ocheck _ = []
   odesc = [ strArg [] ["to-hash"] F.UpToHash "HASH"
-    "select changes up to a patch with HASH" ]
+    "select changes up to a patch whose hash prefix matches HASH" ]
 
 context = OptSpec {..} where
   ounparse k mfs = k [ F.Context p | Context p <- mfs ]
@@ -163,7 +163,7 @@ fromHash = OptSpec {..} where
   oparse k fs = k [ AfterHash s | F.AfterHash s <- fs ]
   ocheck _ = []
   odesc = [ strArg [] ["from-hash"] F.AfterHash "HASH"
-    "select changes starting with a patch with HASH" ]
+    "select changes starting with a patch whose hash prefix matches HASH" ]
 
 fromTag = OptSpec {..} where
   ounparse k mfs = k [ F.AfterTag s | AfterTag s <- mfs ]
@@ -179,10 +179,10 @@ tag = OptSpec {..} where
   odesc = [ strArg ['t'] ["tag"] F.OneTag "REGEXP" "select tag matching REGEXP" ]
 
 tags = OptSpec {..} where
-  ounparse k mfs = k [ F.OneTag s | OneTag s <- mfs ]
-  oparse k fs = k [ OneTag s | F.OneTag s <- fs ]
+  ounparse k mfs = k [ F.SeveralTag s | SeveralTag s <- mfs ]
+  oparse k fs = k [ SeveralTag s | F.SeveralTag s <- fs ]
   ocheck _ = []
-  odesc = [ strArg ['t'] ["tags"] F.OneTag "REGEXP" "select tags matching REGEXP" ]
+  odesc = [ strArg ['t'] ["tags"] F.SeveralTag "REGEXP" "select tags matching REGEXP" ]
 
 patch = OptSpec {..} where
   ounparse k mfs = k [ F.OnePatch s | OnePatch s <- mfs ]
@@ -203,7 +203,7 @@ hash = OptSpec {..} where
   oparse k fs = k [ OneHash s | F.OneHash s <- fs ]
   ocheck _ = []
   odesc = [ strArg ['h'] ["hash"] F.OneHash "HASH"
-    "select a single patch with HASH" ]
+    "select a single patch whose hash prefix matches HASH" ]
 
 match = OptSpec {..} where
   ounparse k mfs = k [ F.OnePattern s | OnePattern s <- mfs ]
