@@ -1,38 +1,45 @@
 module Darcs.UI.PatchHeader
     ( getLog
     , getAuthor
+    , editLog
     , updatePatchHeader, AskAboutDeps(..)
+    , PatchHeaderConfig
+    , patchHeaderConfig
     , HijackT, HijackOptions(..)
     , runHijackT
     ) where
 
 import Darcs.Prelude
 
-import Darcs.Patch
-    ( IsRepoType, RepoPatch, PrimPatch, PrimOf
-    , summaryFL
-    )
+import Darcs.Patch ( PrimOf, RepoPatch, summaryFL )
 import Darcs.Patch.Apply ( ApplyState )
-import Darcs.Patch.Info ( PatchInfo,
-                          piAuthor, piName, piLog, piDateString,
-                          patchinfo
-                        )
+import Darcs.Patch.Info
+    ( PatchInfo
+    , patchinfo
+    , piAuthor
+    , piDateString
+    , piLog
+    , piName
+    )
 import Darcs.Patch.Named
-   ( Named, patchcontents, patch2patchinfo, infopatch, getdeps, adddeps
-   )
+    ( Named
+    , adddeps
+    , getdeps
+    , infopatch
+    , patch2patchinfo
+    , patchcontents
+    , setinfo
+    )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia )
 import Darcs.Patch.Prim ( canonizeFL )
 
-import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+) )
+import Darcs.Patch.Witnesses.Ordered ( FL(..), RL(..), (+>+) )
 
-import Darcs.Repository ( Repository )
-import Darcs.Util.Lock
-    ( readTextFile
-    , writeTextFile
-    )
+import Darcs.Util.Lock ( readTextFile, writeTextFile )
 
 import Darcs.UI.External ( editFile )
 import Darcs.UI.Flags ( getEasyAuthor, promptAuthor, getDate )
+import Darcs.UI.Options ( Config, (?) )
 import qualified Darcs.UI.Options.All as O
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions(..) )
 import Darcs.UI.SelectChanges ( askAboutDepends )
@@ -42,7 +49,7 @@ import Darcs.Util.English ( capitalize )
 import Darcs.Util.Global ( darcsLastMessage )
 import Darcs.Util.Path ( FilePathLike, toFilePath )
 import Darcs.Util.Prompt ( PromptConfig(..), askUser, promptChar, promptYorn )
-import Darcs.Util.Printer ( text, ($$), vcat, prefixLines, renderString )
+import Darcs.Util.Printer ( Doc, text, ($+$), vcat, prefixLines, renderString )
 import qualified Darcs.Util.Ratified as Ratified ( hGetContents )
 
 import Darcs.Util.Tree ( Tree )
@@ -81,13 +88,12 @@ type HijackT = StateT HijackOptions
 -- It ensures the patch name is not empty nor starts with the prefix TAG.
 --
 -- The last result component is a possible path to a temporary file that should be removed later.
-getLog :: forall prim wX wY . PrimPatch prim
-       => Maybe String                          -- ^ patchname option
+getLog :: Maybe String                          -- ^ patchname option
        -> Bool                                  -- ^ pipe option
        -> O.Logfile                             -- ^ logfile option
        -> Maybe O.AskLongComment                -- ^ askLongComment option
        -> Maybe (String, [String])              -- ^ possibly an existing patch name and long description
-       -> FL prim wX wY                         -- ^ changes to record
+       -> Doc                                   -- ^ summary of changes to record
        -> IO (String, [String], Maybe String)   -- ^ patch name, long description and possibly the path
                                                 --   to the temporary file that should be removed later
 getLog m_name has_pipe log_file ask_long m_old chs =
@@ -206,19 +212,33 @@ getLog m_name has_pipe log_file ask_long m_old chs =
   append_info f oldname = do
     fc <- readTextFile f
     writeTextFile f $ renderString
-       $ vcat (map text $ if null fc then [oldname] else fc)
-      $$ text "# Please enter the patch name in the first line, and"
-      $$ text "# optionally, a long description in the following lines."
-      $$ text "#"
-      $$ text "# Lines starting with '#' will be ignored."
-      $$ text "#"
-      $$ text "#"
-      $$ text "# Summary of selected changes:"
-      $$ text "#"
-      $$ prefixLines (text "#") (summaryFL chs)
+      $ vcat (map text $ if null fc then [oldname] else fc)
+      $+$ vcat
+        [ text "# Please enter the patch name in the first line, and"
+        , text "# optionally, a long description in the following lines."
+        , text "#"
+        , text "# Lines starting with '#' will be ignored."
+        , text "#"
+        , text "#"
+        , text "# Summary of selected changes:"
+        , text "#"
+        , prefixLines (text "#") chs
+        ]
 
--- |specify whether to ask about dependencies with respect to a particular repository, or not
-data AskAboutDeps rt p wR wU wT = AskAboutDeps (Repository rt p wR wU wT) | NoAskAboutDeps
+editLog :: Named prim wX wY -> IO (Named prim wX wY)
+editLog p = do
+  let pi = patch2patchinfo p
+  (name, log, _) <-
+    getLog Nothing False (O.Logfile Nothing False)
+      (Just O.YesEditLongComment) (Just (piName pi, piLog pi)) mempty
+  pi' <- patchinfo (piDateString pi) name (piAuthor pi) log
+  return $ setinfo pi' p
+
+-- | Specify whether to ask about dependencies with respect to a particular
+-- 'PatchSet', or not
+data AskAboutDeps p wX where
+  AskAboutDeps :: (RL (PatchInfoAnd p) w wX) -> AskAboutDeps p wX
+  NoAskAboutDeps :: AskAboutDeps p wX
 
 -- | Run a job that involves a hijack confirmation prompt.
 --
@@ -226,42 +246,59 @@ data AskAboutDeps rt p wR wU wT = AskAboutDeps (Repository rt p wR wU wT) | NoAs
 runHijackT :: Monad m => HijackOptions -> HijackT m a -> m a
 runHijackT = flip evalStateT
 
+data PatchHeaderConfig = PatchHeaderConfig
+  { diffAlgorithm :: D.DiffAlgorithm
+  , keepDate :: Bool
+  , selectAuthor :: Bool
+  , author :: Maybe String
+  , patchname :: Maybe String
+  , askLongComment :: Maybe O.AskLongComment
+  }
+
+patchHeaderConfig :: Config -> PatchHeaderConfig
+patchHeaderConfig cfg = PatchHeaderConfig
+  { diffAlgorithm   = O.diffAlgorithm ? cfg
+  , keepDate        = O.keepDate ? cfg
+  , selectAuthor    = O.selectAuthor ? cfg
+  , author          = O.author ? cfg
+  , patchname       = O.patchname ? cfg
+  , askLongComment  = O.askLongComment ? cfg
+  }
+
 -- | Update the metadata for a patch.
 --   This potentially involves a bit of interactivity, so we may return @Nothing@
 --   if there is cause to abort what we're doing along the way
-updatePatchHeader :: forall rt p wX wY wR wU wT . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+updatePatchHeader :: forall p wX wY wZ . (RepoPatch p, ApplyState p ~ Tree)
                   => String -- ^ verb: command name
-                  -> AskAboutDeps rt p wR wU wT
+                  -> AskAboutDeps p wX
                   -> S.PatchSelectionOptions
-                  -> D.DiffAlgorithm
-                  -> Bool -- keepDate
-                  -> Bool -- selectAuthor
-                  -> Maybe String -- author
-                  -> Maybe String -- patchname
-                  -> Maybe O.AskLongComment
-                  -> Named (PrimOf p) wT wX
-                  -- ^ patch to edit, must be conflict-free as conflicts can't be preserved when changing
-                  -- the identity of a patch. If necessary this can be achieved by calling @fmapFL_Named effect@
-                  -- on an @Named p@ first, but some callers might already have @Named (PrimOf p)@ available.
-                  -> FL (PrimOf p) wX wY -- ^new primitives to add
-                  -> HijackT IO (Maybe String, PatchInfoAnd rt p wT wY)
-updatePatchHeader verb ask_deps pSelOpts da nKeepDate nSelectAuthor nAuthor nPatchname nAskLongComment oldp chs = do
+                  -> PatchHeaderConfig
+                  -> Named (PrimOf p) wX wY
+                  -- ^ patch to edit, must be conflict-free as conflicts can't
+                  -- be preserved when changing the identity of a patch. If
+                  -- necessary this can be achieved by calling @fmapFL_Named
+                  -- effect@ on an @Named p@ first, but some callers might
+                  -- already have @Named (PrimOf p)@ available.
+                  -> FL (PrimOf p) wY wZ -- ^new primitives to add
+                  -> HijackT IO (Maybe String, PatchInfoAnd p wX wZ)
+updatePatchHeader verb ask_deps pSelOpts PatchHeaderConfig{..} oldp chs = do
 
-    let newchs = canonizeFL da (patchcontents oldp +>+ chs)
+    let newchs = canonizeFL diffAlgorithm (patchcontents oldp +>+ chs)
 
     let old_pdeps = getdeps oldp
     newdeps <-
         case ask_deps of
-           AskAboutDeps repository -> liftIO $ askAboutDepends repository newchs pSelOpts old_pdeps
+           AskAboutDeps patches ->
+              liftIO $ askAboutDepends patches newchs pSelOpts old_pdeps
            NoAskAboutDeps -> return old_pdeps
 
     let old_pinf = patch2patchinfo oldp
         prior    = (piName old_pinf, piLog old_pinf)
-    date <- if nKeepDate then return (piDateString old_pinf) else liftIO $ getDate False
-    new_author <- getAuthor verb nSelectAuthor nAuthor old_pinf
+    date <- if keepDate then return (piDateString old_pinf) else liftIO $ getDate False
+    new_author <- getAuthor verb selectAuthor author old_pinf
     liftIO $ do
         (new_name, new_log, mlogf) <- getLog
-            nPatchname False (O.Logfile Nothing False) nAskLongComment (Just prior) chs
+            patchname False (O.Logfile Nothing False) askLongComment (Just prior) (summaryFL chs)
         new_pinf <- patchinfo date new_name new_author new_log
         let newp = n2pia (adddeps (infopatch new_pinf newchs) newdeps)
         return (mlogf, newp)
