@@ -26,6 +26,7 @@ module Darcs.UI.RunCommand
 import Darcs.Prelude
 
 import Control.Monad ( unless, when )
+import Data.List ( intercalate )
 import System.Console.GetOpt( ArgOrder( Permute, RequireOrder ),
                               OptDescr( Option ),
                               getOpt )
@@ -34,12 +35,12 @@ import System.Exit ( ExitCode ( ExitSuccess ), exitWith )
 import Darcs.UI.Options ( (^), odesc, oparse, parseFlags, optDescr, (?) )
 import Darcs.UI.Options.All
     ( stdCmdActions, StdCmdAction(..)
-    , debugging, verbosity, Verbosity(..), network, NetworkOptions(..)
+    , debugging, verbosity, Verbosity(..)
     , HooksConfig(..), hooks )
 
 import Darcs.UI.Defaults ( applyDefaults )
 import Darcs.UI.External ( viewDoc )
-import Darcs.UI.Flags ( DarcsFlag, matchAny, fixRemoteRepos, withNewRepo )
+import Darcs.UI.Flags ( DarcsFlag, matchAny, withNewRepo )
 import Darcs.UI.Commands
     ( CommandArgs( CommandOnly, SuperCommandOnly, SuperCommandSub )
     , CommandControl
@@ -51,7 +52,7 @@ import Darcs.UI.Commands
     , commandExtraArgs
     , commandArgdefaults
     , commandCompleteArgs
-    , commandOptions
+    , commandOptDescr
     , commandName
     , disambiguateCommands
     , getSubcommands
@@ -60,6 +61,7 @@ import Darcs.UI.Commands
     )
 import Darcs.UI.Commands.GZCRCs ( doCRCWarnings )
 import Darcs.UI.Commands.Clone ( makeRepoName, cloneToSSH )
+import Darcs.UI.RunHook ( runPosthook, runPrehook )
 import Darcs.UI.Usage
     ( getCommandHelp
     , getCommandMiniHelp
@@ -67,10 +69,8 @@ import Darcs.UI.Usage
     )
 
 import Darcs.Patch.Match ( checkMatchSyntax )
-import Darcs.Repository.Prefs ( getGlobal, getPreflist )
-import Darcs.Repository.Test ( runPosthook, runPrehook )
+import Darcs.Repository.Prefs ( Pref(Defaults), getGlobal, getPreflist )
 import Darcs.Util.AtExit ( atexit )
-import Darcs.Util.Download ( setDebugHTTP, disableHTTPPipelining )
 import Darcs.Util.Exception ( die )
 import Darcs.Util.Global ( setDebugMode, setTimingsMode )
 import Darcs.Util.Path ( AbsolutePath, getCurrentDirectory, toPath, ioAbsoluteOrRemote, makeAbsolute )
@@ -92,7 +92,7 @@ runCommand _ _ args -- Check for "dangerous" typoes...
         die "Are you sure you didn't mean --all rather than -all?"
 runCommand msuper cmd args = do
   old_wd <- getCurrentDirectory
-  let options = commandOptions old_wd cmd
+  let options = commandOptDescr old_wd cmd
   case fixupMsgs $ getOpt Permute options args of
     (cmdline_flags,orig_extra,getopt_errs) -> do
       -- FIXME This code is highly order-dependent because of hidden state: the
@@ -105,9 +105,9 @@ runCommand msuper cmd args = do
       prereq_errors <- commandPrereq cmd cmdline_flags
       -- we must get the cwd again because commandPrereq has the side-effect of changing it.
       new_wd <- getCurrentDirectory
-      user_defs <- getGlobal   "defaults"
-      repo_defs <- getPreflist "defaults"
-      let (flags,flag_errors) =
+      user_defs <- getGlobal Defaults
+      repo_defs <- getPreflist Defaults
+      let (flags, (flag_warnings, flag_errors)) =
             applyDefaults (fmap commandName msuper) cmd old_wd user_defs repo_defs cmdline_flags
       case parseFlags stdCmdActions flags of
         Just Help -> viewDoc $ getCommandHelp msuper cmd
@@ -122,11 +122,14 @@ runCommand msuper cmd args = do
             "Unable to '" ++ "darcs " ++ superName msuper ++ commandName cmd ++
             "' here:\n" ++ complaint
           Right () -> do
-            ePutDocLn $ vcat $ map text $ getopt_errs ++ flag_errors
-            extra <- commandArgdefaults cmd flags old_wd orig_extra
-            case extraArgumentsError extra cmd msuper of
-              Nothing     -> runWithHooks cmd (new_wd, old_wd) flags extra
-              Just msg    -> die msg
+            ePutDocLn $ vcat $ map text $ flag_warnings
+            case getopt_errs ++ flag_errors of
+              [] -> do
+                extra <- commandArgdefaults cmd flags old_wd orig_extra
+                case extraArgumentsError extra cmd msuper of
+                  Nothing     -> runWithHooks cmd (new_wd, old_wd) flags extra
+                  Just msg    -> die msg
+              errors -> fail $ intercalate "\n" errors
 
 fixupMsgs :: (a, b, [String]) -> (a, b, [String])
 fixupMsgs (fs,as,es) = (fs,as,map (("command line: "++).chompTrailingNewline) es)
@@ -140,26 +143,23 @@ runWithHooks :: DarcsCommand
 runWithHooks cmd (new_wd, old_wd) flags extra = do
    checkMatchSyntax $ matchAny ? flags
    -- set any global variables
-   oparse (verbosity ^ debugging ^ network) setGlobalVariables flags
+   oparse (verbosity ^ debugging) setGlobalVariables flags
    -- actually run the command and its hooks
    let hooksCfg = parseFlags hooks flags
    let verb = parseFlags verbosity flags
    preHookExitCode <- runPrehook (pre hooksCfg) verb new_wd
    if preHookExitCode /= ExitSuccess
       then exitWith preHookExitCode
-      else do fixedFlags <- fixRemoteRepos old_wd flags
-              phDir <- getPosthookDir new_wd cmd fixedFlags extra
-              commandCommand cmd (new_wd, old_wd) fixedFlags extra
+      else do phDir <- getPosthookDir new_wd cmd flags extra
+              commandCommand cmd (new_wd, old_wd) flags extra
               postHookExitCode <- runPosthook (post hooksCfg) verb phDir
               exitWith postHookExitCode
 
-setGlobalVariables :: Verbosity -> Bool -> Bool -> Bool -> NetworkOptions -> IO ()
-setGlobalVariables verb debug debugHttp timings net = do
+setGlobalVariables :: Verbosity -> Bool -> Bool -> IO ()
+setGlobalVariables verb debug timings = do
   when timings setTimingsMode
   when debug setDebugMode
-  when debugHttp setDebugHTTP
   when (verb == Quiet) $ setProgressMode False
-  when (noHttpPipelining net) disableHTTPPipelining
   unless (verb == Quiet) $ atexit $ doCRCWarnings (verb == Verbose)
 
 -- | Returns the working directory for the posthook. For most commands, the
