@@ -40,7 +40,7 @@ import Darcs.UI.Commands.Record ( record, commit )
 import Darcs.UI.Commands.Remove ( remove, rm, unadd )
 import Darcs.UI.Commands.Repair ( repair, check )
 import Darcs.UI.Commands.Replace ( replace )
-import Darcs.UI.Commands.Revert ( revert )
+import Darcs.UI.Commands.Revert ( revert, clean )
 import Darcs.UI.Commands.Rollback ( rollback )
 import Darcs.UI.Commands.Send ( send )
 import Darcs.UI.Commands.SetPref ( setpref )
@@ -83,6 +83,7 @@ commandControlList =
     , normalCommand rollback
     , normalCommand unrecord
     , normalCommand obliterate, hiddenCommand unpull
+    , normalCommand clean
     , commandGroup "Direct modification of the repository:"
     , normalCommand tag
     , normalCommand setpref
