@@ -2,15 +2,20 @@
 
 module Darcs.Util.Compat
     ( stdoutIsAPipe
-    , canonFilename
     , maybeRelink
     , atomicCreate
     , sloppyAtomicCreate
     ) where
 
-import Darcs.Prelude
+#ifdef WIN32
+#define USE_CREAT
+#else
+#if MIN_VERSION_unix(2,8,0)
+#define USE_CREAT
+#endif
+#endif
 
-import Darcs.Util.File ( withCurrentDirectory )
+import Darcs.Prelude
 
 import Control.Monad ( unless )
 import Foreign.C.Types ( CInt(..) )
@@ -20,25 +25,14 @@ import System.Directory ( getCurrentDirectory )
 import System.IO.Error ( mkIOError, alreadyExistsErrorType )
 import System.Posix.Files ( stdFileMode )
 import System.Posix.IO ( openFd, closeFd,
+#ifdef USE_CREAT
+                         creat,
+#endif
                          defaultFileFlags, exclusive,
                          OpenMode(WriteOnly) )
 
 import Darcs.Util.SignalHandler ( stdoutIsAPipe )
 
-canonFilename :: FilePath -> IO FilePath
-canonFilename f@(_:':':_) = return f -- absolute windows paths
-canonFilename f@('/':_) = return f
-canonFilename ('.':'/':f) = do cd <- getCurrentDirectory
-                               return $ cd ++ "/" ++ f
-canonFilename f = case reverse $ dropWhile (/='/') $ reverse f of
-                  "" -> fmap (++('/':f)) getCurrentDirectory
-                  rd -> withCurrentDirectory rd $
-                          do fd <- getCurrentDirectory
-                             return $ fd ++ "/" ++ simplefilename
-    where
-    simplefilename = reverse $ takeWhile (/='/') $ reverse f
-
-
 foreign import ccall unsafe "maybe_relink.h maybe_relink" maybe_relink
     :: CString -> CString -> CInt -> IO CInt
 
@@ -63,7 +57,11 @@ maybeRelink src dst =
 
 sloppyAtomicCreate :: FilePath -> IO ()
 sloppyAtomicCreate fp
+#ifdef USE_CREAT
+    = do fd <- openFd fp WriteOnly flags {creat = Just stdFileMode}
+#else
     = do fd <- openFd fp WriteOnly (Just stdFileMode) flags
+#endif
          closeFd fd
   where flags = defaultFileFlags { exclusive = True }
 
