@@ -58,7 +58,6 @@ module Darcs.Util.Diff.Myers
     , getSlice
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Control.Monad
@@ -134,7 +133,7 @@ getChanges' (a, abounds) (b, bbounds) =
                       Just (_,_,False,_) -> Nothing
                       Just (False,True,True,_) -> Just (i, h)
                       Just (True,True,True,_) -> Just (i, markColl)
-                      Nothing -> impossible
+                      Nothing -> error "impossible case"
 
         a' = mapMaybe get [(i, ah!i) | i <- range (bounds ah)]
         b' = mapMaybe get [(i, bh!i) | i <- range (bounds bh)]
@@ -163,7 +162,7 @@ diffArr a b (p_a, (off_a, l_a)) (p_b, (off_b, l_b)) = runST (
      _ <- cmpseq h_a h_b p_a p_b m_a m_b c_a c_b 0 0 (aLen h_a) (aLen h_b)
      let unchanged ar = do {xs <- getElems ar; return $ length (filter not xs) -1}
      err <- liftM2 (/=) (unchanged c_a) (unchanged c_b)
-     when err impossible
+     when err $ error "impossible case"
      -- Mark common lines at beginning and end
      mapM_ (\ i -> writeArray c_a i False ) [1..(off_a - 1)]
      mapM_ (\ i -> writeArray c_b i False ) [1..(off_b - 1)]
@@ -172,7 +171,7 @@ diffArr a b (p_a, (off_a, l_a)) (p_b, (off_b, l_b)) = runST (
      shiftBoundaries c_a c_b p_a 1 1
      shiftBoundaries c_b c_a p_b 1 1
      err1 <- liftM2 (/=) (unchanged c_a) (unchanged c_b)
-     when err1 impossible
+     when err1 $ error "impossible case"
      c_a' <- unsafeFreeze c_a
      c_b' <- unsafeFreeze c_b
      return (c_a', c_b'))
@@ -211,7 +210,7 @@ cmpseq h_a h_b p_a p_b m_a m_b c_a c_b off_a off_b l_a l_b = do
                                 off_a' off_b' l_a' l_b' del dodd
              when ((xmid == 0 && ymid == 0) || (xmid == l_a' && ymid == l_b')
                    || (xmid < 0 || ymid < 0 || xmid > l_a' || ymid > l_b'))
-                     impossible
+                     $ error "impossible case"
              c1 <- cmpseq h_a h_b p_a p_b m_a m_b c_a c_b
                           off_a' off_b' xmid ymid
              c2 <- cmpseq h_a h_b p_a p_b m_a m_b c_a c_b
@@ -333,10 +332,10 @@ shiftBoundaries c_a c_b p_a i_ j_ =
                 else return (i2, j2)
        shiftBackward start i j =
          if start > 1 && p_a!(i-1) == p_a!(start-1)
-            then do when (i == start) impossible
+            then do when (i == start) $ error "impossible case"
                     b1 <- readArray c_a (i-1)
                     b2 <- readArray c_a (start-1)
-                    when (not b1 || b2) impossible
+                    when (not b1 || b2) $ error "impossible case"
                     writeArray c_a (i-1) False
                     writeArray c_a (start-1) True
                     b <- if start > 2 then readArray c_a (start-2)
@@ -350,10 +349,10 @@ shiftBoundaries c_a c_b p_a i_ j_ =
          if i <= aLen p_a && p_a!i == p_a!start &&
              -- B.empty at the end of file marks empty line after final newline
              not ((i == aLen p_a) && (p_a!i == B.empty))
-            then do when (i == start) impossible
+            then do when (i == start) $ error "impossible case"
                     b1 <- readArray c_a i
                     b2 <- readArray c_a start
-                    when (not b2 ||  b1) impossible
+                    when (not b2 ||  b1) $ error "impossible case"
                     writeArray c_a i True
                     writeArray c_a start False
                     i0 <- nextUnchanged c_a (i+1)
@@ -373,8 +372,8 @@ shiftBoundaries c_a c_b p_a i_ j_ =
             then return (start,i,j)
             else do b1 <- readArray c_a (i-1)
                     b2 <- readArray c_a (start-1)
-                    when (not b1 || b2) impossible
-                    when (p_a!(i-1) /= p_a!(start-1)) impossible
+                    when (not b1 || b2) $ error "impossible case"
+                    when (p_a!(i-1) /= p_a!(start-1)) $ error "impossible case"
                     writeArray c_a (i-1) False
                     writeArray c_a (start-1) True
                     j' <- prevUnchanged c_b (j-1)
