@@ -46,15 +46,13 @@ module Darcs.Util.Path
     , makeSubPathOf
     , simpleSubPath
     , floatSubPath
+    , makeRelativeTo
     -- * Miscellaneous
     , FilePathOrURL(..)
     , FilePathLike(toFilePath)
     , getCurrentDirectory
     , setCurrentDirectory
     , getUniquePathName
-    , doesPathExist
-    -- * Check for malicious paths
-    , isMaliciousSubPath
     -- * Tree filtering.
     , filterPaths
     -- * AnchoredPaths: relative paths within a Tree. All paths are
@@ -70,7 +68,6 @@ module Darcs.Util.Path
     , appendPath
     , anchorPath
     , isPrefix
-    , breakOnDir
     , movedirfilename
     , parent
     , parents
@@ -82,40 +79,32 @@ module Darcs.Util.Path
     , realPath
     , isRoot
     , darcsdirName
-    -- * Unsafe AnchoredPath functions.
     , floatPath
+    -- * Unsafe AnchoredPath functions.
+    , unsafeFloatPath
     ) where
 
 import Darcs.Prelude
 
-import Data.List
-    ( isPrefixOf
-    , isSuffixOf
-    , stripPrefix
-    , intersect
-    , inits
-    )
-import Data.Char ( isSpace, chr, ord, toLower )
-import Data.Typeable ( Typeable )
-import Control.Exception ( tryJust, bracket_, throw, Exception )
-import Control.Monad ( when )
-import System.IO.Error ( isDoesNotExistError )
-
-import qualified Darcs.Util.Workaround as Workaround ( getCurrentDirectory )
-import qualified System.Directory ( setCurrentDirectory )
-import System.Directory ( doesDirectoryExist, doesFileExist )
-import qualified System.FilePath.Posix as FilePath ( (</>), normalise, isRelative )
-import qualified System.FilePath as NativeFilePath ( takeFileName, takeDirectory )
-import System.FilePath( splitDirectories, normalise, dropTrailingPathSeparator )
-import System.Posix.Files ( isDirectory, getSymbolicLinkStatus )
-
-import Darcs.Util.ByteString ( encodeLocale, decodeLocale )
-import qualified Data.ByteString.Char8 as BC
-import qualified Data.ByteString       as B
-
+import Control.Exception ( bracket_ )
+import Control.Monad ( when, (<=<) )
+import Darcs.Util.ByteString ( decodeLocale, encodeLocale )
 import Data.Binary
+import qualified Data.ByteString as B
+import qualified Data.ByteString.Char8 as BC
+import Data.Char ( chr, isSpace, ord, toLower )
+import Data.List ( inits, isPrefixOf, isSuffixOf, stripPrefix )
+import GHC.Stack ( HasCallStack )
+import qualified System.Directory ( setCurrentDirectory )
+import System.Directory ( doesDirectoryExist, doesPathExist )
+import qualified System.FilePath as NativeFilePath
+import qualified System.FilePath.Posix as FilePath
+import System.Posix.Files ( fileID, getFileStatus, isDirectory )
+
+import Darcs.Util.Exception ( ifDoesNotExistError )
 import Darcs.Util.Global ( darcsdir )
-import Darcs.Util.URL ( isAbsolute, isRelative, isSshNopath )
+import Darcs.Util.URL ( isAbsolute, isHttpUrl, isRelative, isSshNopath, isSshUrl )
+import qualified Darcs.Util.Workaround as Workaround ( getCurrentDirectory )
 
 
 -- Utilities for use by command implementations
@@ -151,18 +140,18 @@ encodeWhite [] = []
 -- | 'decodeWhite' interprets the Darcs-specific \"encoded\" filenames
 --   produced by 'encodeWhite'
 --
---   > decodeWhite "hello\32\there"  == "hello there"
---   > decodeWhite "hello\92\there"  == "hello\there"
---   > decodeWhite "hello\there"   == error "malformed filename"
-decodeWhite :: String -> FilePath
+--   > decodeWhite "hello\32\there"  == Right "hello there"
+--   > decodeWhite "hello\92\there"  == Right "hello\there"
+--   > decodeWhite "hello\there"   == Left "malformed filename"
+decodeWhite :: String -> Either String FilePath
 decodeWhite cs_ = go cs_ [] False
- where go "" acc True  = reverse acc -- if there was a replace, use new string
-       go "" _   False = cs_         -- if not, use input string
+ where go "" acc True  = Right (reverse acc) -- if there was a replace, use new string
+       go "" _   False = Right cs_         -- if not, use input string
        go ('\\':cs) acc _ =
          case break (=='\\') cs of
            (theord, '\\':rest) ->
              go rest (chr (read theord) :acc) True
-           _ -> error "malformed filename"
+           _ -> Left $ "malformed filename: " ++ cs_
        go (c:cs) acc modified = go cs (c:acc) modified
 
 class FilePathOrURL a where
@@ -187,28 +176,21 @@ instance FilePathOrURL AbsolutePath where
   toPath (AbsolutePath x) = x
 instance FilePathOrURL SubPath where
   toPath (SubPath x) = x
-instance CharLike c => FilePathOrURL [c] where
-  toPath = toFilePath
-
 instance FilePathOrURL AbsoluteOrRemotePath where
   toPath (AbsP a) = toPath a
   toPath (RmtP r) = r
+instance FilePathOrURL FilePath where
+  toPath = id
 
 instance FilePathLike AbsolutePath where
   toFilePath (AbsolutePath x) = x
 instance FilePathLike SubPath where
   toFilePath (SubPath x) = x
+instance FilePathLike FilePath where
+  toFilePath = id
 
-class CharLike c where
-  toChar :: c -> Char
-
-instance CharLike Char where
-  toChar = id
-
-instance CharLike c => FilePathLike [c] where
-  toFilePath = map toChar
-
--- | Make the second path relative to the first, if possible
+-- | Make the second path relative to the first, if possible.
+-- Note that this returns an empty 'SubPath' if the inputs are equal.
 makeSubPathOf :: AbsolutePath -> AbsolutePath -> Maybe SubPath
 makeSubPathOf (AbsolutePath p1) (AbsolutePath p2) =
  -- The slash prevents "foobar" from being treated as relative to "foo"
@@ -216,30 +198,16 @@ makeSubPathOf (AbsolutePath p1) (AbsolutePath p2) =
     then Just $ SubPath $ drop (length p1 + 1) p2
     else Nothing
 
-simpleSubPath :: FilePath -> Maybe SubPath
+simpleSubPath :: HasCallStack => FilePath -> Maybe SubPath
 simpleSubPath x | null x = error "simpleSubPath called with empty path"
                 | isRelative x = Just $ SubPath $ FilePath.normalise $ pathToPosix x
                 | otherwise = Nothing
 
--- | Ensure directory exists and is not a symbolic link.
-doesDirectoryReallyExist :: FilePath -> IO Bool
-doesDirectoryReallyExist f = do
-    x <- tryJust (\x -> if isDoesNotExistError x then Just () else Nothing) $
-        isDirectory <$> getSymbolicLinkStatus f
-    return $ case x of
-        Left () -> False
-        Right y -> y
-
-doesPathExist :: FilePath -> IO Bool
-doesPathExist p = do
-   dir_exists <- doesDirectoryExist p
-   file_exists <- doesFileExist p
-   return $ dir_exists || file_exists
-
 -- | Interpret a possibly relative path wrt the current working directory.
+-- This also canonicalizes the path, resolving symbolic links etc.
 ioAbsolute :: FilePath -> IO AbsolutePath
 ioAbsolute dir =
-    do isdir <- doesDirectoryReallyExist dir
+    do isdir <- doesDirectoryExist dir
        here <- getCurrentDirectory
        if isdir
          then bracket_ (setCurrentDirectory dir)
@@ -254,6 +222,36 @@ ioAbsolute dir =
                                else ioAbsolute super_dir
                     return $ makeAbsolute abs_dir file
 
+-- | The first argument must be the absolute path of a @directory@, the second
+-- is an arbitrary absolute @path@. Find the longest prefix of @path@ that
+-- points to the same @directory@; if there is none, return 'Nothing', else
+-- return 'Just' the remainder.
+makeRelativeTo :: HasCallStack => AbsolutePath -> AbsolutePath -> IO (Maybe SubPath)
+makeRelativeTo (AbsolutePath dir) (AbsolutePath path) = do
+  dir_stat <- getFileStatus dir
+  let dir_id = fileID dir_stat
+  when (not (isDirectory dir_stat)) $
+    error $ "makeRelativeTo called with non-dir " ++ dir
+  findParent dir_id path []
+  where
+    findParent dir_id ap acc = do
+      map_stat <- ifDoesNotExistError Nothing (Just <$> getFileStatus ap)
+      case map_stat of
+        Just ap_stat | fileID ap_stat == dir_id -> do
+          -- found ancestor that matches dir
+          return $ Just $ SubPath $ FilePath.joinPath acc
+        _ -> do
+          -- recurse
+          let (parent_,child) =
+                -- splitFileName only does what one expects if there is no
+                -- trailing path separator
+                NativeFilePath.splitFileName $
+                  NativeFilePath.dropTrailingPathSeparator ap
+          if null child then
+            return Nothing
+          else
+            findParent dir_id parent_ (child:acc)
+
 -- | Take an absolute path and a string representing a (possibly relative)
 -- path and combine them into an absolute path. If the second argument is
 -- already absolute, then the first argument gets ignored. This function also
@@ -339,11 +337,13 @@ instance Show AbsoluteOrRemotePath where
 -- | Normalize the path separator to Posix style (slash, not backslash).
 -- This only affects Windows systems.
 pathToPosix :: FilePath -> FilePath
-pathToPosix = map convert where
 #ifdef WIN32
+pathToPosix = map convert where
   convert '\\' = '/'
-#endif
   convert c = c
+#else
+pathToPosix = id
+#endif
 
 -- | Reduce multiple leading slashes to one. This only affects Posix systems.
 normSlashes :: FilePath -> FilePath
@@ -356,48 +356,11 @@ normSlashes p = p
 getCurrentDirectory :: IO AbsolutePath
 getCurrentDirectory = AbsolutePath `fmap` Workaround.getCurrentDirectory
 
-setCurrentDirectory :: FilePathLike p => p -> IO ()
-setCurrentDirectory = System.Directory.setCurrentDirectory . toFilePath
-
-{-|
-  What is a malicious path?
-
-  A spoofed path is a malicious path.
-
-  1. Darcs only creates explicitly relative paths (beginning with @\".\/\"@),
-     so any not explicitly relative path is surely spoofed.
-
-  2. Darcs normalizes paths so they never contain @\"\/..\/\"@, so paths with
-     @\"\/..\/\"@ are surely spoofed.
-
-  A path to a darcs repository's meta data can modify \"trusted\" patches or
-  change safety defaults in that repository, so we check for paths
-  containing @\"\/_darcs\/\"@ which is the entry to darcs meta data.
-
-  To do?
-
-  * How about get repositories?
-
-  * Would it be worth adding a --semi-safe-paths option for allowing
-    changes to certain preference files (_darcs\/prefs\/) in sub
-    repositories'?
-
-  TODO:
-    Properly review the way we handle paths on Windows - it's not enough
-    to just use the OS native concept of path separator. Windows often
-    accepts both path separators, and repositories always use the UNIX
-    separator anyway.
--}
-
-isMaliciousSubPath :: String -> Bool
-isMaliciousSubPath fp =
-    not (FilePath.isRelative fp) || isGenerallyMalicious fp
-
-isGenerallyMalicious :: String -> Bool
-isGenerallyMalicious fp =
-    splitDirectories fp `contains_any` [ "..", darcsdir ]
- where
-    contains_any a b = not . null $ intersect a b
+setCurrentDirectory :: HasCallStack => FilePathLike p => p -> IO ()
+setCurrentDirectory path
+  | isHttpUrl (toFilePath path) || isSshUrl (toFilePath path) =
+    error $ "setCurrentDirectory " ++ toFilePath path
+setCurrentDirectory path = System.Directory.setCurrentDirectory (toFilePath path)
 
 -- | Iteratively tries find first non-existing path generated by
 -- buildName, it feeds to buildName the number starting with -1.  When
@@ -428,7 +391,7 @@ newtype Name = Name { unName :: B.ByteString } deriving (Binary, Eq, Show, Ord)
 -- usually used to refer to a location within a Tree, but a relative filesystem
 -- path works just as well. These are either constructed from individual name
 -- components (using "appendPath", "catPaths" and "makeName"), or converted
--- from a FilePath ("floatPath" -- but take care when doing that).
+-- from a FilePath ("unsafeFloatPath" -- but take care when doing that).
 newtype AnchoredPath = AnchoredPath [Name] deriving (Binary, Eq, Show, Ord)
 
 -- | Check whether a path is a prefix of another path.
@@ -450,20 +413,11 @@ parent :: AnchoredPath -> Maybe AnchoredPath
 parent (AnchoredPath []) = Nothing
 parent (AnchoredPath x) = Just (AnchoredPath (init x))
 
--- | List all parents of a given path. foo/bar/baz -> [.,foo, foo/bar]
+-- | List all (proper) parents of a given path. foo/bar/baz -> [.,foo, foo/bar]
 parents :: AnchoredPath -> [AnchoredPath]
 parents (AnchoredPath []) = [] -- root has no parents
 parents (AnchoredPath xs) = map AnchoredPath $ inits $ init xs
 
--- | If the patch is under a directory, split into Right of the first component
--- (which must be a directory name) and the rest of teh path. Otherwise
--- return Left of the single component.
--- This function is *undefined* on the root path (which has no components).
-breakOnDir :: AnchoredPath -> Either Name (Name, AnchoredPath)
-breakOnDir (AnchoredPath []) = error "breakOnDir called on root"
-breakOnDir (AnchoredPath (n:[])) = Left n
-breakOnDir (AnchoredPath (n:ns)) = Right (n, AnchoredPath ns)
-
 -- | Take a "root" directory and an anchored path and produce a full
 -- 'FilePath'. Moreover, you can use @anchorPath \"\"@ to get a relative
 -- 'FilePath'.
@@ -479,31 +433,31 @@ flatten :: AnchoredPath -> BC.ByteString
 flatten (AnchoredPath []) = BC.singleton '.'
 flatten (AnchoredPath p) = BC.intercalate (BC.singleton '/') [n | (Name n) <- p]
 
--- | Make a 'Name' from a 'String'. If the input 'String'
--- is invalid, that is, "", ".", "..", or contains a '/', return 'Left'
--- with an error message.
+-- | Make a 'Name' from a 'String'. May fail if the input 'String'
+-- is invalid, that is, "", ".", "..", or contains a '/'.
 makeName :: String -> Either String Name
 makeName = rawMakeName . encodeLocale
 
--- | Make a 'Name' from a 'String'. If the input 'String'
--- is invalid, that is, "", ".", "..", or contains a '/', call error.
-internalMakeName :: String -> Name
-internalMakeName = either error id . rawMakeName . encodeLocale
-
 -- | Take a relative FilePath and turn it into an AnchoredPath. This is a
--- partial function. Basically, by using floatPath, you are testifying that the
+-- partial function. Basically, by using unsafeFloatPath, you are testifying that the
 -- argument is a path relative to some common root -- i.e. the root of the
 -- associated "Tree" object. In particular, the input path may not contain any
 -- ocurrences of "." or ".." after normalising. You should sanitize any
 -- FilePaths before you declare them "good" by converting into AnchoredPath
 -- (using this function), especially if the FilePath come from any external
 -- source (command line, file, environment, network, etc)
-floatPath :: FilePath -> AnchoredPath
-floatPath =
-    AnchoredPath . map internalMakeName . filter sensible .
-    splitDirectories . normalise . dropTrailingPathSeparator
+unsafeFloatPath :: HasCallStack => FilePath -> AnchoredPath
+unsafeFloatPath = either error id . floatPath
+
+floatPath :: FilePath -> Either String AnchoredPath
+floatPath path = do
+    r <- mapM makeName (prepare path)
+    return (AnchoredPath r)
   where
     sensible s = s `notElem` ["", "."]
+    prepare = filter sensible .
+      NativeFilePath.splitDirectories . NativeFilePath.normalise .
+      NativeFilePath.dropTrailingPathSeparator
 
 anchoredRoot :: AnchoredPath
 anchoredRoot = AnchoredPath []
@@ -545,14 +499,9 @@ eqAnycase (Name a) (Name b) = BC.map toLower a == BC.map toLower b
 encodeWhiteName :: Name -> B.ByteString
 encodeWhiteName = encodeLocale . encodeWhite . decodeLocale . unName
 
-data CorruptPatch = CorruptPatch String deriving (Eq, Typeable)
-instance Exception CorruptPatch
-instance Show CorruptPatch where show (CorruptPatch s) = s
-
-decodeWhiteName :: B.ByteString -> Name
+decodeWhiteName :: B.ByteString -> Either String Name
 decodeWhiteName =
-  either (throw . CorruptPatch) id .
-  rawMakeName . encodeLocale . decodeWhite . decodeLocale
+  rawMakeName . encodeLocale <=< decodeWhite . decodeLocale
 
 -- | The effect of renaming on paths.
 -- The first argument is the old path, the second is the new path,
@@ -570,9 +519,8 @@ movedirfilename (AnchoredPath old) newp@(AnchoredPath new) orig@(AnchoredPath pa
 filterPaths :: [AnchoredPath] -> AnchoredPath -> t -> Bool
 filterPaths files p _ = any (\x -> x `isPrefix` p || p `isPrefix` x) files
 
-
 -- | Transform a SubPath into an AnchoredPath.
-floatSubPath :: SubPath -> AnchoredPath
+floatSubPath :: SubPath -> Either String AnchoredPath
 floatSubPath = floatPath . toFilePath
 
 -- | Is the given path in (or equal to) the _darcs metadata directory?
@@ -581,7 +529,7 @@ inDarcsdir (AnchoredPath (x:_)) | x == darcsdirName = True
 inDarcsdir _ = False
 
 darcsdirName :: Name
-darcsdirName = internalMakeName darcsdir
+darcsdirName = either error id (makeName darcsdir)
 
 isRoot :: AnchoredPath -> Bool
 isRoot (AnchoredPath xs) = null xs
