@@ -1,6 +1,6 @@
 {-# LANGUAGE CPP #-}
 module Darcs.Util.Printer.Color
-    ( unsafeRenderStringColored, traceDoc, debugDoc, fancyPrinters
+    ( unsafeRenderStringColored, traceDoc, fancyPrinters
     , environmentHelpColor, environmentHelpEscape, environmentHelpEscapeWhite
     , ePutDocLn
     ) where
@@ -14,7 +14,6 @@ import Darcs.Util.Printer
     , renderStringWith, prefix
     , hPutDocLnWith
     )
-import Darcs.Util.Global ( whenDebugMode, putTiming )
 
 import Debug.Trace ( trace )
 import Data.Char ( isAscii, isPrint, isSpace, isControl, ord, chr )
@@ -42,11 +41,6 @@ cr     = unsafeBothText "\r"
 ePutDocLn :: Doc -> IO ()
 ePutDocLn = hPutDocLnWith fancyPrinters stderr
 
-debugDoc :: Doc -> IO ()
-debugDoc m = whenDebugMode $ do
-  putTiming
-  hPutDocLnWith fancyPrinters stderr m
-
 traceDoc :: Doc -> a -> a
 traceDoc = trace . unsafeRenderStringColored
 
