@@ -13,6 +13,8 @@ module Darcs.Util.Progress
       beginTedious
     , endTedious
     , tediousSize
+    , withProgress
+    , withSizedProgress
     , debugMessage
     , withoutProgress
     , progress
@@ -29,7 +31,7 @@ import Darcs.Prelude
 
 import Control.Arrow ( second )
 import Control.Exception ( bracket )
-import Control.Monad ( when, unless, void )
+import Control.Monad ( when, void )
 import Control.Concurrent ( forkIO, threadDelay )
 
 import Data.Char ( toLower )
@@ -37,12 +39,11 @@ import Data.Map ( Map, empty, adjust, insert, delete, lookup )
 import Data.Maybe ( isJust )
 import Data.IORef ( IORef, newIORef, readIORef, writeIORef, modifyIORef )
 
-import System.IO ( stdout, stderr, hFlush, hPutStr, hPutStrLn,
-                   hSetBuffering, hIsTerminalDevice,
-                   Handle, BufferMode(LineBuffering) )
+import qualified System.Console.Terminal.Size as TS ( size, width )
+import System.IO ( hFlush, stdout )
 import System.IO.Unsafe ( unsafePerformIO )
 
-import Darcs.Util.Global ( withDebugMode, debugMessage, putTiming )
+import Darcs.Util.Global ( debugMessage )
 
 
 data ProgressData = ProgressData
@@ -80,34 +81,24 @@ printProgress :: String
               -> ProgressData
               -> IO ()
 printProgress k (ProgressData {sofar=s, total=Just t, latest=Just l}) =
-    myput output output
-  where
-    output = k ++ " " ++ show s ++ " done, " ++ show (t - s) ++ " queued. " ++ l
+    putCr (k ++ " ... " ++ show s ++ " done, " ++ show (t - s) ++ " queued. " ++ l)
 printProgress k (ProgressData {latest=Just l}) =
-    myput (k ++ " " ++ l) k
+    putCr (k ++ " ... " ++ l)
 printProgress k (ProgressData {sofar=s, total=Just t}) | t >= s =
-    myput (k ++ " " ++ show s ++ " done, " ++ show (t - s) ++ " queued")
-          (k ++ " " ++ show s)
+    putCr (k ++ " ... " ++ show s ++ " done, " ++ show (t - s) ++ " queued")
 printProgress k (ProgressData {sofar=s}) =
-    myput (k ++ " " ++ show s) k
+    putCr (k ++ " ... " ++ show s)
 
+putCr :: String -> IO ()
+putCr = unsafePerformIO mkPutCr
+{-# NOINLINE putCr #-}
 
-myput :: String -> String -> IO ()
-myput l s = withDebugMode $ \debugMode ->
-    if debugMode
-      then putTiming >> hPutStrLn stderr l
-      else
-        if '\n' `elem` l
-          then myput (takeWhile (/= '\n') l) s
-          else putTiming >> if length l < 80
-                              then simpleput l
-                              else simpleput (take 80 s)
-
-
-simpleput :: String -> IO ()
-simpleput = unsafePerformIO $ mkhPutCr stderr
-{-# NOINLINE simpleput #-}
+withProgress :: String -> (String -> IO a) -> IO a
+withProgress k = bracket (beginTedious k >> return k) endTedious
 
+withSizedProgress :: String -> Int -> (String -> IO a) -> IO a
+withSizedProgress k n =
+  bracket (beginTedious k >> tediousSize k n >> return k) endTedious
 
 -- | @beginTedious k@ starts a tedious process and registers it in
 -- '_progressData' with the key @k@. A tedious process is one for which we want
@@ -132,7 +123,7 @@ endTedious :: String -> IO ()
 endTedious k = whenProgressMode $ do
     p <- getProgressData k
     modifyIORef _progressData (second $ delete k)
-    when (isJust p) $ debugMessage $ "Done " ++ map toLower k
+    when (isJust p) $ putCr $ k ++ " ... done"
 
 
 tediousSize :: String
@@ -184,8 +175,6 @@ progressIO "" = return ()
 progressIO k = do
     updateProgressData k $ \p ->
         p { sofar = sofar p + 1, latest = Nothing }
-    putDebug k ""
-
 
 progressKeepLatest :: String
                    -> a
@@ -197,8 +186,6 @@ progressKeepLatestIO :: String -> IO ()
 progressKeepLatestIO "" = return ()
 progressKeepLatestIO k = do
     updateProgressData k (\p -> p {sofar = sofar p + 1})
-    putDebug k ""
-
 
 finishedOne :: String -> String -> a -> a
 finishedOne k l a = unsafePerformIO $ finishedOneIO k l >> return a
@@ -209,21 +196,10 @@ finishedOneIO "" _ = return ()
 finishedOneIO k l = do
     updateProgressData k (\p -> p { sofar = sofar p + 1,
                                     latest = Just l })
-    putDebug k l
-
-
-putDebug :: String
-         -> String
-         -> IO ()
-putDebug _ _ = return ()
---putDebug k "" = when (False && debugMode) $ hPutStrLn stderr $ "P: "++k
---putDebug k l = when (False && debugMode) $ hPutStrLn stderr $ "P: "++k++" : "++l
 
 
 _progressMode :: IORef Bool
-_progressMode = unsafePerformIO $ do
-    hSetBuffering stderr LineBuffering
-    newIORef True
+_progressMode = unsafePerformIO $ newIORef True
 {-# NOINLINE _progressMode #-}
 
 _progressData :: IORef (String, Map String ProgressData)
@@ -232,21 +208,18 @@ _progressData = unsafePerformIO $ do
     newIORef ("", empty)
 {-# NOINLINE _progressData #-}
 
-mkhPutCr :: Handle
-         -> IO (String -> IO ())
-mkhPutCr fe = do
-    isTerm <- hIsTerminalDevice fe
-    stdoutIsTerm <- hIsTerminalDevice stdout
-    return $
-        if isTerm
-          then \s -> do
-              hPutStr fe $ '\r':s ++ "\r"
-              hFlush fe
-              let spaces = '\r':replicate (length s) ' ' ++ "\r"
-              hPutStr fe spaces
-              when stdoutIsTerm $ putStr spaces
-          else \s -> unless (null s) $ do hPutStrLn fe s
-                                          hFlush fe
+mkPutCr :: IO (String -> IO ())
+mkPutCr =
+  TS.size >>= \case
+    Nothing ->
+      -- stdout is not a terminal
+      return $ \_ -> return ()
+    Just window -> do
+      let limitToWidth = take (TS.width window - 1)
+      return $ \s -> do
+        putStr $ '\r':limitToWidth s ++ "\r"
+        hFlush stdout
+        putStr $ '\r':limitToWidth ((replicate (length s)) ' ') ++ "\r"
 
 setProgressMode :: Bool -> IO ()
 setProgressMode = writeIORef _progressMode
