@@ -2,13 +2,17 @@
 --
 --  BSD3
 
--- | A monadic interface to Tree mutation. The main idea is to
--- simulate IO-ish manipulation of real filesystem (that's the state part of
--- the monad), and to keep memory usage down by reasonably often dumping the
--- intermediate data to disk and forgetting it. The monad interface itself is
--- generic, and a number of actual implementations can be used. This module
--- provides just 'virtualTreeIO' that never writes any changes, but may trigger
--- filesystem reads as appropriate.
+{- | A monad transformer for 'Tree' mutation. The main idea is to simulate
+IO-ish manipulation of real filesystem (that's the state part of the monad),
+and to keep memory usage down by reasonably often dumping the intermediate
+data to disk and forgetting it.
+
+The implementation is configured by passing a procedure of type 'DumpItem'
+to 'runTreeMonad'.
+
+This module provides the pre-configured 'virtualTreeIO' that never
+writes any changes, but may trigger filesystem reads as appropriate. -}
+
 module Darcs.Util.Tree.Monad
     ( -- * 'TreeMonad'
       TreeMonad
@@ -35,9 +39,7 @@ module Darcs.Util.Tree.Monad
 
 import Darcs.Prelude hiding ( readFile, writeFile )
 
-import Control.Exception ( throw )
-
-import Darcs.Util.Path
+import Darcs.Util.Path ( AnchoredPath, anchoredRoot, displayPath, movedirfilename )
 import Darcs.Util.Tree
 
 import Data.List( sortBy )
@@ -45,9 +47,14 @@ import Data.Int( Int64 )
 import Data.Maybe( isNothing, isJust )
 
 import qualified Data.ByteString.Lazy as BL
-import Control.Monad.RWS.Strict
+import Control.Monad ( forM_, when, unless )
+import Control.Monad.Catch ( MonadThrow(..) )
+import Control.Monad.RWS.Strict (RWST, runRWST, ask, gets, lift, modify)
 import qualified Data.Map as M
 
+import System.IO.Error ( ioeSetErrorString, mkIOError )
+import GHC.IO.Exception ( IOErrorType(..) )
+
 -- | Keep track of the size and age of changes to the tree.
 type Changed = M.Map AnchoredPath (Int64, Int64) -- size, age
 
@@ -60,23 +67,19 @@ data TreeState m = TreeState
   , maxage :: !Int64
   }
 
-data TreeEnv m = TreeEnv
-  { updateHash :: TreeItem m -> m Hash
-  , update :: AnchoredPath -> TreeItem m -> TreeMonad m (TreeItem m)
-  }
+-- | A procedure for dumping a single 'TreeItem' to disk. If the implementation
+-- uses item 'Hash'es, it is also responsible to ensure that its 'Hash' is
+-- up-to-date. It is not allowed to make any changes to the actual content of
+-- the 'TreeItem'.
+type DumpItem m = AnchoredPath -> TreeItem m -> m (TreeItem m)
 
 -- | A monad transformer that adds state of type 'TreeState' and an environment
--- of type 'AnchoredPath' (for the current directory).
-type TreeMonad m = RWST (TreeEnv m) () (TreeState m) m
+-- of type 'DumpItem'.
+type TreeMonad m = RWST (DumpItem m) () (TreeState m) m
 
 -- | 'TreeMonad' specialized to 'IO'
 type TreeIO = TreeMonad IO
 
-initialEnv :: (TreeItem m -> m Hash)
-           -> (AnchoredPath -> TreeItem m -> TreeMonad m (TreeItem m))
-           -> TreeEnv m
-initialEnv uh u = TreeEnv {updateHash = uh, update = u}
-
 initialState :: Tree m -> TreeState m
 initialState t =
   TreeState {tree = t, changed = M.empty, changesize = 0, maxage = 0}
@@ -85,24 +88,19 @@ flush :: Monad m => TreeMonad m ()
 flush = do changed' <- map fst . M.toList <$> gets changed
            dirs' <- gets tree >>= \t -> return [ path | (path, SubTree _) <- list t ]
            modify $ \st -> st { changed = M.empty, changesize = 0 }
-           forM_ (changed' ++ dirs' ++ [AnchoredPath []]) flushItem
+           forM_ (changed' ++ dirs' ++ [anchoredRoot]) flushItem
 
-runTreeMonad' :: Monad m => TreeMonad m a -> TreeEnv m -> TreeState m -> m (a, Tree m)
+runTreeMonad' :: Monad m => TreeMonad m a -> DumpItem m -> TreeState m -> m (a, Tree m)
 runTreeMonad' action initEnv initState = do
   (out, final, _) <- runRWST action initEnv initState
   return (out, tree final)
 
-runTreeMonad :: Monad m
-             => TreeMonad m a
-             -> Tree m
-             -> (TreeItem m -> m Hash)
-             -> (AnchoredPath -> TreeItem m -> TreeMonad m (TreeItem m))
-             -> m (a, Tree m)
-runTreeMonad action t uh u = do
+runTreeMonad :: Monad m => TreeMonad m a -> Tree m -> DumpItem m -> m (a, Tree m)
+runTreeMonad action t dump = do
   let action' = do x <- action
                    flush
                    return x
-  runTreeMonad' action' (initialEnv uh u) (initialState t)
+  runTreeMonad' action' dump (initialState t)
 
 -- | Run a 'TreeMonad' action without storing any changes. This is useful for
 -- running monadic tree mutations for obtaining the resulting 'Tree' (as opposed
@@ -110,8 +108,7 @@ runTreeMonad action t uh u = do
 -- read and write -- reads are passed through to the actual filesystem, but the
 -- writes are held in memory in the form of a modified 'Tree'.
 virtualTreeMonad :: Monad m => TreeMonad m a -> Tree m -> m (a, Tree m)
-virtualTreeMonad action t =
-  runTreeMonad action t (\_ -> return NoHash) (\_ x -> return x)
+virtualTreeMonad action t = runTreeMonad action t (const return)
 
 -- | 'virtualTreeMonad' specialized to 'IO'
 virtualTreeIO :: TreeIO a -> Tree IO -> IO (a, Tree IO)
@@ -141,12 +138,7 @@ renameChanged :: Monad m
               => AnchoredPath -> AnchoredPath -> TreeMonad m ()
 renameChanged from to = modify $ \st -> st {changed = rename' $ changed st}
   where
-    rename' = M.fromList . map renameone . M.toList
-    renameone (x, d)
-      | from `isPrefix` x = (to `catPaths` relative from x, d)
-      | otherwise = (x, d)
-    relative (AnchoredPath from') (AnchoredPath x) =
-      AnchoredPath $ drop (length from') x
+    rename' = M.mapKeys (movedirfilename from to)
 
 -- | Replace an item with a new version without modifying the content of the
 -- tree. This does not do any change tracking. Ought to be only used from a
@@ -154,26 +146,19 @@ renameChanged from to = modify $ \st -> st {changed = rename' $ changed st}
 -- is that an existing in-memory Blob is replaced with a one referring to an
 -- on-disk file.
 replaceItem :: Monad m
-            => AnchoredPath -> Maybe (TreeItem m) -> TreeMonad m ()
+            => AnchoredPath -> TreeItem m -> TreeMonad m ()
 replaceItem path item = do
-  modify $ \st -> st { tree = modifyTree (tree st) path item }
-
-flushItem :: forall m. Monad m => AnchoredPath -> TreeMonad m ()
-flushItem path =
-  do current <- gets tree
-     case find current path of
-       Nothing -> return () -- vanished, do nothing
-       Just x -> do y <- fixHash x
-                    new <- asks update >>= ($ y) . ($ path)
-                    replaceItem path (Just new)
-    where fixHash :: TreeItem m -> TreeMonad m (TreeItem m)
-          fixHash f@(File (Blob con NoHash)) = do
-            hash <- asks updateHash >>= \x -> lift $ x f
-            return $ File $ Blob con hash
-          fixHash (SubTree s) | treeHash s == NoHash =
-            asks updateHash >>= \f -> SubTree <$> lift (addMissingHashes f s)
-          fixHash x = return x
+  modify $ \st -> st { tree = modifyTree (tree st) path (Just item) }
 
+-- | Flush a single item to disk. This is the only procedure that (directly)
+-- uses the Reader part of the environment (the procedure of type @'DumpItem' m@).
+flushItem :: forall m . Monad m => AnchoredPath -> TreeMonad m ()
+flushItem path = do
+  current <- gets tree
+  dumpItem <- ask
+  case find current path of
+    Nothing -> return () -- vanished, do nothing
+    Just item -> lift (dumpItem path item) >>= replaceItem path
 
 -- | If buffers are becoming large, sync, otherwise do nothing.
 flushSome :: Monad m => TreeMonad m ()
@@ -199,47 +184,67 @@ expandTo p =
        t' <- lift $ expandPath t p
        modify $ \st -> st { tree = t' }
 
+findItem :: Monad m => AnchoredPath -> TreeMonad m (Maybe (TreeItem m))
+findItem path = do
+  expandTo path
+  tr <- gets tree
+  return $ find tr path
+
 -- | Check for existence of a file.
 fileExists :: Monad m => AnchoredPath -> TreeMonad m Bool
-fileExists p =
-    do expandTo p
-       (isJust . (`findFile` p)) <$> gets tree
+fileExists p = do
+  item <- findItem p
+  case item of
+    Just (File{}) -> return True
+    _ -> return False
 
 -- | Check for existence of a directory.
 directoryExists :: Monad m => AnchoredPath -> TreeMonad m Bool
-directoryExists p =
-    do expandTo p
-       (isJust . (`findTree` p)) <$> gets tree
+directoryExists p = do
+  item <- findItem p
+  case item of
+    Just (SubTree{}) -> return True
+    _ -> return False
 
 -- | Check for existence of a node (file or directory, doesn't matter).
-exists :: Monad m => AnchoredPath -> TreeMonad m Bool
-exists p =
-    do expandTo p
-       isJust . (`find` p) <$> gets tree
+exists :: MonadThrow m => AnchoredPath -> TreeMonad m Bool
+exists p = isJust <$> findItem p
 
 -- | Grab content of a file in the current Tree at the given path.
-readFile :: Monad m => AnchoredPath -> TreeMonad m BL.ByteString
-readFile p =
-    do expandTo p
-       t <- gets tree
-       let f = findFile t p
-       case f of
-         Nothing -> throw $ userError $ "No such file " ++ displayPath p
-         Just x -> lift (readBlob x)
+readFile :: MonadThrow m => AnchoredPath -> TreeMonad m BL.ByteString
+readFile p = do
+  f <- findItem p
+  case f of
+    Just (File x) -> lift (readBlob x)
+    Just _ ->
+      throwM $
+        flip ioeSetErrorString "is a directory" $
+        mkIOError InappropriateType "readFile" Nothing (Just (displayPath p))
+    Nothing ->
+      throwM $
+        mkIOError NoSuchThing "readFile" Nothing (Just (displayPath p))
 
 -- | Change content of a file at a given path. The change will be
 -- eventually flushed to disk, but might be buffered for some time.
-writeFile :: Monad m => AnchoredPath -> BL.ByteString -> TreeMonad m ()
-writeFile p con =
-    do expandTo p
-       modifyItem p (Just blob)
-       flushSome
-    where blob = File $ Blob (return con) hash
-          hash = NoHash -- we would like to say "sha256 con" here, but due
-                        -- to strictness of Hash in Blob, this would often
-                        -- lead to unnecessary computation which would then
-                        -- be discarded anyway; we rely on the sync
-                        -- implementation to fix up any NoHash occurrences
+writeFile :: MonadThrow m => AnchoredPath -> BL.ByteString -> TreeMonad m ()
+writeFile p con = do
+  item <- findItem p
+  case item of
+    Just (SubTree _) ->
+      throwM $
+      flip ioeSetErrorString "is a directory" $
+      mkIOError InappropriateType "writeFile" Nothing (Just (displayPath p))
+    _ ->
+      -- note that writing to a non-existing file is allowed,
+      -- in fact there is no primitive for creating a file
+      modifyItem p (Just blob)
+  flushSome
+  where
+    blob = File $ Blob (return con) Nothing
+    -- we would like to say "sha256 con" here, but due to strictness of Hash in
+    -- Blob, this would often lead to unnecessary computation which would then
+    -- be discarded anyway; we rely on the sync implementation to fix up any
+    -- Nothing occurrences
 
 -- | Create a directory.
 createDirectory :: Monad m => AnchoredPath -> TreeMonad m ()
@@ -254,30 +259,32 @@ unlink p =
        modifyItem p Nothing
 
 -- | Rename the item at a path.
-rename :: Monad m => AnchoredPath -> AnchoredPath -> TreeMonad m ()
-rename from to =
-    do expandTo from
-       expandTo to
-       tr <- gets tree
-       let item = find tr from
-           found_to = find tr to
-       unless (isNothing found_to) $
-              throw $ userError $ "Error renaming: destination " ++ displayPath to ++ " exists."
-       if isJust item then do
-              modifyItem from Nothing
-              modifyItem to item
-              renameChanged from to
-       else
-        throw $ userError $ "Error renaming: source " ++ displayPath from ++ " does not exist."
+rename :: MonadThrow m => AnchoredPath -> AnchoredPath -> TreeMonad m ()
+rename from to = do
+  item <- findItem from
+  found_to <- findItem to
+  unless (isJust item) $
+    throwM $
+      mkIOError NoSuchThing "rename" Nothing (Just (displayPath from))
+  unless (isNothing found_to) $
+    throwM $
+      mkIOError AlreadyExists "rename" Nothing (Just (displayPath to))
+  modifyItem from Nothing
+  modifyItem to item
+  renameChanged from to
 
 -- | Copy an item from some path to another path.
-copy :: Monad m => AnchoredPath -> AnchoredPath -> TreeMonad m ()
-copy from to =
-    do expandTo from
-       expandTo to
-       tr <- gets tree
-       let item = find tr from
-       unless (isNothing item) $ modifyItem to item
+-- Doing this with a SubTree is weird... it means copy recursively,
+-- but with lazy copy-on-write semantics. What happens when we flush that?
+-- Seems to work, though, as it is used in Darcs.UI.Commands.Convert.Import
+copy :: MonadThrow m => AnchoredPath -> AnchoredPath -> TreeMonad m ()
+copy from to = do
+  expandTo to
+  item <- findItem from
+  when (isNothing item) $
+    throwM $
+    mkIOError NoSuchThing "copy" Nothing (Just (displayPath from))
+  modifyItem to item
 
 findM' :: forall m a . Monad m
        => (Tree m -> AnchoredPath -> a) -> Tree m -> AnchoredPath -> m a
