@@ -1,15 +1,18 @@
-{-# LANGUAGE ForeignFunctionInterface #-}
-
+{-# LANGUAGE CPP, ForeignFunctionInterface #-}
 module Darcs.Util.CtrlC ( withCtrlCHandler ) where
 
+import Darcs.Prelude
+
 import Data.Word ( Word32 )
 import Foreign.Ptr ( FunPtr )
 import Control.Exception ( bracket_ )
 
+#include <windows_cconv.h>
+
 type Handler = Word32 -> IO Int
 
-foreign import ccall "wrapper" wrap :: Handler -> IO (FunPtr Handler)
-foreign import stdcall "SetConsoleCtrlHandler" setConsoleCtrlHandler :: FunPtr Handler -> Int -> IO ()
+foreign import WINDOWS_CCONV "wrapper" wrap :: Handler -> IO (FunPtr Handler)
+foreign import WINDOWS_CCONV "SetConsoleCtrlHandler" setConsoleCtrlHandler :: FunPtr Handler -> Int -> IO ()
 
 
 withCtrlCHandler :: IO () -> IO a -> IO a
