@@ -1,15 +1,15 @@
 module System.Posix.Files
     ( isNamedPipe, isDirectory, isRegularFile, isSymbolicLink
     , getFdStatus, getFileStatus, getSymbolicLinkStatus
-    , modificationTime, setFileMode, fileSize, fileMode, fileOwner
+    , modificationTimeHiRes, setFileMode, fileSize, fileMode, fileOwner
     , stdFileMode, FileStatus, fileID
-    , linkCount, createLink
+    , linkCount, createLink, ownerModes
     ) where
 
 import System.PosixCompat.Files
     ( isNamedPipe, isDirectory, isRegularFile, isSymbolicLink
     , getFdStatus, getFileStatus, getSymbolicLinkStatus
-    , modificationTime, setFileMode, fileSize, fileMode, fileOwner
+    , modificationTimeHiRes, setFileMode, fileSize, fileMode, fileOwner
     , stdFileMode, FileStatus, fileID
-    , linkCount, createLink
+    , linkCount, createLink, ownerModes
     )
