@@ -24,9 +24,8 @@ import Control.Exception
 
 stamp i j = system ("echo " ++ show i ++ " > ./i") >>
             system ("echo " ++ show j ++ " > ./j") >>
-            -- system ("sleep 1") >>
             hFlush stdout >>
-            system ("darcs record --ignore-times -am '" ++ show i ++ "'")
+            system ("darcs record -am '" ++ show i ++ "'")
 
 generate :: [Int] -> IO ()
 generate = mapM_ (uncurry stamp) . zip [1..]
