@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ve
+. ./lib
 binary=example_binary.png
 function checkbinary(){
   cmp $binary ../temp1/$binary
