@@ -10,9 +10,9 @@
 check_conflict() {
 cat out
 if test "$format" = darcs-2; then
-    not grep 'conflict' out
+    not grep 'conflicts' out
 else
-    grep 'conflict' out
+    grep 'conflicts' out
 fi
 }
 
@@ -24,7 +24,7 @@ darcs init
 
 touch a.txt
 darcs add a.txt
-darcs record -A base -am 'adding a.txt'
+darcs record -am 'adding a.txt'
 cd ..
 
 darcs get tmp_dopple tmp_ganger
@@ -39,7 +39,7 @@ done
 
 # Now that the conflict has been set up, try pull one patch from the other.
 cd tmp_ganger
-darcs pull -a ../tmp_dopple > out
+darcs pull -a ../tmp_dopple 2> out
 check_conflict
 cd ..
 
@@ -48,6 +48,8 @@ rm -rf temp0 temp1 temp2 tmp_dopple tmp_ganger
 mkdir temp0
 cd temp0
 darcs init
+touch a.txt
+darcs record -la a.txt -m init
 cd ..
 
 # Create a conflict
@@ -55,14 +57,12 @@ darcs get temp0 temp1
 cd temp1
 darcs show repo
 echo temp1 > a.txt
-darcs add a.txt
-darcs record  -A base -am 'adding temp1 a.txt'
+darcs record -am temp1
 cd ..
 darcs get temp0 temp2
 cd temp2
 echo temp2 > a.txt
-darcs add a.txt
-darcs record  -A base -am 'adding temp2 a.txt'
+darcs record -am temp2
 cd ..
 
 # Resolve the conflict the same way on both sides
@@ -78,5 +78,5 @@ done
 
 # Now that the conflict has been set up, try pull one patch from the other.
 cd tmp_ganger
-darcs pull -a ../tmp_dopple > out
+darcs pull -a ../tmp_dopple 2> out
 check_conflict
