@@ -42,7 +42,7 @@ runtest() {
     darcs init --darcs-1
     darcs apply --allow-conflicts $TESTDATA/convert/darcs1/$name.dpatch
     cd ..
-    echo 'I understand the consequences of my action' | darcs convert darcs-2 repo repo2 --$opt
+    echo 'yes' | darcs convert darcs-2 repo repo2 --$opt
     mkdir empty-darcs2
     cd empty-darcs2
     darcs init --darcs-2
@@ -50,8 +50,7 @@ runtest() {
     cd repo2
     darcs send --no-minimize -a -o ../$name-darcs2.dpatch ../empty-darcs2
     cd ..
-    diff -I'1 patch for repository ' -I'patches for repository ' -I'Oct 1' \
-      -u $TESTDATA/convert/darcs2/$name.dpatch $name-darcs2.dpatch
+    compare_bundles $TESTDATA/convert/darcs2/$name.dpatch $name-darcs2.dpatch
 
     cd ../..
 }
