@@ -43,10 +43,6 @@ import Data.Char (chr)
 main = getContents >>= print . not . any (> Data.Char.chr 127)
 EOF
 
-ghc --make is_ascii.hs -o is_ascii
-./is_ascii < ../temp1/mail_as_file | grep '^True$'
+runghc is_ascii < ../temp1/mail_as_file | grep True
 
 cd ..
-rm -rf temp1
-rm -rf temp2
-rm -rf temp3
