@@ -23,10 +23,8 @@ cd ..
 # local vs remote filepaths
 # ----------------------------------------------------------------------
 
-# trick: OS-detection (if needed)
-if echo $OS | grep -i windows; then
-  echo This test does not work on Windows
-else
+# Windows does not allow ':' in file names
+if ! os_is_windows; then
   darcs get temp1 temp2
   cd temp2
   mkdir -p dir
@@ -82,7 +80,7 @@ darcs replace --repodir=temp1 foo bar baz
 darcs setpref --repodir=temp1 test echo | grep -i "Changing value of test"
 
 # test --linear accepts --repodir.
-darcs test --linear --repodir=temp1 | grep -i "Success!"
+darcs test --linear --repodir=temp1 | grep -i "Test does not fail on head"
 
 # ----------------------------------------------------------------------
 # converting between absolute and relative paths
