@@ -55,6 +55,8 @@ abort_windows
 # only run if git present
 git --version | grep -i "git version"   || exit 200
 
+rm -rf gitsource gitmirror darcssource darcsmirror
+
 git init gitsource
 cd gitsource
 
@@ -64,7 +66,8 @@ git commit -m "$commitMsg"
 
 cd ..
 
-(cd gitsource && git fast-export --all) | darcs convert import darcsmirror
+(cd gitsource && git fast-export --all) > gitexport
+darcs convert import darcsmirror < gitexport
 
 # working copies should be the same
 wcDiff gitsource darcsmirror
@@ -81,7 +84,8 @@ darcs add --reserved-ok $quoted
 darcs rec -lam "$commitMsg"
 
 git init ../gitmirror
-darcs convert export | (cd ../gitmirror && git fast-import && git checkout) 
+darcs convert export > ../darcsexport
+(cd ../gitmirror && git fast-import && git checkout) < ../darcsexport
 cd ..
 
 # working copies should be the same
