@@ -30,7 +30,7 @@ if [ $? == 2 ] ; then echo gzcrcs not supported by this darcs ; exit 0 ; fi
 
 . lib                  # Load some portability helpers.
 rm -rf maybench-crc
-gunzip -c $TESTDATA/maybench-crc.tgz | tar xf -
+unpack_testdata maybench-crc
 cd maybench-crc
 not darcs gzcrcs --check
 darcs gzcrcs --repair
