@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -vex
+. ./lib
 
 # Print some stuff out for debugging if something goes wrong:
 echo $HOME
