@@ -4,14 +4,8 @@
 # A test for a missed resolution, inspired by bug #10 in RT
 
 rm -rf temp1 temp2
-mkdir temp1
-cd temp1
-darcs init
-cd ..
-mkdir temp2
-cd temp2
-darcs init
-cd ..
+darcs init temp1
+darcs init temp2
 
 # set up temp1
 cd temp1
@@ -66,5 +60,3 @@ darcs record -m resolution --all
 # now mark-conflicts shouldn't find any unmarked conflicts
 darcs mark-conflicts | grep "No conflicts to mark"
 cd ..
-
-rm -rf temp1 temp2
