@@ -11,17 +11,16 @@ echo -e 'foo\nbar\nbaz' > f
 darcs rec -Ax -alm p1
 echo -e 'foo\nbar\nwibble' > f
 darcs rec -Ax -alm p2
-sleep 1 # ensure the timestamps would differ after this change alone
 echo -e 'baz\nbar\nwibble' > f
 
-# check that wh (without --ignore-times) sees the change now
+# check that wh (without --ignore-times) sees the change
 darcs wh > whatsnew
 grep 'foo' whatsnew
 
-# the problematic unpull
-darcs unpull --last 1 -a --ignore-times
+# the (once) problematic unpull
+darcs unpull --last 1 -a
 
-# whatsnew will now think there are no changes without --ignore-times
+# whatsnew no longer thinks there are no changes without --ignore-times
 darcs wh > whatsnew
 grep 'foo' whatsnew
 
