@@ -20,18 +20,18 @@ darcs clone V S
 
 not test -e U/_darcs/prefs/defaultrepo
 not test -e V/_darcs/prefs/defaultrepo
-grep '/U$' R1/_darcs/prefs/defaultrepo
-grep '/U$' R2/_darcs/prefs/defaultrepo
-grep '/U$' R3/_darcs/prefs/defaultrepo
-grep '/V$' S/_darcs/prefs/defaultrepo
+test U = $(cat R1/_darcs/prefs/defaultrepo | xargs basename)
+test U = $(cat R2/_darcs/prefs/defaultrepo | xargs basename)
+test U = $(cat R3/_darcs/prefs/defaultrepo | xargs basename)
+test V = $(cat S/_darcs/prefs/defaultrepo | xargs basename)
 
 cd R3
 for cmd in pull push send; do
   # set-default works by setting the defaultrepo of the remote repo
   darcs $cmd ../S --set-default
-  grep '/V$' _darcs/prefs/defaultrepo
+  test V = $(cat _darcs/prefs/defaultrepo | xargs basename)
   # but not if remote repo has no defaultrepo
   darcs $cmd ../U --set-default
-  grep '/U$' _darcs/prefs/defaultrepo
+  test U = $(cat _darcs/prefs/defaultrepo | xargs basename)
 done
 cd ..
