@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
+. ./lib
 rm -rf temp1 temp2
 
 # creating the fork point
