@@ -1,9 +1,6 @@
 #!/usr/bin/env bash
-. ./lib
 
-DARCS_EDITOR=echo
-export DARCS_EDITOR
-export SENDMAIL=`which true`
+. ./lib
 
 rm -rf temp1 temp2
 mkdir temp1 temp2
@@ -19,10 +16,13 @@ darcs add foo bar
 darcs record -a -m add_foo_bar -A x
 
 # Test that --cc is also printed as recipient in case of success
-darcs send --mail --author=me -a --to=random@random --cc=foo@example.com ../temp2 2>&1|grep -i foo@example.com
+export SENDMAIL=true
+darcs send --mail --author=me -a --to=random@random --cc=foo@example.com ../temp2 >out
+grep 'foo@example.com' out
 
 # Test that --cc is also printed as recipient in case of error
-darcs send --mail --author=me -a --to=random@random --cc=foo@example.com ../temp2 2>&1|grep -i foo@example.com
+export SENDMAIL=false
+not darcs send --mail --author=me -a --to=random@random --cc=foo@example.com ../temp2 2>err
+grep 'foo@example.com' err
 
 cd ..
-rm -rf temp1 temp2
