@@ -34,7 +34,7 @@ darcs get R0 S
 cd S
 # default to no-set-default
 darcs push ../R1 > log
-grep '/R0$' _darcs/prefs/defaultrepo
+test R0 = $(cat _darcs/prefs/defaultrepo | xargs basename)
 # notification when using no-set-default
 grep -- "--set-default" log
 # ...but not when --inherit-default is active
@@ -42,19 +42,10 @@ darcs push ../R1 --inherit-default > log
 not grep -- "--set-default" log
 # set-default works
 darcs push ../R1 --set-default > log
-grep '/R1$' _darcs/prefs/defaultrepo
+test R1 = $(cat _darcs/prefs/defaultrepo | xargs basename)
 # no notification when already using --set-default
 not grep -- "--set-default" log
 # no notification when already pushing to the default repo
 darcs push > log
 not grep -- "--set-default" log
-# no notification when it's just the --remote-repo
-darcs push --remote-repo ../R1 > log
-not grep -- "--set-default" log
-# but... notification still works in presence of remote-repo
-darcs push --remote-repo ../R1 ../R2 > log
-grep -- "--set-default" log
-# ...but not when --inherit-default is active
-darcs push --remote-repo ../R1 ../R2 --inherit-default > log
-not grep -- "--set-default" log
 cd ..
