@@ -50,8 +50,8 @@ touch funny/user@invalid:path
 touch funny/droundy@invalid:    
 touch funny/invalid:path
 
-# Try to add those files. None should be added, darcs should not fail
-darcs add -qr funny
+# Try to add those files. None should be added, darcs should fail
+not darcs add -qr funny
 darcs wh -l > log 2>&1
 
 # Check that darcs didn't drop dead as 2.4.4 does
