@@ -13,26 +13,23 @@ cd a
 darcs init --darcs-2
 echo A > file
 darcs add file
-darcs rec -a -m A --ignore-times
+darcs rec -a -m A
 cd ..
 
 darcs get a b
 
 cd a
 echo BB > file
-darcs rec -a -m B --ignore-times
+darcs rec -a -m B
 cd ..
 
-sleep 1
-
 cd b
 touch ts
 echo R > file
 touch file --reference=ts
 darcs rec -a -m R
-# sleep 1
 echo S > file
 touch file --reference=ts
-# darcs rec -a -m S --ignore-times
+# darcs rec -a -m S
 echo y| darcs pull ../a -ap B
 cd ..
