@@ -8,10 +8,7 @@ darcs  init
 touch titi
 darcs add titi
 darcs record -am titi
-cat > _darcs/format <<EOF
-hashed|gobbledygook
-darcs-2
-EOF
+sed -i 's/hashed/hashed\|gobbledygook/' _darcs/format
 cat _darcs/format
 cd ..
 
