@@ -26,9 +26,6 @@
 
 . ./lib
 
-DARCS_EDITOR=echo
-export DARCS_EDITOR
-
 mkdir temp1 temp2
 
 cd temp2
