@@ -25,9 +25,12 @@
 
 . lib
 
-darcs init R
+rm -rf R
+mkdir R
 cd R
+
 # simple full complete replace (record)
+darcs init
 echo "foo" > file
 darcs record -al -m "add file"
 echo "bar_longer" > file          # replace by token of different length
@@ -87,6 +90,16 @@ diff -u log log.expected
 rm -rf *
 
 # partial replace (only some of the words/chunks replaced) (amend-record)
+# note that coalescing simplifies
+#     hunk ./file 1
+#     +foo foo
+#     replace ./file [A-Za-z_0-9] foo bar
+#     hunk ./file 1
+#     -bar bar
+#     +bar foo
+# to
+#     hunk ./file 1
+#     +bar foo
 darcs init
 echo "foo foo" > file
 darcs record -al -m "add file"
@@ -97,10 +110,6 @@ cat > log.expected <<EOF
   * add file
     addfile ./file
     hunk ./file 1
-    +foo foo
-    replace ./file [A-Za-z_0-9] foo bar
-    hunk ./file 1
-    -bar bar
     +bar foo
 EOF
 diff -u log log.expected
@@ -153,19 +162,19 @@ foo
 foo
 bar
 EOF
-darcs record -al -m "add file" --ignore-times
+darcs record -al -m "add file"
 cat > file <<EOF
 foo
 bar
 bar
 EOF
-darcs record -a -m "change file" --ignore-times
+darcs record -a -m "change file"
 cat > file <<EOF
 bar
 bar
 bar
 EOF
-echo yyyy | darcs amend-record --look-for-replaces --ignore-times
+echo yyyy | darcs amend-record --look-for-replaces
 darcs changes --last 1 -v 2>&1 | tail -n +4 | grep -v "^    {\|    }$" > log
 cat > log.expected <<EOF
   * change file
@@ -322,9 +331,7 @@ rm -rf *
 
 # 2
 
-cd ..
-darcs init S
-cd S
+darcs init
 echo 'Example content' > f
 echo 'This is golden content , super interesting content' >> f
 
@@ -336,12 +343,11 @@ sed -i "s/content/matter/g" f
 sed -i "s/content/topic/g" g
 darcs whatsnew --look-for-replace | grep replace
 # replace same token differently in different files is OK
+rm -rf *
 
 # 3
 
-cd ..
-darcs init T
-cd T
+darcs init
 echo 'Example content' > f
 echo 'This is golden content , super interesting content' >> f
 
@@ -363,3 +369,5 @@ darcs whatsnew --look-for-replace > out
 grep "^replace ./e" out  # 1 replace in e
 not grep "^replace ./f" out  # 0 replace in f
 grep "^replace ./g" out  # 1 replace in g
+
+cd ..
