@@ -34,6 +34,10 @@ cd ..
 
 darcs get R S --no-working-dir
 
+# rename the source repo to trigger the bug, since we are
+# now using caches for pristine
+mv R X
+
 cd S
 not darcs whatsnew | grep "No changes"
 
