@@ -2,7 +2,7 @@
 
 . lib
 
-# test that 'whatsnew -l --boring' actually lists boring files
+# test that 'whatsnew --boring' actually lists boring files
 
 rm -rf R
 darcs init R
@@ -12,8 +12,8 @@ echo xxx > boring
 darcs setpref boringfile boring
 darcs record -lam'added boring and set as boringile'
 touch xxx
-darcs whatsnew -l --boring | grep xxx
-darcs whatsnew -l --boring | grep -v 'No changes'
+darcs whatsnew --boring | grep xxx
+darcs whatsnew --boring | grep -v 'No changes'
 
 cd ..
 rm -rf R
