@@ -8,6 +8,7 @@
 # this test fails for darcs-1 repos
 skip-formats darcs-1
 
+rm -rf temp1
 mkdir temp1
 cd temp1
 darcs init
@@ -17,26 +18,26 @@ darcs record -am 00
 cd ..
 
 for r in a b c d; do
+  rm -rf temp_$r
   darcs get temp1 temp_$r
-  cd temp_$r;
+  cd temp_$r
   echo $r > f
-  darcs record -am "patch:$r";
+  darcs record -am "patch:$r"
   cd ..
 done
 
 cd temp_d
-darcs pull -a ../temp_a
-darcs pull -a ../temp_b
-darcs pull -a ../temp_c
+darcs pull -a ../temp_a --allow-conflicts
+darcs pull -a ../temp_b --allow-conflicts
+darcs pull -a ../temp_c --allow-conflicts
 cd ..
 cd temp_c
-darcs pull -a ../temp_a
-darcs pull -a ../temp_b
+darcs pull -a ../temp_a --allow-conflicts
+darcs pull -a ../temp_b --allow-conflicts
 echo rc > f
 darcs record -a -m rc
 cd ..
 cd temp_d
 darcs pull -a ../temp_c > log
-not grep -i "no remote" log
-not grep -i get_extra log
+not grep -i "Failed to commute common patches" log
 cd ..
