@@ -3,12 +3,10 @@
 ### darcs get --set-scripts-executable ignores umask
 . ./lib
 
-## Windows doesn't support proper permissions.
-
-if echo $OS | grep -i windows; then
-    echo Windows does not support posix permissions
-    exit 0
-fi
+# We can set and clear permission bits with bash on Windows but that
+# has not the expected effect on programs. So even though this test
+# actually succeeds on Windows, it makes no sense to run it.
+abort_windows
 
 rm -rf temp
 mkdir temp
@@ -26,4 +24,3 @@ echo -rwx------ > desired-mode
 diff -u desired-mode mode
 
 cd ..
-rm -rf temp
