@@ -2,9 +2,6 @@
 
 . ./lib
 
-## I would use the builtin !, but that has the wrong semantics.
-not () { "$@" && exit 1 || :; }
-
 rm -rf temp1
 mkdir temp1
 cd temp1
@@ -21,4 +18,3 @@ darcs changes --last=1 -v > output
 cat output
 not grep first output
 cd ..
-rm -rf temp1
