@@ -1,4 +1,4 @@
-#!/bin/env bash
+#!/usr/bin/env bash
 # A test for issue 538 - that an executable test script will run successfully if
 # it is recorded with --set-scripts-executable.
 
@@ -73,7 +73,7 @@ cd ..
 # test --linear with --set-scripts-executable
 rm -rf temp1
 make_repo_with_test
-if darcs test --linear --set-scripts-executable | grep 'Success!' ; then
+if darcs test --linear --set-scripts-executable | grep 'Test does not fail on head' ; then
     echo "ok 5"
 else
     echo "not ok 5 tracking down with --set-scripts-executable failed (because test failed?)"
