@@ -15,36 +15,24 @@
 
 abort_windows
 
-rm -rf temp1 temp2
+rm -rf tmp_remote tmp_restrictive
 
 # Set up a "remote" repo
-mkdir tmp_remote
-cd tmp_remote
-darcs 'init'
-cd ..
+darcs init tmp_remote
+
+DIR=`/bin/pwd`
+trap "chmod +wx $PWD/tmp_restrictive" EXIT
 
-DIR=`pwd`
 # Set up a directory with restrictive permissions
 mkdir -p tmp_restrictive/liberal
 cd tmp_restrictive/liberal
-chmod 0111 ../../tmp_restrictive
+chmod a-wx ../../tmp_restrictive
+# sanity check that we can not cd ..
+not cd ..
 # sanity check that we can cd out and back
-cd ../..; cd tmp_restrictive/liberal
-# TODO: we avoid this test on Solaris because it seems we can't create
-# anything in tmp_restrictive/liberal
-touch can_touch
-if [ -e can_touch ]; then
-  if hwpwd; then
-    darcs get "$DIR/tmp_remote" 2> log
-    not grep -i 'permission denied' log
-  else
-    echo "Apparently I can't do `basename $0` on this platform"
-  fi
-else
-  echo "Can't do `basename $0` on this platform"
-fi
-cd "$DIR"
-# We have to fix the permissions, just so we can delete it.
-chmod 0755 tmp_restrictive
-
-rm -rf tmp_remote tmp_restrictive
+(cd ../.. && cd tmp_restrictive/liberal) || exit 200
+(touch can_touch && test -e can_touch) || exit 200
+# now run the real test
+darcs get "$DIR/tmp_remote" 2>&1 >log
+not grep -i 'permission denied' log
+cd ../..
