@@ -3,9 +3,6 @@
 
 # for issue706: "Filenames with spaces issue"
 
-DARCS_EDITOR=echo
-export DARCS_EDITOR
-
 rm -rf temp
 mkdir temp
 cd temp
