@@ -25,7 +25,7 @@ darcs setpref boringfile .boring
 
 echo bar > bar
 darcs add bar
-echo yyd | darcs record -mbar
+darcs record -mbar -a bar
 
 cat _darcs/patches/pending
 
@@ -42,7 +42,7 @@ darcs wh -l
 # remove any files added by profiling or hpc...
 rm -f darcs.tix darcs.prof
 
-echo yyd | darcs record --look-for-adds -mfoo
+darcs record --look-for-adds -mfoo -a foo
 
 cat _darcs/patches/pending
 
