@@ -54,9 +54,6 @@ grep_changes () {
 # Please leave it this way :-)
 switch_to_latin9_locale
 
-# This test clobbers the global darcs author
--f $HOME/.darcs/author && exit 200
-
 rm -rf temp1
 mkdir temp1
 cd temp1
@@ -135,7 +132,7 @@ import qualified Data.ByteString as B
 str = B.pack [65,108,108,32,109,121,32,164,115,32,97,114,101,32,103,111,110,101]
 main = getArgs >>= \[x] -> B.writeFile x str
 FAKE
-ghc --make -o editor editor.hs
+ghc $GHC_FLAGS --make -o editor editor.hs
 export DARCS_EDITOR="`pwd`/editor"
 if echo $OS | not grep -i windows; then # issue2591
     printf "y\ny\n" | darcs amend --edit -p 'Patch by '
