@@ -21,20 +21,20 @@ darcs add f2
 darcs record -am 'add f2'
 cd ..
 
-darcs get --lazy temp1 temp2
+darcs get --lazy temp1 temp2 --no-cache
 
-darcs get --lazy temp2 temp3
+darcs get --lazy temp2 temp3 --no-cache
 
 cd temp2
 
 # Run darcs changes so we pull in the inventories (but no the patches)
-darcs changes
+darcs changes --no-cache
 
 # Remove original repository, so we have no references to changes f1 and f2.
 rm -rf ../temp1
 
 # Now we should be unable to read some of the history
-darcs changes -s > out
+darcs changes -s --no-cache > out
 cat out
 grep unavailable out
 
@@ -57,6 +57,6 @@ darcs pull -av
 darcs optimize reorder
 
 # Just a double-check: we shouldn't be able to check in this case.
-not darcs check
+not darcs check --no-cache
 
 cd ..
