@@ -2,10 +2,9 @@
 . ./lib
 
 # RT#544 using context created with 8-bit chars;
+
 rm -rf temp1
-
 mkdir temp1
-
 cd temp1
 darcs init
 touch foo
@@ -14,5 +13,8 @@ darcs log --context >context
 date > foo
 darcs record -a -m 'date foo' | grep 'Finished record'
 darcs send -a -o patch --context context . | grep 'Wrote patch to'
+# again with an absolute path for the context file, see issue1240
+# note we must not mix \\ and / so we use /bin/pwd here
+cwd=$(/bin/pwd)
+darcs send -a -o patch --context "$cwd/context" . | grep 'Wrote patch to'
 cd ..
-rm -rf temp1
