@@ -1,16 +1,5 @@
 #!/usr/bin/env bash
 
-cat > empty_pending <<EOF
-{
-}
-EOF
-check_empty_pending() {
-  echo ================ pending ==================
-  cat _darcs/patches/pending
-  echo ================ pending ==================
-  [ -z _darcs/patches/pending ] || cmp _darcs/patches/pending ../empty_pending
-}
-
 . ./lib
 
 rm -rf temp1 temp2
@@ -19,13 +8,13 @@ cd temp1
 darcs init
 mkdir dir
 darcs record -a -m add_dir -A x --look-for-adds
-check_empty_pending
+not darcs whatsnew
 echo zig > dir/foo
 echo zag > foo
 mkdir dir2
 echo hi > dir2/foo2
 darcs record -a -m add_foo -A x --look-for-adds
-check_empty_pending
+not darcs whatsnew
 cd ../temp2
 darcs init
 darcs pull -a ../temp1
