@@ -20,7 +20,7 @@ runtest () {
 }
 
 num_lines () {
-  test "$(cat $2 | wc -l)" = "$1"
+  test $(wc -l < $2) = $1
 }
 
 move1='move \./old1 \./new1'
