@@ -32,11 +32,11 @@
 
 # Repo name with ':' is either scp repo or http repo.
 # Let's check scp repo first.
-( darcs clone user@invalid:path || true ) > log 2>&1
-[ -n "$(grep 'ssh: Could not resolve hostname invalid' log)" ]
+( darcs clone user@ssh.bogus.domain.so.it.will.surely.fail.example.org.:path || true ) >log 2>&1
+fgrep 'ssh: Could not resolve host' log
 
 # HTTP repo
-( http_proxy= darcs clone http://www.bogus.domain.so.it.will.surely.fail.com || true ) 2>&1 | tee log
+( http_proxy= darcs clone http://www.bogus.domain.so.it.will.surely.fail.example.org. || true ) >log 2>&1
 egrep 'CouldNotResolveHost|host lookup failure|Name or service not known' log
 
 # local repos are tested by tests/issue1932-colon-breaks-add.sh
