@@ -38,6 +38,7 @@ mkdir ${REMOTE_DIR}; \
 "
 
 # Set up a repo to test
+rm -rf R
 darcs init --repo R
 cd R
 touch f g
@@ -46,7 +47,8 @@ darcs record f g -a --ignore-times -m 'add some files' -A moi
 darcs clone . $REMOTE:$REMOTE_DIR/R
 cd ..
 
+rm -rf S
 darcs clone $REMOTE:$REMOTE_DIR/R S --debug > log 2>&1
-COUNT=$(grep -c '^Exec.*darcs.*transfer-mode' log)
+COUNT=$(grep -c '^Exec: "ssh" .* "darcs" "transfer-mode"' log)
 # with issue2090, this was 6!
 test $COUNT -eq 1
