@@ -59,14 +59,14 @@ cd ..
 
 rm -rf repo repo2
 unpack_testdata simple-v1
-echo 'I understand the consequences of my action' | darcs convert darcs-2 repo repo2 
+echo 'yes' | darcs convert darcs-2 repo repo2 
 cd repo2
 darcs show patch-index | grep 'Patch Index is not yet created.' # convert
 cd ..
 
 rm -rf repo repo2
 unpack_testdata simple-v1
-echo 'I understand the consequences of my action' | darcs convert darcs-2 repo repo2 --with-patch-index
+echo 'yes' | darcs convert darcs-2 repo repo2 --with-patch-index
 cd repo2
 darcs show patch-index | grep 'Patch Index is in sync with repo.' # convert --patch-index
 
