@@ -9,7 +9,7 @@
 # time of writing was the version in use in the last released
 # version of darcs.
 
-tar -xz < $TESTDATA/patch-index-v2.tgz
+unpack_testdata patch-index-v2
 
 cd pi
 
