@@ -54,7 +54,8 @@ darcs rec -am "4"
 echo yyy | darcs rebase apply -a ../R2/2.dpatch
 
 darcs changes --count 2>&1 | grep "Rebase in progress: 2 suspended patches"
-echo yny | darcs rebase unsuspend | grep "We have conflicts"
+echo yny | darcs rebase unsuspend 2>log
+grep conflicts log
 cat > expected <<EOF
 v v v v v v v
 1
@@ -69,7 +70,8 @@ diff -u expected foo
 echo 23 > foo
 echo yyy | darcs amend --patch '3'
 
-echo yy | darcs rebase unsuspend | grep "We have conflicts"
+echo yy | darcs rebase unsuspend 2>log
+grep conflicts log
 cat > expected <<EOF
 v v v v v v v
 3
