@@ -32,9 +32,9 @@ cd t1
 darcs init
 echo 'wibble' > wibble
 darcs add wibble
-darcs rec -am"wibble" --ignore-times
+darcs rec -am"wibble"
 echo 'wobble' > wibble
-darcs rec -am"wobble" --ignore-times
+darcs rec -am"wobble"
 
 echo 'yd' | darcs rebase suspend
 darcs mv wibble wobble
