@@ -54,10 +54,9 @@ cat > expected <<EOF
   * add wobble
 
 Making no changes: this is a dry run.
-Rebase in progress: 1 suspended patches
 EOF
 darcs unpull --dry-run | tail -n+5 | grep -v tester | diff -u expected -
 
 
-echo 'yy' | darcs rebase unsuspend | grep "Rebase finished"
+echo 'yy' | darcs rebase unsuspend 2>&1 | grep "Rebase finished"
 test -f wibble
