@@ -32,7 +32,7 @@ cd t1
 darcs init
 echo 'wibble' > wibble
 darcs add wibble
-darcs rec -am"wibble" --ignore-times
+darcs rec -am"wibble"
 echo 'yy' | darcs rebase suspend
 darcs tag 'ugh'
 darcs check
