@@ -18,11 +18,11 @@ darcs record -a -m auth
 
 echo > _darcs/prefs/defaults ALL disable # try to disable all
 not darcs changes
-not darcs changes --repodir "$PWD"
+not darcs changes --repodir "`pwd`"
 
 cd ..
 
 not darcs changes --repodir repo
-not darcs changes --repodir "$PWD/repo"
+not darcs changes --repodir "`pwd`/repo"
 
 rm -rf temp
