@@ -39,8 +39,7 @@ unpack_testdata simple-v1
 cd repo
 darcs send --no-minimize -o repo.dpatch -a ../empty
 
-day=$(grep "Date: " $TESTDATA/simple-v1.dpatch | head -n 1 | cut -f1-3 -d' ')
-diff -u -I'1 patch for repository ' -I'patches for repository ' -I"$day" $TESTDATA/simple-v1.dpatch repo.dpatch
+compare_bundles $TESTDATA/simple-v1.dpatch repo.dpatch
 cd ..
 
 # context-v1 tests that we are including some context lines in hunk patches
@@ -48,6 +47,6 @@ rm -rf repo
 unpack_testdata context-v1
 cd repo
 darcs send --no-minimize -o repo.dpatch -a ../empty
-day=$(grep "Date: " $TESTDATA/context-v1.dpatch | head -n 1 | cut -f1-3 -d' ')
-diff -u -I'1 patch for repository ' -I'patches for repository ' -I"$day" $TESTDATA/context-v1.dpatch repo.dpatch
+
+compare_bundles $TESTDATA/context-v1.dpatch repo.dpatch
 cd ..
