@@ -1,9 +1,6 @@
 #!/usr/bin/env bash
 . ./lib
 
-DARCS_EDITOR=echo
-export DARCS_EDITOR
-
 rm -rf temp1 temp2
 mkdir temp1 temp2
 
