@@ -1,21 +1,14 @@
 #!/usr/bin/env bash
-echo
-echo Checking that each command expects each option only once
-echo
+# Check that each command expects each option only once
 
 . ./lib
 
-if echo $OS | grep -i windows; then
-    echo Noone knows how to handle newlines under cygwin, so we skip this test
-    exit 0
-fi
-
 rm -rf temp1
 mkdir temp1
 cd temp1
 
 for i in `darcs --commands | grep -v -- -- | xargs echo`; do
-    echo -n Checking $i... ' '
+    echo -n "Checking $i... "
     # only output actual command options, i.e. lines that contain a --
     darcs $i --help | grep -- "--" | sort > $i
     uniq $i > uni$i
@@ -29,5 +22,3 @@ for i in `darcs --commands | grep -v -- -- | xargs echo`; do
 done
 
 cd ..
-rm -rf temp1
-
